---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  B64_TO_TASH_UTILS.ADB 
--  Description : Converts Base-64 encoded GIFs to TASH files
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with ada.text_io;
with Tcl_Helpers;

-- Generates Package_Name & ".ads" and Package_Name & ".adb"
-- which have the single procedure Generate_Image.  Calling this
-- will load the picture into memory.  Note the picture is hardcoded
-- into the .adb file so that the B64 file is not needed at runtime.

procedure b64_to_tash_utils(Filename : in String;
   Package_Name : in String) is

   -- Generate a spec that looks like the following:
   -- -- Auto generated by ...
   -- -- By: Martin C. Carlisle
   --
   -- with Tcl;
   -- package Package_Name is
   --    procedure Generate_Image(Interp : in Tcl.Tcl_Interp);
   -- end Package_Name;
   --
   -- into the file Package_Name & ".ads"
   procedure Generate_Spec(Package_Name : in String) is
      Output_File : Ada.Text_IO.File_Type;
   begin
      Ada.Text_IO.Create(File => Output_File,
         Name => Package_Name & ".ads",
         Mode => Ada.Text_IO.Out_File);
      Ada.Text_IO.Put_Line(Output_File,
         "-- Auto generated by b64_to_tash/tash_from_gif");
      Ada.Text_IO.Put_Line(Output_File,
         "-- By: Martin C. Carlisle");
      Ada.Text_IO.Put_Line(Output_File,
         "with Tcl;");
      Ada.Text_IO.Put_Line(Output_File,
         "package " & Package_Name & " is");
      Ada.Text_IO.Put_Line(Output_File,
         "   procedure Generate_Image(Interp : in Tcl.Tcl_Interp);");
      Ada.Text_IO.Put_Line(Output_File,
         "end " & Package_Name & ";");
      Ada.Text_IO.Close(Output_File);
   end Generate_Spec;

   Line        : String(1..80);
   Last        : Natural;
   Input_File  : Ada.Text_IO.File_Type;
   Output_File : Ada.Text_IO.File_Type;
begin
   -- Open the B64 file and create the .ADB file
   Ada.Text_IO.Open(File => Input_File,
      Name => Filename,
      Mode => Ada.Text_IO.In_File);
   Ada.Text_IO.Create(File => Output_File,
      Name => Package_Name & ".adb",
      Mode => Ada.Text_IO.Out_File);

   Generate_Spec(Package_Name);

   -- the ADB file begins with:
   --
   -- -- Auto generated ...
   -- -- By: Martin C. Carlisle
   -- with Tcl;
   -- with Tcl.Ada;
   -- package body Package_Name is
   --    procedure Generate_Image(Interp : in Tcl.Tcl_Interp) is
   --    begin
   Ada.Text_IO.Put_Line(Output_File,
      "-- Auto generated by b64_to_tash/tash_from_gif");
   Ada.Text_IO.Put_Line(Output_File,
      "-- By: Martin C. Carlisle");
   Ada.Text_IO.Put_Line(Output_File,
      "with Tcl;");
   Ada.Text_IO.Put_Line(Output_File,
      "with Tcl.Ada;");
   Ada.Text_IO.Put_Line(Output_File,
      "package body " & Package_Name & " is");
   Ada.Text_IO.Put_Line(Output_File,
      "   procedure Generate_Image(Interp : in Tcl.Tcl_Interp) is");
   Ada.Text_IO.Put_Line(Output_File,
      "   begin");

   -- We now call Tcl_Helpers to output Tcl code
   -- We start with set image_data "
   Tcl_Helpers.Begin_Block(Output_File);
   Tcl_Helpers.Put(Output_File,"set image_data ");
   Tcl_Helpers.Put_Quotation(Output_File);
   Tcl_Helpers.New_Line(Output_File);
   
   -- Read each line of the B64 file in and 
   -- output code with it
   while not Ada.Text_IO.End_Of_File(Input_File) loop
      Ada.Text_IO.Get_Line(Input_File,Line,Last);
      -- don't worry about blank lines
      if Last >= Line'First then
         -- _=_ seems to mark a comment-- don't print these
         if Last < 3 or else Line(1..3) /= "_=_" then
            Tcl_Helpers.Put_Line(Output_File,Line(1..Last));
         end if;
      end if;
   end loop;
   
   -- output ending quote for set image_data
   Tcl_Helpers.Put_Quotation(Output_File);
   Tcl_Helpers.New_Line(Output_File);
   
   -- use image create photo command on the image data to read
   -- it into memory
   Tcl_Helpers.Put_Line(Output_File,
      "image create photo " & Package_Name &
      " -data $image_data");
   Tcl_Helpers.End_Block(Output_File);

   -- output procedure and package ending
   Ada.Text_IO.Put_Line(Output_File,"   end Generate_Image;");
   Ada.Text_IO.Put_Line(Output_File,"end " & Package_Name & ";");

   Ada.Text_IO.Close(Output_File);
   Ada.Text_IO.Close(Input_File);
end b64_to_tash_utils;