---------------------------------------------------------------
--                                                           
--  CALLED_FROM_TCL 
--  Description : Use to auto generate either TASH code
--  to bind Ada procedures to Tcl ones.
--                       
--  By: Martin Carlisle
--                                         
-- CALLED_FROM_TCL is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- CALLED_FROM_TCL is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- Called from Tcl allows you to bind Ada code to Tcl procedure 
-- names.  It expects an input file of the form:
--
-- TclProcedureName FullyQualifiedAdaProcedureName #ofArguments
-- 
-- For example:
-- delete_subwindow_action  File_Menu.Close_Choice            0
-- 
-- means that the Tcl procedure delete_subwindow_action should
-- call the Ada procedure Close_Choice in package File_Menu
-- with 0 arguments
--
-- Usage: called_from_tcl input_file ada_package_name
-- 
-- Your code should call ada_package_name.generate_bindings
-- to perform the actual binding at runtime
---------------------------------------------------------------
with Ada.Text_Io,Ada.Integer_Text_Io;
with Ada.Command_Line;
with Tcl_Helpers;
with File_Helpers;

procedure Called_From_Tcl is
   procedure Usage is
   begin
      Ada.Text_IO.Put_Line("usage: " & Ada.Command_Line.Command_Name &
         " input_file Ada_package_name");
   end Usage;

   -- Generate a spec that looks like the following:
   -- -- Auto generated by ...
   -- -- By: Martin C. Carlisle
   --
   -- with Tcl;
   -- package Package_Name is
   --    procedure Generate_Bindings(Interp : in Tcl.Tcl_Interp);
   -- end Package_Name;
   --
   -- into the file Package_Name & ".ads"
   procedure Generate_Spec(Package_Name : in String) is
      Output_File : Ada.Text_IO.File_Type;
   begin
      Ada.Text_IO.Create(File => Output_File,
         Name => Package_Name & ".ads",
         Mode => Ada.Text_IO.Out_File);
      Ada.Text_IO.Put_Line(Output_File,
         "-- Auto generated by called_from_tcl");
      Ada.Text_IO.Put_Line(Output_File,
         "-- By: Martin C. Carlisle");
      Ada.Text_IO.Put_Line(Output_File,
         "with Tcl;");
      Ada.Text_IO.Put_Line(Output_File,
         "package " & Package_Name & " is");
      Ada.Text_IO.Put_Line(Output_File,
         "   procedure Generate_Bindings(Interp : in Tcl.Tcl_Interp);");
      Ada.Text_IO.Put_Line(Output_File,
         "end " & Package_Name & ";");
      Ada.Text_IO.Close(Output_File);
   end Generate_Spec;

   Action       : String(1..80);
   Command      : String(1..80);
   Command_Last : Natural;
   Action_Last  : Natural;
   Number_Arguments : Natural;
   Input_File   : Ada.Text_IO.File_Type;
   Output_File  : Ada.Text_IO.File_Type;
begin
   -- requires 2 arguments.  If not, print usage message and exit.
   if Ada.Command_Line.Argument_Count /= 2 then
      Usage;
      return;
   end if;

   -- open input file and create ADB file
   Ada.Text_IO.Open(File => Input_File,
      Name => Ada.Command_Line.Argument(1),
      Mode => Ada.Text_IO.In_File);
   Ada.Text_IO.Create(File => Output_File,
      Name => Ada.Command_Line.Argument(2) & ".adb",
      Mode => Ada.Text_IO.Out_File);

   Generate_Spec(Ada.Command_Line.Argument(2));

   -- the ADB file begins with:
   --
   -- -- Auto generated ...
   -- -- By: Martin C. Carlisle
   -- with Tcl;
   -- with Cargv,Cargv_Helpers;
   -- with Interfaces.C;
   -- with Tcl.Ada;
   -- package body Package_Name is
   --    procedure Generate_Bindings(Interp : in Tcl.Tcl_Interp) is
   --    begin
   Ada.Text_IO.Put_Line(Output_File,
      "-- Auto generated by called_from_tcl");
   Ada.Text_IO.Put_Line(Output_File,
      "-- By: Martin C. Carlisle");
   Ada.Text_IO.Put_Line(Output_File,
      "with Tcl;");
   Ada.Text_IO.Put_Line(Output_File,
      "with Cargv,Cargv_Helpers;");
   Ada.Text_IO.Put_Line(Output_File,
      "with Interfaces.C;");
   Ada.Text_IO.Put_Line(Output_File,
      "with Tcl.Ada;");

   -- generate with clauses here
   -- each Ada name was fully qualified, so we need to generate
   -- additional with clauses for each of the Ada procedures
   -- that will be called (Tcl_Helpers.Generate_With)
   while not Ada.Text_IO.End_Of_File(Input_File) loop
      File_Helpers.Get_String(Input_File,Command,Command_Last);
      File_Helpers.Get_String(Input_File,Action,Action_Last);
      Ada.Integer_Text_IO.Get(Input_File,Number_Arguments);
      Tcl_Helpers.Generate_With(Output_File,
         Action(Action'First..Action_Last));
      Ada.Text_IO.Skip_Line(Input_File);
   end loop;

   Ada.Text_IO.Put_Line(Output_File,
      "package body " & Ada.Command_Line.Argument(2) & " is");
   Ada.Text_IO.Put_Line(Output_File,
      "   package CreateCommands is new Tcl.Ada.Generic_Command (Integer);");

   -- generate actions here
   -- Tcl_Helpers.Generate_Command does most of the work
   -- these procedures call the Ada procedure and convert the
   -- arguments using the Cargv_Helpers package
   Ada.Text_IO.Reset(Input_File);
   while not Ada.Text_IO.End_Of_File(Input_File) loop
      File_Helpers.Get_String(Input_File,Command,Command_Last);
      File_Helpers.Get_String(Input_File,Action,Action_Last);
      Ada.Integer_Text_IO.Get(Input_File,Number_Arguments);
      Tcl_Helpers.Generate_Command(File => Output_File,
         Command_Name => Command(Command'First..Command_Last),
         Action       => Action(Action'First..Action_Last),
         Number_Arguments => Number_Arguments);
      Ada.Text_IO.Skip_Line(Input_File);
   end loop;

   Ada.Text_IO.Put_Line(Output_File,
      "   procedure Generate_Bindings(Interp : in Tcl.Tcl_Interp) is");
   Ada.Text_IO.Put_Line(Output_File,
      "      Command : Tcl.Tcl_Command;");
   Ada.Text_IO.Put_Line(Output_File,
      "   begin");

   -- generate bindings here
   -- This generates Tcl code that binds each Tcl name to the
   -- Ada procedures generated above
   -- Tcl_Helpers.Generate_Command_Binding does most of the work.
   Ada.Text_IO.Reset(Input_File);
   while not Ada.Text_IO.End_Of_File(Input_File) loop
      File_Helpers.Get_String(Input_File,Command,Command_Last);
      File_Helpers.Get_String(Input_File,Action,Action_Last);
      Ada.Integer_Text_IO.Get(Input_File,Number_Arguments);
      Tcl_Helpers.Generate_Command_Binding(Output_File,
         Command(Command'First..Command_Last));
      Ada.Text_IO.Skip_Line(Input_File);
   end loop;

   Ada.Text_IO.Put_Line(Output_File,"   end Generate_Bindings;");
   Ada.Text_IO.Put_Line(Output_File,"end " & Ada.Command_Line.Argument(2) & ";");

   Ada.Text_IO.Close(Output_File);
   Ada.Text_IO.Close(Input_File);
exception
   when Ada.Text_IO.Name_Error =>
      Ada.Text_IO.Put_Line("Invalid filename specified");
end Called_From_Tcl;