---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  EDIT_MENU.ADB
--  Description : Implements choices from the Edit menu
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Gui;
use type Gui.String_Pointer;
with Gui.Widget,Gui.Window;
use type Gui.Widget.Widget_Access,Gui.Window.Window_Pointer;
with State;
with Main;
with Subwindow_Actions;
with Change_Window_Dialog;
with Tcl_Utilities;
with Tcl_Helpers;
with Ada.Characters.Handling;
with Tcl.Ada;
--with Common_Dialogs; -- for debugging

package body Edit_Menu is
   -------------------------------------------------------------
   -- procedure Change_Window
   --
   -- used to change window's name, width or height
   --
   -- Algorithm
   -- 1) Find current window
   -- 2) Generate change dialog
   --    dialog (.change_window_dialog) has 3 entries 
   --      entry1: name
   --      entry2: width
   --      entry3: height
   --      entry4: title (added 2/22/99 using suggestion by Dmitri 
   --                     Anisimkov)
   -- 3) Make change dialog transient (changes appearance) 
   -- 4) Main window is stored as ".", but display as "main"
   --    Set text in entry1, 2 and 3
   -- 5) Callbacks implement changes, so we are done!
   -------------------------------------------------------------
   procedure Change_Window is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      Change_Window_Dialog.Generate_Window(Main.Interp);
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient .change_window_dialog .");
      Tcl.Ada.Tcl_Eval(Main.Interp,"focus .change_window_dialog.entry1");
      if Window.Window_Name.all = "." then
         Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry1",
            Text        => "main");
      else
         Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry1",
            Text        =>
               Window.Window_Name.all(
                 Window.Window_Name.all'First+1..
                 Window.Window_Name.all'Last));
      end if;
      Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
         Window_Name => ".change_window_dialog",
         Name        => "entry2",
         Text        => Integer'Image(Window.Width));
      Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
         Window_Name => ".change_window_dialog",
         Name        => "entry3",
         Text        => Integer'Image(Window.Height));
      if Window.Title/=null then
         Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry4",
            Text        => Tcl_Utilities.Unescape(Window.Title.all));
      end if;
      
   end Change_Window;

   -------------------------------------------------------------
   -- function Check_Changes
   --
   -- verify that entries have legal values
   -- highlight illegal entry (if it is negative)
   -------------------------------------------------------------
   function Check_Changes(Window : in Gui.Window.Window_Pointer)
         return Boolean is
   begin
      if Window.Width < 0 then
         Tcl_Utilities.Highlight_Text_Entry(Interp => Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry2");
         return false;
      elsif Window.Height < 0 then
         Tcl_Utilities.Highlight_Text_Entry(Interp => Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry3");
         return false;
      else
         return true;
      end if;
   end Check_Changes;

   -------------------------------------------------------------
   -- procedure Set_Changes
   --
   -- put values from entries in Window record
   --
   -- Algorithm
   -- 1) Read entry1 into Word, change "main" to "." (name)
   -- 2) Put Word into Window.Name, replacing "." with "_"
   --    and making sure first letter is in lowercase
   --    (Tcl doesn't like . in its names, or begin UC)
   -- 3) Read entry2 directly into Window.Width
   -- 4) Read entry3 directly into Window.Height
   -- 5) Read entry4 into Word, then into Window.Title (added 2/22/99)
   -------------------------------------------------------------
   procedure Set_Changes(Window : in Gui.Window.Window_Pointer) is
      Word : Gui.Word_Type;
      Last : Integer;
   begin
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => ".change_window_dialog",
         Name        => "entry1",
         Result      => Word,
         Last        => Last);

      if Last = Word'First+3 and then
         Ada.Characters.Handling.To_Lower(
            Word(Word'First..Last)) = "main" then
         Word(Word'First) := '.';
         Last := Word'First;
      end if;

      if Last > Word'First then
         Window.Window_Name := new String'("." &
            Tcl_Helpers.Undot_Name(Word(Word'First..Last)));
         -- name can't begin with upper case
         Window.Window_Name(Window.Window_Name'First+1) :=
            Ada.Characters.Handling.To_Lower(
            Window.Window_Name(Window.Window_Name'First+1));
      else
         Window.Window_Name := new String'(".");
      end if;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry2",
            Result      => Window.Width);
      exception
         when others => Window.Width := -99;
      end;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry3",
            Result      => Window.Height);
      exception
         when others => Window.Height := -99;
      end;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => ".change_window_dialog",
            Name        => "entry4",
            Result      => Word,
            Last        => Last);
         if Last > Word'First then
            Window.Title := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
         else
            Window.Title := null;
         end if;
      exception
         when others => Window.Title := null;
      end;
      
   end Set_Changes;

   -------------------------------------------------------------
   -- procedure Change_Done
   --
   -- Change is Done (hit ok)-- update display
   --
   -- Algorithm
   -- 1) Apply changes -- copy of below b/c we don't
   --    wan't to destroy window if changes not ok
   -- 2) Destroy dialog only if changes ok
   -------------------------------------------------------------
   procedure Change_Done is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      Set_Changes(Window);
      if Check_Changes(Window) then
         Gui.Window.Undisplay_Window(Main.Interp,Window.all);
         Gui.Window.Display_Window(Main.Interp,Window.all);
         Tcl_Utilities.Destroy_Window(Main.Interp,".change_window_dialog");
      end if;
      State.Set_Changed(True);
   end Change_Done;

   -------------------------------------------------------------
   -- procedure Apply_Changes
   --
   -- Update window with changes so far
   --
   -- Algorithm
   -- 1) Set changes into record
   -- 2) If OK, undisplay and redisplay
   -- 3) Note the window has changed
   -------------------------------------------------------------
   procedure Apply_Changes is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      Set_Changes(Window);
      if Check_Changes(Window) then
         Gui.Window.Undisplay_Window(Main.Interp,Window.all);
         Gui.Window.Display_Window(Main.Interp,Window.all);
      end if;
      State.Set_Changed(True);
   end Apply_Changes;

   -------------------------------------------------------------
   -- procedure Cancel_Changes
   --
   -- Destroy dialog without doing anything
   -------------------------------------------------------------
   procedure Cancel_Changes is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      if Check_Changes(Window) then
         Tcl_Utilities.Destroy_Window(Main.Interp,".change_window_dialog");
      end if;
   end Cancel_Changes;

   -------------------------------------------------------------
   -- procedure Duplicate_Choice
   --
   -- duplicate a widget
   --
   -- 1) Can't do this if dialog is running, so just exit (ignore)
   --    Same if there is no window or selection
   -- 2) Create a new widget of the same class
   -- 3) Call Subwindow_Actions.Duplicate to complete
   -- 4) Note window has changed
   -------------------------------------------------------------
   procedure Duplicate_Choice is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Selection : Gui.Widget.Widget_Access := State.Get_Selection;
      New_Guy   : Gui.Widget.Widget_Access;
   begin -- Duplicate_Choice
      -- don't do if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      if Window /= null and then Selection /= null then
         New_Guy := new Gui.Widget.GUI_Widget'Class'(Selection.all);
         Subwindow_Actions.Duplicate(New_Guy);
         State.Set_Changed(True);
      end if;
   end Duplicate_Choice;

   -------------------------------------------------------------
   -- procedure Delete_Choice
   --
   -- duplicate a widget
   --
   -- 1) Can't do this if dialog is running, so just exit (ignore)
   --    Same if there is no window or selection
   -- 2) Find selected widget in list (using name)
   -- 3) Undisplay widget
   -- 4) Remove highlighting (if widget was highlighted)
   -- 5) Delete from list
   -- 6) Note window has changed
   -------------------------------------------------------------
   procedure Delete_Choice is
      Window    : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Selection : Gui.Widget.Widget_Access := State.Get_Selection;
      Position  : Gui.Widget.Widget_Pointer;
   begin -- Delete_Choice
      -- don't do if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      if Window /= null and then Selection /= null then
         Position := Gui.Widget.Widget_Key_List_Package.Find(
            Key => Selection.Name.all,
            Ptr => Window.Widget_List);
         Gui.Widget.Undisplay_Widget(
            Widget      => Selection.all,
            Interp      => Main.Interp,
            Window_Name => Gui.Window.Make_Widget_Area_Name(
               Window.Window_Name.all));
         Gui.Widget.Unhighlight(
            Widget      => Selection.all,
            Interp      => Main.Interp,
            Window_Name => Gui.Window.Make_Widget_Area_Name(
               Window.Window_Name.all));
         Gui.Widget.Widget_List_Package.Delete(
            L => Window.Widget_List, P => Position);
         State.Set_Changed(True);
      end if;

      State.Set_Selection(NULL);
   end Delete_Choice;

end Edit_Menu;