---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  FILE_MENU.ADB
--  Description : Implements choices from the File menu
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with State;
with Gui.Window;
use type Gui.Window.Window_Pointer;
with Common_Dialogs;
with Tcl, Tcl.Ada;
with Main;
with Limits;
with new_dialog;
with Tcl_Utilities;
with Ada.Characters.Handling;

package body File_Menu is
   New_Running : Boolean := False;
   -- Is a new dialog running already?


   -- used in Tcl File Open/Save dialogs to select
   -- which files are displayed
   File_Types : constant String :=
      "{ ""Gui Files"" { .gui } } " &
      "{ ""All Files"" * }";

   -- Holds file name and Directory name returned from
   -- File dialogs
   Filename       : String(1..Limits.Max_Name_Length);
   Filename_Last  : Natural;
   Directory      : String(1..Limits.Max_Name_Length);
   Directory_Last : Natural;

   -----------------------------------------------------------
   -- procedure New_Choice
   -- 
   -- called when user pushes OK in New Dialog (.new_dialog)
   --
   -- 1) Read entry1 (name)
   -- 2) Change first letter to lower case (for Tcl)
   -- 3) Replace other spaces by underscores (for Tcl)
   -- 4) Replace main by "." (for Tcl)
   -- 5) Read width and height
   -- 6) Allocate window, setting current Filename to "Untitled"
   --    (see Save for use of this) and Directory to current (.)
   -- 7) Create and display window
   -- 8) Note that window is unchanged
   -----------------------------------------------------------
   procedure New_Done is
      Window_Name  : String(1..80);
      Window_Last  : Natural;
      Title        : String(1..80);
      Title_Last   : Natural;
      Width,Height : Integer;
   begin
      Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry1",
         Result      => Window_Name,
         Last        => Window_Last);
      -- can't have first letter be upper case.
      if Window_Last > Window_Name'First then
         Window_Name(Window_Name'First) :=
            Ada.Characters.Handling.To_Lower(
               Window_Name(Window_Name'First));
      end if;
      -- eliminate trailing spaces
      while Window_Last > Window_Name'First and then
         Window_Name(Window_Last) = ' ' loop
         Window_Last := Window_Last -1;
      end loop;

      -- get rid of other spaces
      for i in Window_Name'First..Window_Last loop
         if Window_Name(i) = ' ' then
            Window_name(i) := '_';
         end if;
      end loop;

      if Window_Last = Window_Name'First+3 and then
         Ada.Characters.Handling.To_Lower(
            Window_Name(Window_Name'First..Window_Last)) = "main" then
         Window_Name(Window_Name'First) := '.';
         Window_Last := Window_Name'First;
      end if;

      Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry2",
         Result      => Width);

      Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry3",
         Result      => Height);

      Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry4",
         Result      => Title,
         Last        => Title_Last);

      Tcl.Ada.Tcl_Eval(Main.Interp, "destroy .new_dialog");

      State.Set_Current_Window(State.Allocate_Window);
      State.Set_Filename("Untitled");
      State.Set_Directory(".");

      Gui.Window.Create_Window(Window => State.Get_Current_Window.all,
         Width       => Width,
         Height      => Height,
         Filename    => "Untitled",
         Title       => Title(Title'First..Title_Last),
         Window_Name => Window_Name(Window_Name'First..Window_Last));

      Gui.Window.Display_Window(Interp => Main.Interp,
         Window => State.Get_Current_Window.all);
      State.Set_Changed(False);
      New_Running := False;
   end New_Done;

   -----------------------------------------------------------
   -- procedure Cancel_New
   --
   -- Set New_Running to false
   -----------------------------------------------------------
   procedure Cancel_New is
   begin
      Tcl.Ada.Tcl_Eval(Main.Interp,"destroy .new_dialog");
      New_Running := False;
   end Cancel_New;

   -----------------------------------------------------------
   -- procedure New_Choice
   --
   -- 0) if New already running, then return (set New_Running)
   -- 1) Close old file (remove if MDI implemented)
   -- 2) Generate new dialog
   -- 2b) Set window close to Cancel_New (see tcl_bindings.txt)
   -- 3) Set default size to 300x300 in dialog
   -- 4) Set focus on name (entry1)
   -- 5) Exit as rest is done in New_Done
   -----------------------------------------------------------
   procedure New_Choice is
   begin -- New_Choice
      if New_Running then
         return;
      end if;
      New_Running := True;

      Close_Choice;  -- remove for MDI

      New_Dialog.Generate_Window(Main.Interp);
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol .new_dialog" &
         " WM_DELETE_WINDOW " &
         "cancel_new_action");

      Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry2",
         Text        => "300");

      Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
         Window_Name => ".new_dialog",
         Name        => "entry3",
         Text        => "300");

      Tcl.Ada.Tcl_Eval(Main.Interp,"focus .new_dialog.entry1");
   end New_Choice;

   -----------------------------------------------------------
   -- procedure Open_Choice
   -- 
   -- 1) Display dialog
   -- 2) If not cancelled, run close choice (remove for MDI)
   --    this gives opportunity to save previous
   -- 3) Allocate window, setting filename and directory
   --    to result from dialog
   -- 4) Call Read_Window from Gui_Window to read file
   -- 5) Display the Window
   -- 6) Note window is unchanged
   -----------------------------------------------------------
   procedure Open_Choice is

   begin -- Open_Choice
      Common_Dialogs.Open_Dialog(Interp => Main.Interp,
         File_Types => File_Types,
         Filename   => Filename,
         File_Last  => Filename_Last,
         Directory  => Directory,
         Dir_Last   => Directory_Last);
      if Filename_Last > 0 then
         -- remove for MDI
         Close_Choice;

         State.Set_Current_Window(State.Allocate_Window);
         State.Set_Filename(Filename(1..Filename_Last));
         State.Set_Directory(Directory(1..Directory_Last));
         Gui.Window.Read_Window(Window => State.Get_Current_Window.all,
            Filename => State.Get_Filename);
         Gui.Window.Display_Window(Interp => Main.Interp,
            Window => State.Get_Current_Window.all);
         State.Set_Changed(False);
      end if;
   exception
      when others =>
         Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid GUI file?", Main.Interp);
   end Open_Choice;


   -----------------------------------------------------------
   -- procedure Close_Choice
   -- 
   -- 1) If a window is not open, ignore
   -- 2) If window has changed, ask if they want to save
   --    2b) If so, run Save_Choice
   -- 3) Undisplay window, set current to null (change for MDI)
   -----------------------------------------------------------
   procedure Close_Choice is
      Window : Gui.Window.Window_Pointer;
   begin -- Close_Choice
      Window := State.Get_Current_Window;

      if Window /= null then
         if State.Get_Changed then
            if Common_Dialogs.Yesno_Dialog(Interp => Main.Interp,
               Title   => "Save work?",
               Message => "Do you want to save changes to " &
                  State.Get_Filename & "?") then
               Save_Choice;
            end if;
         end if;
         Gui.Window.Undisplay_Window(Interp => Main.Interp,
            Window => State.Get_Current_Window.all);
      end if;
      State.Set_Current_Window(null);
   end Close_Choice;


   -----------------------------------------------------------
   -- procedure Save_Choice
   -- 
   -- 1) If no current window, then ignore
   -- 2) If window name is "Untitled" (see New_Choice)
   --    then go to SaveAs_Choice 
   -- 3) Otherwise, ask Window to write itself
   -- 4) Mark window as unchanged
   -----------------------------------------------------------
   procedure Save_Choice is

   begin -- Save_Choice
      if State.Get_Current_Window /= null then
         if State.Get_Filename = "Untitled" then
            SaveAs_Choice;
         else
            Gui.Window.Write_Window(Window => State.Get_Current_Window.all);
            State.Set_Changed(False);
         end if;
      end if;
   end Save_Choice;


   -----------------------------------------------------------
   -- procedure SaveAs_Choice
   -- 
   -- 1) if no Window, ignore
   -- 2) Put up Save dialog
   -- 3) if we get a filename (not pressed cancel) then
   --    3b) Change window name
   --    3c) Set new filename and directory
   --    3d) call Save to perform write
   -----------------------------------------------------------
   procedure SaveAs_Choice is

   begin -- SaveAs_Choice
      if State.Get_Current_Window /= null then
         Common_Dialogs.Save_Dialog(Interp => Main.Interp,
            File_Types => File_Types,
            Filename   => Filename,
            File_Last  => Filename_Last,
            Directory  => Directory,
            Dir_Last   => Directory_Last);
         if Filename_Last > Filename'First then
            State.Set_Filename(Filename(1..Filename_Last));
            State.Set_Directory(Directory(1..Directory_Last));
            Gui.Window.Change_Filename(Interp => Main.Interp,
               Window   => State.Get_Current_Window.all,
               Filename => State.Get_Filename);
            Save_Choice;
         end if;
      end if;
   end SaveAs_Choice;


   -----------------------------------------------------------
   -- procedure Exit_Choice
   -- 
   -- 1) Close file (to allow possible save)
   -- 2) Quit (do not ask)
   -----------------------------------------------------------
   procedure Exit_Choice is

   begin -- Exit_Choice
      Close_Choice;
      Common_Dialogs.Quit_Dialog(Interp => Main.Interp,
         Verify => False);
   end Exit_Choice;

end File_Menu;