---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-LISTBOX.ADB
--  Description : GUI Widget Listbox
--                       
--  By: Jonathan Busch
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Gui_Enum;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Tcl.Ada;
with Common_Dialogs;
with Main;
with Ada.Strings.Unbounded;
with Ada.Exceptions;
with Ada.Text_Io;

package body Gui.Widget.Listbox is
   package Boolean_Io is new Ada.Text_Io.Enumeration_Io(Boolean);

   type S_Bar is (X,Y);

   -- reads information from file into Listbox,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Listbox;
         File   : in Ada.Text_Io.File_Type) is

      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(Gui_Widget(Widget),File);

      -- Get listbox colors
      File_Helpers.Get_String(File,Word,Last);
      Widget.Fg_Color := new String'(Word(1..Last));

      File_Helpers.Get_String(File,Word,Last);
      Widget.Bg_Color := new String'(Word(1..Last));

      -- Get Select Color
      File_Helpers.Get_String(File,Word,Last);
      Widget.Select_Color := new String'(Word(1..Last));

      -- Check for horizontal scrollbar
      Boolean_Io.Get(File,Widget.Hz_Scroll);

      -- Check for vertical scrollbar
      Boolean_Io.Get(File,Widget.Vr_Scroll);

      -- Get Scrollbar Color
      File_Helpers.Get_String(File,Word,Last);
      Widget.Scrollbar_Color := new String'(Word(1..Last));

   end Read_Widget;

   procedure Get_Widget_Params(Widget : in out Listbox;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.FG_Color := new String'(Get_Widget_Param(Params,"fg"));
      Widget.BG_Color := new String'(Get_Widget_Param(Params,"bg"));
      Widget.Select_Color := new String'(Get_Widget_Param(Params,"sel_color"));
      Widget.Hz_Scroll := Get_Widget_Param(Params,"hz_scroll");
      Widget.Vr_Scroll := Get_Widget_Param(Params,"vr_scroll");
      Widget.Scrollbar_Color := new String'(Get_Widget_Param(Params,"sb_color"));
   end Get_Widget_Params;   

   -- Writes information to file from Listbox
   procedure Write_Widget(Widget : in Listbox;
         File   : in Ada.Text_Io.File_Type) is

   begin -- Write_Widget
      Gui_Enum.Io.Put(File,Gui_Enum.Listbox);
      Write_Widget(Gui_Widget(Widget),File);
      Put_Widget_Param(File,"fg", Widget.Fg_Color.all);
      Put_Widget_Param(File,"bg", Widget.Bg_Color.all);
      Put_Widget_Param(File,"sel_color", Widget.Select_Color.all);

      Put_Widget_Param(File,"hz_scroll", Widget.Hz_Scroll);
      Put_Widget_Param(File,"vr_scroll", Widget.Vr_Scroll);

      Put_Widget_Param(File,"sb_color", Widget.Scrollbar_Color.all);
      Ada.Text_IO.New_Line(File);
   end Write_Widget;


   --Returns string containing scrollbar info
   function Scrollbar_Setup(Frame_Name : in String;
         Widget     : in String;
         Color      : in String;
         Which      : in S_Bar) return String is

   begin --Scrollbar_Setup

      case Which is
         when X =>
            return ("scrollbar " & Frame_Name & ".hscrl" &
               " -orient horizontal -command """ & Frame_Name & '.' &
               Widget & " xview""" & Tcl_Helpers.Colors(Fg => "default", Bg => Color));

         when Y =>
            return ("scrollbar " & Frame_Name & ".vscrl" &
               " -command """ & Frame_Name & '.' & Widget & " yview""" &
               Tcl_Helpers.Colors(Fg => "default", Bg => Color));
      end case;

   end Scrollbar_Setup;

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget      : in Listbox;
         Window_Name : in String;
         File        : in Ada.Text_Io.File_Type;
         Tcl_Only    : in Boolean) is

   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);

      --create frame for listbox
      Tcl_Helpers.Put(File,
         "frame " & Window_Name & Widget.Name.all, Tcl_Only);
      --End of Line for frame
      Tcl_Helpers.New_Line(File,Tcl_Only);

      --Create beginning of listbox
      Tcl_Helpers.Put(File,
         "listbox " & Window_Name & Widget.Name.all & '.' & Widget.Name.all &
         " -width " & Integer'Image(Widget.Width) &
         " -height " & Integer'Image(Widget.Height), Tcl_Only);

      --FG_Color and BG_Color
      Tcl_Helpers.Put(File,Tcl_Helpers.Colors(Fg => Widget.Fg_Color.all,
            Bg => Widget.Bg_Color.all),Tcl_Only);

      --Select_Color
      if (Widget.Select_Color.all /= "default") then
         Tcl_Helpers.Put(File," -selectbackground ",Tcl_Only);
         Tcl_Helpers.Put(File,Widget.Select_Color.all,Tcl_Only);
      end if;

      --Create Listbox with no scrollbars
      if (not Widget.Hz_Scroll) and (not Widget.Vr_Scroll) then

         --End of Line for Listbox
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Pack options for listbox
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & '.' & Widget.Name.all, Tcl_Only);

         --Create Listbox with vertical scrollbar
      elsif (Widget.Vr_Scroll) and not Widget.Hz_Scroll then
         Tcl_Helpers.Put(File, " -yscrollcommand ",Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Put(File, Window_Name & Widget.Name.all &
            ".vscrl set",Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         --End of Line for Listbox
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Scrollbar Setup
         Tcl_Helpers.Put(File, Scrollbar_Setup(
               Frame_Name => (Window_Name & Widget.Name.all),
               Widget     => Widget.Name.all,
               Color      => Widget.Scrollbar_Color.all,
               Which      => Y), Tcl_Only);
         --End of Line for Scrollbar
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Pack options for vertical scrollbar
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & ".vscrl -side right -fill y"
            ,Tcl_Only);
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Pack options for listbox
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & '.' & Widget.Name.all &
            " -side left",Tcl_Only);

         --Create Listbox with horizontal scrollbar
      elsif (Widget.Hz_Scroll) and not Widget.Vr_Scroll then
         Tcl_Helpers.Put(File, " -xscrollcommand ", Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Put(File, Window_Name & Widget.Name.all &
            ".hscrl set",Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         --End of Line for Listbox
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Scrollbar Options
         Tcl_Helpers.Put(File, Scrollbar_Setup(
               Frame_Name => Window_Name & Widget.Name.all,
               Widget     => Widget.Name.all,
               Color      => Widget.Scrollbar_Color.all,
               Which      => X), Tcl_Only);
         --End of Line for Scrollbar
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Pack options for horizontal scrollbar
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & ".hscrl -side bottom -fill x"
            ,Tcl_Only);
         Tcl_Helpers.New_Line(File,Tcl_Only);
         --Pack options for listbox
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & '.' & Widget.Name.all &
            " -side left",Tcl_Only);

         --Create Listbox with both scrollbars
      else
         Tcl_Helpers.Put(File, " -yscrollcommand ", Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Put(File, Window_Name & Widget.Name.all &
            ".vscrl set",Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);

         Tcl_Helpers.Put(File, " -xscrollcommand ", Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Put(File, Window_Name & Widget.Name.all &
            ".hscrl set",Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         --End of Line for Listbox
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Vertical Scrollbar Setup
         Tcl_Helpers.Put(File, Scrollbar_Setup(
               Frame_Name => Window_Name & Widget.Name.all,
               Widget     => Widget.Name.all,
               Color      => Widget.Scrollbar_Color.all,
               Which      => Y), Tcl_Only);
         --End of Line for Vertical Scrollbar
         Tcl_Helpers.New_Line(File,Tcl_Only);

         --Horizontal Scrollbar Setup
         Tcl_Helpers.Put(File, Scrollbar_Setup(
               Frame_Name => Window_Name & Widget.Name.all,
               Widget     => Widget.Name.all,
               Color      => Widget.Scrollbar_Color.all,
               Which      => X), Tcl_Only);
         --End of Line for Horizontal Scrollbar
         Tcl_Helpers.New_Line(File,Tcl_Only);


         --Pack options for vertical scrollbar
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & ".vscrl -side right -fill y"
            ,Tcl_Only);
         Tcl_Helpers.New_Line(File,Tcl_Only);
         --Pack options for horizontal scrollbar
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & ".hscrl -side bottom -fill x"
            ,Tcl_Only);
         Tcl_Helpers.New_Line(File,Tcl_Only);
         --Pack options for listbox
         Tcl_Helpers.Put(File,
            "pack " &
            Window_Name & Widget.Name.all & '.' & Widget.Name.all &
            " -side left",Tcl_Only);

      end if;

      --End of Line for particular Frame group
      Tcl_Helpers.New_Line(File,Tcl_Only);

      --Place Widget in window
      Generate_Widget(Gui_Widget(Widget),Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget      : in Listbox;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is

      Tdisp : Ada.Strings.Unbounded.Unbounded_String;
      Is_Excepted : Boolean := False;
   begin

      --create frame for listbox
      Tcl.Ada.Tcl_Eval(Interp,
         "frame " & Window_Name & '.' & Widget.Name.all);

      --Create beginning of listbox
      Tdisp := Ada.Strings.Unbounded.To_Unbounded_String(
         "listbox " & Window_Name & '.' & Widget.Name.all & '.' & Widget.Name.all &
         " -width " & Integer'Image(Widget.Width) &
         " -height " & Integer'Image(Widget.Height));
      begin
         --FG_Color and BG_Color
         Ada.Strings.Unbounded.Append(
            Source   => Tdisp,
            New_Item => Tcl_Helpers.Colors(
               Fg => Widget.Fg_Color.all,
               Bg => Widget.Bg_Color.all));

         --Select_Color
         if (Widget.Select_Color.all /= "default") then
            Ada.Strings.Unbounded.Append(Source => Tdisp,
               New_Item => " -selectbackground " & Widget.Select_Color.all);
         end if;

         --Create Listbox with no scrollbars
         if (not Widget.Hz_Scroll) and (not Widget.Vr_Scroll) then

            --End of Line for Listbox, evaluate
            Tcl.Ada.Tcl_Eval(Interp,Ada.Strings.Unbounded.To_String(Tdisp));

            --Pack options for listbox
            Tcl.Ada.Tcl_Eval(Interp, "pack " &
               Window_Name & '.' & Widget.Name.all & '.' &
               Widget.Name.all);

            --Create Listbox with vertical scrollbar
         elsif (Widget.Vr_Scroll) and not Widget.Hz_Scroll then
            Ada.Strings.Unbounded.Append(Source => Tdisp,
               New_Item => " -yscrollcommand """ & Window_Name &
               '.' & Widget.Name.all & ".vscrl set""");
            --End of Line for Listbox, evaluate
            Tcl.Ada.Tcl_Eval(Interp,Ada.Strings.Unbounded.To_String(Tdisp));

            --Scrollbar Setup
            Tcl.Ada.Tcl_Eval(Interp,
               Scrollbar_Setup(Frame_Name => Window_Name & '.' & Widget.Name.all,
                  Widget     => Widget.Name.all,
                  Color      => Widget.Scrollbar_Color.all,
                  Which      => Y));

            --Pack options for vertical scrollbar
            Tcl.Ada.Tcl_Eval(Interp, "pack " &
               Window_Name & '.' & Widget.Name.all & ".vscrl -side right -fill y");

            --Pack options for listbox
            Tcl.Ada.Tcl_Eval(Interp, "pack " & Window_Name & '.' &
               Widget.Name.all & '.' & Widget.Name.all & " -side left");

            --Create Listbox with horizontal scrollbar
         elsif (Widget.Hz_Scroll) and not Widget.Vr_Scroll then
            Ada.Strings.Unbounded.Append(Source => Tdisp,
               New_Item => " -xscrollcommand """ & Window_Name &
               '.' & Widget.Name.all & ".hscrl set""");
            --End of Line for Listbox, evaluate
            Tcl.Ada.Tcl_Eval(Interp,Ada.Strings.Unbounded.To_String(Tdisp));

            --Scrollbar Options
            Tcl.Ada.Tcl_Eval(Interp,
               Scrollbar_Setup(Frame_Name => Window_Name & '.' & Widget.Name.all,
                  Widget     => Widget.Name.all,
                  Color      => Widget.Scrollbar_Color.all,
                  Which      => X));

            --Pack options for horizontal scrollbar
            Tcl.Ada.Tcl_Eval(Interp, "pack " &
               Window_Name & '.' & Widget.Name.all & ".hscrl -side bottom -fill x");

            --Pack options for listbox
            Tcl.Ada.Tcl_Eval(Interp, "pack " & Window_Name & '.' &
               Widget.Name.all & '.' & Widget.Name.all & " -side left");

            --Create Listbox with both scrollbars
         else
            Ada.Strings.Unbounded.Append(Source => Tdisp,
               New_Item => " -yscrollcommand """ & Window_Name &
               '.' & Widget.Name.all & ".vscrl set""");
            Ada.Strings.Unbounded.Append(Source => Tdisp,
               New_Item => " -xscrollcommand """ & Window_Name &
               '.' & Widget.Name.all & ".hscrl set""");
            --End of Line for Listbox, evaluate
            Tcl.Ada.Tcl_Eval(Interp,Ada.Strings.Unbounded.To_String(Tdisp));


            --Vertical Scrollbar Setup
            Tcl.Ada.Tcl_Eval(Interp,
               Scrollbar_Setup(Frame_Name => Window_Name & '.' & Widget.Name.all,
                  Widget     => Widget.Name.all,
                  Color      => Widget.Scrollbar_Color.all,
                  Which      => Y));

            --Horizontal Scrollbar Setup
            Tcl.Ada.Tcl_Eval(Interp,
               Scrollbar_Setup(Frame_Name => Window_Name & '.' & Widget.Name.all,
                  Widget     => Widget.Name.all,
                  Color      => Widget.Scrollbar_Color.all,
                  Which      => X));

            --Pack options for vertical scrollbar
            Tcl.Ada.Tcl_Eval(Interp, "pack " &
               Window_Name & '.' & Widget.Name.all & ".vscrl -side right -fill y");

            --Pack options for horizontal scrollbar
            Tcl.Ada.Tcl_Eval(Interp, "pack " &
               Window_Name & '.' & Widget.Name.all & ".hscrl -side bottom -fill x");

            --Pack options for listbox
            Tcl.Ada.Tcl_Eval(Interp, "pack " & Window_Name & '.' &
               Widget.Name.all & '.' & Widget.Name.all & " -side left");
         end if;
      exception when others =>
         --End of Line for Listbox, evaluate
         Tcl.Ada.Tcl_Eval(Interp,
            "listbox " & Window_Name & '.' & Widget.Name.all & '.' &
            Widget.Name.all &
            " -width " & Integer'Image(Widget.Width) &
            " -height " & Integer'Image(Widget.Height));
         Tcl.Ada.Tcl_Eval(Interp, "pack " &
            Window_Name & '.' & Widget.Name.all & '.' &
            Widget.Name.all);
         Is_Excepted := True;
      end;
      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);

      if Is_Excepted then
         Tcl_Utilities.Prepend_Listbox_Text(Interp,Window_Name,
            Widget.Name.all,"Error");
      end if;

      -- Because most of the widget area is in a subwidget, it
      -- is very hard to select, so we repeat this code that is
      -- done in the parent method on the subwidget
      -- MUST DO AFTER THE CALL TO PARENT METHOD so the widget
      -- has already been placed
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " <Button-1> {set startx %x; set starty %y; " &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " configure -cursor fleur; select_widget " &
         Window_Name & "." & Widget.Name.all &
         "}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " <Button-3> {modify_widget " &
         Window_Name & "." & Widget.Name.all &
         "}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " <ButtonRelease-1> {" &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " configure -cursor {}}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all & '.' & Widget.Name.all &
         " <Button1-Motion> {move_widget " &
         Window_Name & "." & Widget.Name.all &
         " $startx $starty %x %y}");
   exception when e:others =>
         Common_Dialogs.Ok_Box("Can't display: " &
            Widget.Name.all & ascii.lf &
            ada.exceptions.exception_information(e), Interp);
   end Display_Widget;


--   procedure Set_Properties(Widget : in out Listbox) is
--   begin
--      Listbox_Dialog.Generate_Window(Main.Interp);
--      Set_Properties(Gui_Widget(Widget));

--      --setup FG_color input
--      if (Widget.Fg_Color /= null) then
--         if (Widget.Fg_Color.all /= "default") then
--            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
--               Window_Name => Properties_Dialog_Name(Widget),
--               Name        => "entry6",
--               Text        => Tcl_Utilities.Unescape(Widget.Fg_Color.all));
--         end if;
--      end if;

--      --setup BG_color input
--      if (Widget.Bg_Color /= null) then
--         if (Widget.Bg_Color.all /= "default") then
--            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
--               Window_Name => Properties_Dialog_Name(Widget),
--               Name        => "entry7",
--               Text        => Tcl_Utilities.Unescape(Widget.Bg_Color.all));
--         end if;
--      end if;

--      --setup scrollbar color input
--      if (Widget.Scrollbar_Color /= null) then
--         if (Widget.Scrollbar_Color.all /= "default") then
--            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
--               Window_Name => Properties_Dialog_Name(Widget),
--               Name        => "entry8",
--               Text        => Tcl_Utilities.Unescape(Widget.Scrollbar_Color.all));
--         end if;
--      end if;

--      --setup select color input
--      if (Widget.Select_Color /= null) then
--         if (Widget.Select_Color.all /= "default") then
--            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
--               Window_Name => Properties_Dialog_Name(Widget),
--               Name        => "entry9",
--               Text        => Tcl_Utilities.Unescape(Widget.Select_Color.all));
--         end if;
--      end if;

--      --vertical scrollbar?
--      if (Widget.Vr_Scroll = True) then
--         Tcl_Utilities.Select_Box(Interp      => Main.Interp,
--            Window_Name => Properties_Dialog_Name(Widget),
--            Name        => "vertical");
--      end if;

--      --horizontal scrollbar?
--      if (Widget.Hz_Scroll = True) then
--         Tcl_Utilities.Select_Box(Interp      => Main.Interp,
--            Window_Name => Properties_Dialog_Name(Widget),
--            Name        => "horizontal");
--      end if;

--   end Set_Properties;

      procedure Set_Property_Entries(Widget : in out Listbox;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),pos);

--      Auto_Position.Start_New_Column;
      Auto_Position.Add_Message(pos,"ListBox properties","LightBlue");

      --setup FG_color input
      Auto_Position.Add_Entry(pos,
            Label   => "Foreground Color",
            Name        => "entry6",
            Value        => Tcl_Utilities.Unescape(unaccess(Widget.Fg_Color)));

      --setup BG_color input
      Auto_Position.Add_Entry(pos,
            Label   => "Background Color",
            Name        => "entry7",
            Value   => Tcl_Utilities.Unescape(unaccess(Widget.Bg_Color)));

      --setup scrollbar color input
      Auto_Position.Add_Entry(pos,
            Label   => "Scrollbar Color",
            Name        => "entry8",
            Value   => Tcl_Utilities.Unescape(Unaccess(Widget.Scrollbar_Color)));

      --setup select color input
      Auto_Position.Add_Entry(pos,
            Label   => "Select Color",
            Name        => "entry9",
            Value   => Tcl_Utilities.Unescape(unaccess(Widget.Select_Color)));

      --vertical scrollbar?
      Auto_Position.Add_Check_Button(pos,
            text   => "Vertical Scrollbar",
            Name        => "vertical",
            Value   => Widget.Vr_Scroll);

      --horizontal scrollbar?
      Auto_Position.Add_Check_Button(pos,
            text   => "Horizontal Scrollbar",
            Name        => "horizontal",
            Value   => Widget.Hz_Scroll);

   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Listbox) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));

      --The FG Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Fg_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Fg_Color := new String'("default");
      end if;

      --The BG Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry7",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Bg_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Bg_Color := new String'("default");
      end if;

      --Scrollbar Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry8",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Scrollbar_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Scrollbar_Color := new String'("default");
      end if;

      --Listbox select Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry9",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Select_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Select_Color := new String'("default");
      end if;

      --Vertical scrollbar?
      Widget.Vr_Scroll := Tcl_Utilities.Is_Checked(
         Main.Interp, "vertical");

      --Horizontal scrollbar?
      Widget.Hz_Scroll := Tcl_Utilities.Is_Checked(
         Main.Interp, "horizontal");

   end Apply_Properties;


   procedure Check_Properties(Widget : in out Listbox;
         Ok     : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Gui_Widget(Widget),Ok);
      if Ok then

         --For the foreground color
         Ok := Widget.Fg_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(
               Main.Interp,
               Window_Name,"entry6");
         end if;
      end if;

      if Ok then
         --For the background color
         Ok := Widget.Bg_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(
               Main.Interp,
               Window_Name,"entry7");
         end if;
      end if;

      if Ok then
         --For the scrollbar color
         Ok := Widget.Scrollbar_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(
               Main.Interp,
               Window_Name,"entry8");
         end if;
      end if;

      if Ok then
         --For the select color
         Ok := Widget.Select_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(
               Main.Interp,
               Window_Name,"entry9");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;

   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Listbox) return String is
   begin
      return ".listbox_dialog";
   end Properties_Dialog_Name;

end Gui.Widget.Listbox;