---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-PICTURE.ADB
--  Description : GUI Widget Picture Picture
--                       
--  By: Martin Carlisle 
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with Gui_Enum;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Tcl.Ada;
with Common_Dialogs;
with Auto_Position;
with Main;
with Gif_To_Tash;

package body Gui.Widget.Picture is
   -- reads information from file into Picture,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Picture;
         File : in Ada.Text_Io.File_Type) is

      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(GUI_Widget(Widget),File);

      File_Helpers.Get_String(File,Word,Last);
      Widget.Image := new String'(Word(1..Last));
   end Read_Widget;

   procedure Get_Widget_Params(Widget : in out Picture;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Image := new String'(Get_Widget_Param(Params,"image"));
      Widget.Bound_Relief := Relief'Value(Get_Widget_Param(Params,"relief","flat"));
   end;

   function To_String(e : Relief) return string renames
      Relief_Choise.To_String;

   -- Writes information to file from Picture
   procedure Write_Widget(Widget : in Picture;
         File : in Ada.Text_Io.File_Type) is

   begin -- Write_Widget
      Gui_Enum.IO.Put(File,Gui_Enum.Picture);
      Write_Widget(GUI_Widget(Widget),File);
      Put_Widget_Param(File,"image", Widget.Image.all);
      Put_Widget_Param(File,"relief", To_String(Widget.Bound_Relief));
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Picture;
         Window_Name : in String;
         File : in Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      if Tcl_Utilities.Is_Newer(Main.Interp,Widget.Image.all & ".gif",
            Widget.Image.all & ".adb") then
         begin
            Gif_To_Tash.Convert(Widget.Image.all);
         exception
            when others =>
               Common_Dialogs.OK_Box("Unable to compile: " & ASCII.LF &
                  Widget.Image.all & ".gif",Main.Interp);
         end;
      end if;

      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         "label " &
         Window_Name & Widget.Name.all &
         " -image ",Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         Widget.Image.all,Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.New_Line(File,Tcl_Only);

      --Relief
      Tcl_Helpers.Put(File," -relief " & 
         To_String(Widget.Bound_Relief),Tcl_Only);
     
      Generate_Widget(GUI_Widget(Widget),Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget : in Picture;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String) is
      Excepted : Boolean := False;
   begin
      begin
         Tcl.Ada.Tcl_Eval(Interp,
            "image create photo " &
            Widget.Image.all &
            " -file " &
            Widget.Image.all & ".gif");
      exception
         when others =>
            if Widget.Image /= null then
               Common_Dialogs.OK_Box("Unable to find picture: " &
                  Widget.Image.all, Interp);
            end if;
            Tcl.Ada.Tcl_Eval(Interp,
               "label " &
               Window_Name & "." & Widget.Name.all);
            Excepted := True;
      end;

      if not Excepted then
         Tcl.Ada.Tcl_Eval(Interp,
            "label " &
            Window_Name & "." & Widget.Name.all &
            " -image """ &
            Widget.Image.all & '"' &
            " -relief " & To_String(Widget.Bound_Relief));
      end if;

      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
      Common_Dialogs.OK_Box("Can't display: " &
         Widget.Name.all, Interp);
   end Display_Widget;


   procedure Set_Property_Entries(Widget : in out Picture;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),pos);
      Auto_Position.Add_Message(pos,"Picture properties","LightBlue");
      Auto_Position.Add_Entry(pos,
            Label => "Picture",
            Name        => "entry6",
            Value        => Tcl_Utilities.Unescape(Unaccess(Widget.Image)));

      --setup Relief input
      Relief_Choise.Add_Entry(pos,
         Label => "Relief",
         Name  => "relief",
         Value => Widget.Bound_Relief);

   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Picture) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Image := new String'(Tcl_Utilities.Fix_Quotes(
            Word(Word'First..Last)));
      end if;
      
      Widget.Bound_Relief := 
         Relief_Choise.Get_Enum_Entry(Main.Interp,Window_Name,"relief");

   end Apply_Properties;


   procedure Check_Properties(Widget : in out Picture;
      Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Gui_Widget(Widget),Ok);
      if Ok then
         Ok := Widget.Image /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry6");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;

   procedure Generate_Context_Clause(Widget : in Picture;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_With(File,Widget.Image.all & ".");
   end Generate_Context_Clause;

   procedure Generate_Action_Binding(Widget : in Picture;
      File : in Ada.Text_IO.File_Type) is
   begin
      Ada.Text_IO.Put_Line(File,
         "      " & Widget.Image.all & "." &
         "Generate_Image(Interp);");
   end Generate_Action_Binding;

   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Picture) return String is
   begin
      return ".picture_dialog";
   end Properties_Dialog_Name;

end Gui.Widget.Picture;