---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-PROGRESS.ADB
--  Description : GUI Widget Progress
--                       
--  By: Dmitri Anisimkov
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Gui_Enum;
with Tcl_Helpers;
with Tcl.Ada;
with Common_Dialogs;
with Main;
with Ada.Exceptions;
with Ada.Text_Io;

package body Gui.Widget.Progress is

   -- get information from file into Progress,
   package Progress_Value_IO is new Fixed_Param_IO(Progress_Value,0.0);

   procedure Get_Widget_Params(Widget : in out Progress;
         Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Entire_Color := new String'(Get_Widget_Param(Params,"entire_color"));
      Widget.Part_Color := new String'(Get_Widget_Param(Params,"part_color"));
      Widget.Value := Progress_Value_IO.Get_Widget_Param(Params,"value");
      Widget.Entire_Relief := Relief'Value(Get_Widget_Param
         (Params,"entire_relief","Sunken"));
      Widget.Part_Relief := Relief'Value(Get_Widget_Param
         (Params,"part_relief","Raised"));
   end Get_Widget_Params;

   function To_String(e : Relief) return string renames
      Relief_Choise.To_String;

   -- Writes information to file from Progress
   procedure Write_Widget(Widget : in Progress;
         File   : in Ada.Text_Io.File_Type) is
   begin -- Write_Widget
      Gui_Enum.Io.Put(File,Gui_Enum.Progress);
      Write_Widget(Gui_Widget(Widget),File);
      Put_Widget_Param(File,"entire_color", Widget.Entire_Color.all);
      Put_Widget_Param(File,"part_color", Widget.Part_Color.all);
      Progress_Value_IO.Put_Widget_Param(File,"value", Widget.Value);
      Put_Widget_Param(File,"entire_relief",To_String(Widget.entire_relief));
      Put_Widget_Param(File,"part_relief",To_String(Widget.part_relief));
      Ada.Text_Io.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget      : in Progress;
         Window_Name : in String;
         File        : in Ada.Text_Io.File_Type;
         Tcl_Only    : in Boolean) is
         Name : String := Window_Name&Widget.Name.all;
   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      --create frame for progress
      Tcl_Helpers.Put_Line(File,"frame "&Name&
         " -relief "&To_String(Widget.entire_relief)&" -borderwidth 2"&
         Tcl_Helpers.Colors(Bg => Widget.Entire_Color.all,Fg => "default"),
         Tcl_Only);
      Tcl_Helpers.Put_Line(File,
         "frame "&Name&".part -relief "&To_String(Widget.Part_relief)&" -borderwidth 2"&
         Tcl_Helpers.Colors(Bg => Widget.Part_Color.all,Fg => "default"),
         Tcl_Only);
      Tcl_Helpers.Put_Line(File,
         "place "&Name&".part -relheight 1 -relwidth "&
         Progress_Value_IO.To_String(Widget.Value),
         Tcl_Only);
      Generate_Widget(Gui_Widget(Widget),Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget      : in Progress;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
         Name : String := Window_Name&'.'&Widget.Name.all;
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "frame "&Name&" -relief "&To_String(Widget.entire_relief)&" -borderwidth 2"&
         Tcl_Helpers.Colors(Bg => Widget.Entire_Color.all,Fg => "default")&
         Ascii.Lf&
         "frame "&Name&".part -relief "&To_String(Widget.Part_relief)&" -borderwidth 2"&
         Tcl_Helpers.Colors(Bg => Widget.Part_Color.all,Fg => "default")&
         Ascii.Lf&
         "place "&Name&".part -relheight 1 -relwidth "&
         Progress_Value_IO.To_String(Widget.Value)&
         Ascii.Lf);
      -- Because most of the widget area is in a subwidget, it
      -- is very hard to select, so we repeat this code that is
      -- done in the parent method on the subwidget
      -- MUST DO AFTER THE CALL TO PARENT METHOD so the widget
      -- has already been placed
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Name & ".part <Button-1> {set startx %x; set starty %y; " &
         Name & ".part configure -cursor fleur; select_widget " &Name&
         "}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &Name & ".part <Button-3> {modify_widget " &Name&
         "}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Name & ".part <ButtonRelease-1> {" &
         Name & ".part configure -cursor {}}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Name & ".part <Button1-Motion> {move_widget " &Name&
         " $startx $starty %x %y}");
      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when E:others =>
         Common_Dialogs.Ok_Box("Can't display: " &
            Widget.Name.all & Ascii.Lf &
            Ada.Exceptions.Exception_Information(E), Interp);
   end Display_Widget;

   procedure Set_Property_Entries(Widget : in out Progress;
         Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),Pos);

      Auto_Position.Add_Message(Pos,"Progress properties","LightBlue");

      --setup Entire_color input
      Auto_Position.Add_Entry(Pos,
         Label   => "Entire Color",
         Name        => "entire_color",
         Value        => Tcl_Utilities.Unescape(Unaccess(Widget.Entire_Color)));

      --setup Part_color input
      Auto_Position.Add_Entry(Pos,
         Label   => "Part Color",
         Name        => "part_color",
         Value   => Tcl_Utilities.Unescape(Unaccess(Widget.Part_Color)));

      --setup Value input
      Auto_Position.Add_Entry(Pos,
         Label   => "Value",
         Name    => "progress_part",
         Value   =>
            Progress_Value_IO.To_String(Widget.Value));

      Relief_Choise.Add_Entry(pos,
         Label => "Entire relief",
         Name  => "entire_relief",
         Value => Widget.Entire_Relief);

      Relief_Choise.Add_Entry(pos,
         Label => "Part relief",
         Name  => "part_relief",
         Value => Widget.Part_Relief);

   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Progress) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));

      --The Entire Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entire_color",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Entire_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Entire_Color := new String'("default");
      end if;

      --The Part Color
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "part_color",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Part_Color := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      else
         Widget.Part_Color := new String'("default");
      end if;

      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "progress_part",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         begin
            Widget.Value := Progress_Value_IO.To_Numeric(Word(Word'First..Last));
         exception
            when others =>
               Widget.Value := Progress_Value'First;
         end;
      else
         Widget.Value := 0.0;
      end if;

      Widget.Entire_Relief := Relief_Choise.Get_Enum_Entry(
         Main.Interp,Window_Name,"entire_relief");

      Widget.Part_Relief := Relief_Choise.Get_Enum_Entry(
         Main.Interp,Window_Name,"part_relief");

   end Apply_Properties;

   procedure Check_Properties(Widget : in out Progress;
         Ok     : out Boolean) is
   begin
      Check_Properties(Gui_Widget(Widget),Ok);
      if Ok then
         Ok := Widget.Value in 0.0..1.0;
      end if;
      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;

   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Progress) return String is
   begin
      return ".progress_dialog";
   end Properties_Dialog_Name;

end Gui.Widget.Progress;