---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-STATIC.ADB
--  Description : GUI Widget Static Label
--                       
--  By: Martin Carlisle and Jonathan Busch
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Gui_Enum;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Tcl.Ada;
with Common_Dialogs;
with Main;

package body Gui.Widget.Static is

     package Justification_Choise is new
         Auto_Position.Enumeration_Entry(Justification);
   function To_String(e : Justification) return string renames
      Justification_Choise.To_String;
   function To_String(e : Relief) return string renames
      Relief_Choise.To_String;

   function Justify_Code(Justify : in Justification) return String is
   begin
      if Justify = left then
         return "left -anchor w";
      elsif Justify = right then
         return "right -anchor e";
      else
         return "center -anchor center";
      end if;
   end Justify_Code;

   -- reads information from file into Label,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Label;
                         File   : in Ada.Text_IO.File_Type) is

      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(GUI_Widget(Widget),File);

      -- Get subclass of stuff in Widget record added in static ads
      File_Helpers.Get_String(File,Word,Last);
      Widget.Text := new String'(Word(1..Last));

      --Check for justification & colors 
      --  (if statement for RAPID Ver. 1.0 gui files compatibility)
      File_Helpers.Get_String(File,Word,Last);
      if (Last /= Word'First-1) then
         Widget.Justify := Justification'Value(Word(1..Last));

         -- Get Colors
	 File_Helpers.Get_String(File,Word,Last);
         Widget.FG_Color := new String'(Word(1..Last));
         File_Helpers.Get_String(File,Word,Last);
         Widget.BG_Color := new String'(Word(1..Last));
      else
         Widget.Justify  := left;
         Widget.FG_Color := new String'("default");
         Widget.BG_Color := new String'("default");
      end if;
   end Read_Widget;

   procedure Get_Widget_Params(Widget : in out Label;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Text := new String'(Get_Widget_Param(Params,"text"));
      Widget.Justify := Justification'Value(Get_Widget_Param(Params,"justify"));
      Widget.Bound_Relief := Relief'Value(Get_Widget_Param(Params,"relief","flat"));
      Widget.Bg_Color := new String'(Get_Widget_Param(Params,"bg"));
      Widget.Fg_Color := new String'(Get_Widget_Param(Params,"fg"));
   end;

   -- Writes information to file from Label
   procedure Write_Widget(Widget : in Label;
                          File   : in Ada.Text_IO.File_Type) is

   begin -- Write_Widget
      Gui_Enum.IO.Put(File,Gui_Enum.Label);
      Write_Widget(GUI_Widget(Widget),File);
      Put_Widget_Param(File,"text", Widget.Text.all);
      Put_Widget_Param(File,"bg", Widget.bg_color.all);
      Put_Widget_Param(File,"fg", Widget.fg_color.all);
      Put_Widget_Param(File,"justify", To_String(Widget.Justify));
      Put_Widget_Param(File,"relief", To_String(Widget.Bound_Relief));
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget      : in Label;
                             Window_Name : in String;
                             File        : in Ada.Text_IO.File_Type;
                             Tcl_Only    : in Boolean) is

   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         "message " &
         Window_Name & Widget.Name.all &
         " -width " & Integer'Image(Widget.Width) &
         " -text ",Tcl_Only);

      --Text Entry
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         Widget.Text.all,Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);

      --Justify
      Tcl_Helpers.Put(File," -justify ",Tcl_Only);
      Tcl_Helpers.Put(File,Justify_Code(Widget.Justify),Tcl_Only);

      --Relief
      Tcl_Helpers.Put(File," -relief " & 
         To_String(Widget.Bound_Relief),Tcl_Only);
         
      --FG_Color
      Tcl_Helpers.Put(File,
                      Tcl_Helpers.Colors(FG => Widget.FG_Color.all,
                                         BG => Widget.BG_Color.all),
                      Tcl_Only);

      --End of Line for particular Label
      Tcl_Helpers.New_Line(File,Tcl_Only);

      --Place Widget in window
      Generate_Widget(GUI_Widget(Widget),Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget      : in Label;
                            Interp      : in Tcl.Tcl_Interp;
                            Window_Name : in String) is
   begin
      begin
         Tcl.Ada.Tcl_Eval(Interp,
            "message " &
            Window_Name & "." & Widget.Name.all &
            " -width " & Integer'Image(Widget.Width) &
            " -text """ & Widget.Text.all & '"' &
            " -justify " & Justify_Code(Widget.Justify) &
            " -relief " & To_String(Widget.Bound_Relief) &
            Tcl_Helpers.Colors(FG => Widget.FG_Color.all,
                               BG => Widget.BG_Color.all));

      exception when others =>
         Tcl.Ada.Tcl_Eval(Interp,
            "message " &
            Window_Name & "." & Widget.Name.all &
            " -width " & Integer'Image(Widget.Width) &
            " -text ""Error!"" -bg red");
      end;

      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
      Common_Dialogs.OK_Box("Can't display: " &
         Widget.Name.all, Interp);
   end Display_Widget;

      procedure Set_Property_Entries(Widget : in out Label;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),pos);

      --setup text input
      Auto_Position.Add_Message(Pos,"Label properties","LightBlue");
      Auto_Position.Add_Entry(pos,
	              Label => "Text",
                      Name    => "entry6",
                      Value    => Tcl_Utilities.Unescape(Unaccess(Widget.Text)));

      --setup Justification input
      Justification_Choise.Add_Entry(pos,
         Label => "Justification",
         Name  => "entry7",
         Value => Widget.Justify);

      --setup Relief input
      Relief_Choise.Add_Entry(pos,
         Label => "Relief",
         Name  => "relief",
         Value => Widget.Bound_Relief);

      --setup FG_color input
      Auto_Position.Add_Entry(pos,
	              Label => "Foreground Color",
	              Name   => "entry8",
	              Value   => Tcl_Utilities.Unescape(Unaccess(Widget.FG_Color)));

      --setup BG_color input
      Auto_Position.Add_Entry(pos,
	              Label => "Background Color",
	              Name        => "entry9",
	              Value   => Tcl_Utilities.Unescape(Unaccess(Widget.BG_Color)));
   end Set_Property_Entries;


   procedure Apply_Properties(Widget : in out Label) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));

      --The Text Part
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Text := new String'(Tcl_Utilities.Fix_Quotes(
            Word(Word'First..Last)));
      end if;

      --Justification
--      Tcl_Utilities.Get_Text_Entry(Main.Interp,
--         Window_Name => Window_Name,
--         Name        => "entry7",
--         Result      => Word,
--         Last        => Last);
--      if Last >= Word'First then
--         Widget.Justify := new String'(Tcl_Utilities.Fix_Quotes(
--            Ada.Characters.Handling.To_Lower(Word(Word'First..Last))));
--      else
--         Widget.Justify := new String'("left");
--      end if;
      Widget.Justify := 
         Justification_Choise.Get_Enum_Entry(Main.Interp,Window_Name,"entry7");

      Widget.Bound_Relief := 
         Relief_Choise.Get_Enum_Entry(Main.Interp,Window_Name,"relief");

      --Foreground Color Part
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry8",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.FG_Color := new String'(Tcl_Utilities.Fix_Quotes(
            Word(Word'First..Last)));
      else
         Widget.FG_Color := new String'("default");
      end if;

      --Background Color Part
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry9",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.BG_Color := new String'(Tcl_Utilities.Fix_Quotes(
            Word(Word'First..Last)));
      else
         Widget.BG_Color := new String'("default");
      end if;
   end Apply_Properties;


   procedure Check_Properties(Widget : in out Label;
                              Ok     : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Gui_Widget(Widget),Ok);
      if Ok then

 	 --For the entered text
         Ok := Widget.Text /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
                                               Window_Name,"entry6");
         end if;
      end if;

--      if Ok then
--	 --For Justification
--         Ok := Widget.Justify /= null;
--         if not Ok then
--            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
--                                               Window_Name,"entry7");
--         -- Note it is important that we used To_Lower in
--         -- Apply properties so we don't worry about case here
----         elsif not ((Widget.Justify = center) OR ELSE
----            (Widget.Justify = right) OR ELSE
----            (Widget.Justify = left)) THEN
----            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
----                                               Window_Name,"entry7");
--         end if;
--      end if;

      if Ok then
 	 --For the foreground color
         Ok := Widget.FG_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
                                               Window_Name,"entry8");
         end if;
      end if;

      if Ok then
 	 --For the background color
         Ok := Widget.BG_Color /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
                                               Window_Name,"entry9");
         end if;

      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;

   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Label) return String is
   begin
      return ".label_dialog";
   end Properties_Dialog_Name;

end Gui.Widget.Static;