---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET.ADB
--  Description : Root of GUI Widget Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Add Set_Property_Entries 
-- and change Set_Properties for ability of creation 
-- inheritable propery editors without designer
---------------------------------------------------------------
-- Dmitri Anisimkov 3/11/99 Resize widget with windows functionality.
-- Dmitri Anisimkov 3/11/99 change properties string format 
--                          in GUI files
---------------------------------------------------------------
with Tcl_Helpers,Tcl_Utilities;
with File_Helpers;
with Ada.Integer_Text_Io,Ada.Text_Io;
with Tcl.Ada;
with Main;
with Common_Dialogs;
with Ada.Characters.Handling;
with State;
with Subwindow_Actions;
--with Ada.Strings.Maps.Constants;
--with Ada.Strings.Fixed;

package body Gui.Widget is
   Handle_Size : constant Integer := 6;
   -- size of handles that are created if a widget is selected
   -- better if even

   procedure Get_Widget_Params(Widget : in out Gui_Widget; Params : in String) is
   begin
      Widget.Name := new String'(Get_Widget_Param(Params,"name"));
      Widget.X := Get_Widget_Param(Params,"x");
      Widget.Y := Get_Widget_Param(Params,"y");
      Widget.Width := Get_Widget_Param(Params,"width");
      Widget.Height := Get_Widget_Param(Params,"height");
      Widget.Relx := Relative_Resize'Value(Get_Widget_Param(Params,"relx","none"));
      Widget.Rely := Relative_Resize'Value(Get_Widget_Param(Params,"rely","none"));
   end;

   -----------------------------------------------------
   -- File is already open and widget type read
   -- read name, x, y, width and height
   -- used in child types to read these common
   -- attributes
   -----------------------------------------------------
   procedure Read_Widget(Widget : in out Gui_Widget;
         File        : in Ada.Text_Io.File_Type) is
      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      File_Helpers.Get_String(File,Word,Last);
      Widget.Name := new String'(Word(1..Last));

      Ada.Integer_Text_Io.Get(File,Widget.X);
      Ada.Integer_Text_Io.Get(File,Widget.Y);
      Ada.Integer_Text_Io.Get(File,Widget.Width);
      Ada.Integer_Text_Io.Get(File,Widget.Height);
   end Read_Widget;

   -----------------------------------------------------
   -- write out widget type, name, x, y, width and
   -- height to already created file.
   -- used in child types to write these common
   -- attributes
   -----------------------------------------------------

   procedure Write_Widget(Widget : in Gui_Widget;
         File : in Ada.Text_Io.File_Type) is
   begin -- Write_Widget
      Put_Widget_Param(File,"name",Widget.Name.all);
      Put_Widget_Param(File,"x",Widget.X);
      Put_Widget_Param(File,"y",Widget.Y);
      Put_Widget_Param(File,"width",Widget.Width);
      Put_Widget_Param(File,"height",Widget.Height);
      if Widget.relx/=None then
         Put_Widget_Param(File,"relx",Relative_Resize'Image(Widget.relx));
      end if;
      if Widget.rely/=None then
         Put_Widget_Param(File,"rely",Relative_Resize'Image(Widget.rely));
      end if;
   end Write_Widget;

   function place_coords(Widget : in Gui_Widget) return string is
      window : Gui.Window.Window_Pointer := State.Get_Current_Window;
      relx : integer := boolean'Pos(Widget.relx=coord);
      relw : integer := boolean'Pos(Widget.relx=size);
      rely : integer := boolean'Pos(Widget.rely=coord);
      relh : integer := boolean'Pos(Widget.rely=size);
      x : integer :=  Widget.X - window.width*relx;
      y : integer :=  Widget.y - window.Height*rely;
      w : integer :=  Widget.Width - window.width*relw;
      h : integer :=  Widget.Height- window.Height*relh;
   begin
       return
         " -x " & Integer'Image(X) &
         " -y " & Integer'Image(Y) &
         " -width " & Integer'Image(W) &
         " -height " & Integer'Image(H) &
         " -relx" & Integer'Image(relx) &
         " -rely" & Integer'Image(rely) &
         " -relwidth" & Integer'Image(relw) &
         " -relheight" & Integer'Image(relh);
   end place_coords;

   -----------------------------------------------------
   -- this generates the section of the code
   -- that places the widget at its proper location and
   -- with its proper size.  Called from children
   -----------------------------------------------------
   procedure Generate_Widget(Widget : in Gui_Widget;
         Window_Name : in String;
         File : in Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean) is
   begin
      Tcl_Helpers.Put_Line(File,"place " &
         Window_Name & Widget.Name.all &
         " -anchor nw" &
         place_coords(Widget), Tcl_Only);
   end Generate_Widget;

   -----------------------------------------------------
   -- this is not abstract, but null, and should be overridden for
   -- each widget containing action(s).
   -----------------------------------------------------
   procedure Generate_Action(Widget : in Gui_Widget;
         File : in Ada.Text_Io.File_Type) is

   begin -- Generate_Action
      null;
   end Generate_Action;

   -----------------------------------------------------
   -- this is not abstract, but null, and should be overridden for
   -- each widget containing action(s).
   -----------------------------------------------------
   procedure Generate_Action_Binding(Widget : in Gui_Widget;
         File : in Ada.Text_Io.File_Type) is

   begin -- Generate_Action_Binding
      null;
   end Generate_Action_Binding;

   -----------------------------------------------------
   -- this is not abstract, but null, and should be overridden for
   -- each widget containing action(s).
   -----------------------------------------------------
   procedure Generate_Context_Clause(Widget : in Gui_Widget;
         File : in Ada.Text_Io.File_Type) is
   begin -- Generate_Context_Clause
      null;
   end Generate_Context_Clause;

   -----------------------------------------------------
   -- display the widget to a window
   --
   -- place the widget at its proper location
   -- (will be called after it is created by a
   --  method of the same name in a child type)
   -- Also binds the right mouse button to modify_widget
   --            the left mouse button to select_widget
   --            moving left button to moving widget
   -- Cursor is changed while left mouse is depressed on 
   -- widget
   -----------------------------------------------------
   procedure Display_Widget(Widget : in Gui_Widget;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      -- place the widget
      Tcl.Ada.Tcl_Eval(Interp,
         "place " &
         Window_Name & "." & Widget.Name.all &
         " -anchor nw" &
         place_coords(Widget));

      -- bind right mouse to modifying widget
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all &
         " <Button-3> {modify_widget %W}");

      -- bind left mouse to selecting widget
      -- and change cursor
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all &
         " <Button-1> {set startx %x; set starty %y; " &
         Window_Name & "." & Widget.Name.all &
         " configure -cursor fleur; select_widget %W}");

      -- bind release of left mouse to change
      -- cursor back to normal
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all &
         " <ButtonRelease-1> {" &
         Window_Name & "." & Widget.Name.all &
         " configure -cursor {}}");

      -- bind drag with left mouse to move widget
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " &
         Window_Name & "." & Widget.Name.all &
         " <Button1-Motion> {move_widget %W $startx $starty %x %y}");
   exception
      when others =>
         Common_Dialogs.Ok_Box("Unable to display: " &
            Widget.Name.all & Ascii.Lf &
            "in window: " & Window_Name, Interp);
         Undisplay_Widget(Widget,Interp,Window_Name);
   end Display_Widget;

   -----------------------------------------------------
   -- highlight a selected widget
   --
   -- calls a create_handles (Tcl) procedure created
   -- in the package begin/end block below
   -- then binds dragging on each of the 8 resize handles
   -- (nw,ne,sw,se,nn,ss,ee,ww)
   -----------------------------------------------------
   procedure Highlight(Widget : in Gui_Widget;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "create_handles " & Window_Name & " " &
         Window_Name & "." & Widget.Name.all & " " &
         Integer'Image(Widget.X-Handle_Size) & " " &
         Integer'Image(Widget.X+Widget.Width/2-Handle_Size/2) & " " &
         Integer'Image(Widget.X+Widget.Width) & " " &
         Integer'Image(Widget.Y-Handle_Size) & " " &
         Integer'Image(Widget.Y+Widget.Height/2-Handle_Size/2) & " " &
         Integer'Image(Widget.Y+Widget.Height));
      -- I gave up trying to do this in the procedure
      -- these below are very peculiar, but I want widget_name
      -- evaluated now, and everything else later
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " & Window_Name & ".highlight_nw" &
         " <Button1-Motion> {resize_widget nw " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_ne " &
         " <Button1-Motion> {resize_widget ne " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_sw " &
         " <Button1-Motion> {resize_widget sw " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_se " &
         " <Button1-Motion> {resize_widget se " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_nn " &
         " <Button1-Motion> {resize_widget nn " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_ss " &
         " <Button1-Motion> {resize_widget ss " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_ww " &
         " <Button1-Motion> {resize_widget ww " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Window_Name & ".highlight_ee " &
         " <Button1-Motion> {resize_widget ee " & Window_Name & "." & Widget.Name.all & " $startx $starty %x %y}");
   exception
      when others => null;
   end Highlight;

   -----------------------------------------------------
   -- highlight a selected widget
   --
   -- done by calling Tcl procedure move_handles
   -- defined below in the package begin/end block
   -----------------------------------------------------
   procedure Move_Highlight(Widget : in Gui_Widget;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "move_handles " & Window_Name & ' ' &
         Integer'Image(Widget.X-Handle_Size) & ' ' &
         Integer'Image(Widget.X+Widget.Width/2-Handle_Size/2) & ' ' &
         Integer'Image(Widget.X+Widget.Width) & ' ' &
         Integer'Image(Widget.Y-Handle_Size) & ' ' &
         Integer'Image(Widget.Y+Widget.Height/2-Handle_Size/2) & ' ' &
         Integer'Image(Widget.Y+Widget.Height));

   end Move_Highlight;

   -----------------------------------------------------
   -- calls the destroy_handles (Tcl) procedure
   -- defined below in package begin/end block
   -----------------------------------------------------
   procedure Unhighlight(Widget : in Gui_Widget;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "destroy_handles " & Window_Name);
   end Unhighlight;

   -----------------------------------------------------
   -- Tell Tcl to destroy the widget!
   -----------------------------------------------------
   procedure Undisplay_Widget(Widget : in Gui_Widget;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "destroy " &
         Window_Name & "." & Widget.Name.all);
   exception
      when others => null;
   end Undisplay_Widget;

   -----------------------------------------------------
   -- Algorithm:
   -- 1) Get dialog name
   -- 2) Change dialog appearance to transient
   -- 3) If widget has name (won't if just created), 
   --    set entry1
   -- 4) set entries2-5 to x,y,width,height
   -- 5) Bind clicking window close to 
   --    delete_properties_action (see tcl_bindings.txt)
   --    this calls Subwindow_Actions.Cancel_Properties_Dialog
   -- 6) Put cursor in entry1
   -- Note: other properties set in method of same name
   -- in child class
   -----------------------------------------------------
   procedure Set_Properties(Widget : in out Gui_Widget) is
      Window_Name : String := Properties_Dialog_Name(Gui_Widget'Class(Widget));
      Pos : Auto_Position.Positioner;
   begin
      Auto_Position.Start_New_Sequence(Pos,Main.Interp,Window_Name);
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient "&Window_Name&" .");

      Set_Property_Entries(Gui_Widget'Class(Widget),Pos);
      Auto_Position.Add_Apply_Ok_Cancel(Pos,
         Subwindow_Actions.Apply_Properties_Dialog'access,
         Subwindow_Actions.Done_Properties_Dialog'access,
         Subwindow_Actions.Cancel_Properties_Dialog'access);

      Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol "&Window_Name&
         " WM_DELETE_WINDOW delete_properties_action");

      Tcl.Ada.Tcl_Eval(Main.Interp,"focus " &
         Window_Name &
         ".entry1");

   end Set_Properties;

   function Unaccess(Strp : String_Pointer) return String is
   begin
      if Strp=null then return "";
      else return Strp.all;
      end if;
   end;

   package Relative_Resize_Edit is new
        Auto_Position.Enumeration_Entry(Relative_Resize);

   procedure Set_Property_Entries(Widget : in out Gui_Widget;
         Pos : in out Auto_Position.Positioner) is
   begin
      Auto_Position.Add_Message(Pos,"Widget properties","LightBlue");
      Auto_Position.Add_Entry(Pos,
         Label => "Widget name",
         Name        => "entry1",
         Value      => Unaccess(Widget.Name));
      Auto_Position.Add_Entry(Pos,
         Label   => "X",
         Name    => "entry2",
         Value   => Widget.X);
      Auto_Position.Add_Entry(Pos,
         Label   => "Y",
         Name    => "entry3",
         Value   => Widget.Y);
      Auto_Position.Add_Entry(Pos,
         Label   => "Width",
         Name    => "entry4",
         Value   => Widget.Width);
      Auto_Position.Add_Entry(Pos,
         Label   => "Heigh",
         Name    => "entry5",
         Value   => Widget.Height);

      Relative_Resize_Edit.Add_Entry(pos,
         Label => "Resize X",
         Name  => "relx",
         Value => Widget.relx);

      Relative_Resize_Edit.Add_Entry(pos,
         Label => "Resize Y",
         Name  => "rely",
         Value => Widget.rely);

   end Set_Property_Entries;

   -----------------------------------------------------
   -- Algorithm:
   -- 1) Get dialog name
   -- 2) Read Widget_Name into Word
   -- 3) Read x,y,width,height directly into window
   --    3a) set to -99 if invalid text 
   -- Note: other properties applied in method of same
   -- name in child class
   -- Widget name can't begin upper case-- fix this!
   -----------------------------------------------------
   procedure Apply_Properties(Widget : in out Gui_Widget) is
      Word : Word_Type;
      Last : Integer;
      Window_Name : String := Properties_Dialog_Name(Gui_Widget'Class(Widget));
   begin
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry1",
         Result      => Word,
         Last        => Last);

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry2",
            Result      => Widget.X);
      exception
         when others => Widget.X := -99;
      end;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry3",
            Result      => Widget.Y);
      exception
         when others => Widget.Y := -99;
      end;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry4",
            Result      => Widget.Width);
      exception
         when others => Widget.Width := -99;
      end;

      begin
         Tcl_Utilities.Get_Text_Entry(Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry5",
            Result      => Widget.Height);
      exception
         when others => Widget.Height := -99;
      end;

      if Last >= Word'First then
         Widget.Name := new String'(Word(Word'First..Last));
         -- name can't begin with upper case
         Widget.Name(Widget.Name'First) :=
            Ada.Characters.Handling.To_Lower(
            Widget.Name(Widget.Name'First));
      end if;

      Widget.relx := Relative_Resize_Edit.Get_Enum_Entry(
         Main.Interp,Window_Name,"relx");

      Widget.rely := Relative_Resize_Edit.Get_Enum_Entry(
         Main.Interp,Window_Name,"rely");

   end Apply_Properties;

   -----------------------------------------------------
   -- Check properties common to all widgets.  Others
   -- checked from method of same name in child class
   --
   -- Make sure name isn't null and first character is
   -- lower case letter
   -- Make sure name is alphanumeric and has no "_"
   -- If not OK, highlight entry1
   --
   -- Note we stop checking as soon as we find something
   -- that isn't OK.
   --
   -- Check x,y,width,height to make sure they are positive
   -- and highlight if not
   -----------------------------------------------------
   procedure Check_Properties(Widget : in out Gui_Widget;
         Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(
         Gui_Widget'Class(Widget));
   begin
      Ok := Widget.Name /= null;
      if Ok then
         Ok := Ada.Characters.Handling.Is_Lower(Widget.Name.all(
               Widget.Name.all'First));
      end if;

      if Ok then
         for I in Widget.Name.all'range loop
            if not Ada.Characters.Handling.Is_Alphanumeric(
                  Widget.Name.all(I)) and then
                  not (Widget.Name.all(I) = '_') then
               Ok := False;
            end if;
         end loop;
      end if;

      if not Ok then
         Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
            Window_Name,"entry1");
      end if;

      if Ok and then Widget.X < 0 then
         Ok := False;
         Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
            Window_Name,"entry2");
      end if;

      if Ok and then Widget.Y < 0 then
         Ok := False;
         Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
            Window_Name,"entry3");
      end if;

      if Ok and then Widget.Width < 0 then
         Ok := False;
         Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
            Window_Name,"entry4");
      end if;

      if Ok and then Widget.Height < 0 then
         Ok := False;
         Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
            Window_Name,"entry5");
      end if;
   end Check_Properties;

   -----------------------------------------------------
   -- Destroy the dialog.  Note that we don't have one
   -- running anymore. (Can only have 1 at a time since
   -- the dialog names are reused).
   -----------------------------------------------------
   procedure Close_Properties(Widget : in Gui_Widget) is
      Window_Name : String := Properties_Dialog_Name(
         Gui_Widget'Class(Widget));
   begin
      Tcl.Ada.Tcl_Eval(Main.Interp,"destroy " &
         Window_Name);
      State.Set_Dialog_Running(False);
   end Close_Properties;

   -----------------------------------------------------
   -- return the name
   -----------------------------------------------------
   function Get_Name(Widget : in Widget_Access) return String is
   begin
      return Widget.Name.all;
   end Get_Name;

   -------------------------------------------------
   -- procedure Set_Location
   --
   -- set x,y,width and height for a widget
   -------------------------------------------------
   procedure Set_Location(Widget : in Widget_Access;
         X,Y,Width,Height : in Integer) is
   begin
      Widget.X := X;
      Widget.Y := Y;
      Widget.Width := Width;
      Widget.Height := Height;
   end Set_Location;

   -- creates Tcl procedures to improve speed
begin
   -- Tcl procedure destroy_handles
   -- destroys 8 resizer handles
   Tcl.Ada.Tcl_Eval(Main.Interp,
      "proc destroy_handles {window_name} {" & Ascii.Lf &
      "destroy ${window_name}.highlight_nw" & Ascii.Lf &
      "destroy ${window_name}.highlight_ne" & Ascii.Lf &
      "destroy ${window_name}.highlight_sw" & Ascii.Lf &
      "destroy ${window_name}.highlight_se" & Ascii.Lf &
      "destroy ${window_name}.highlight_ww" & Ascii.Lf &
      "destroy ${window_name}.highlight_ee" & Ascii.Lf &
      "destroy ${window_name}.highlight_nn" & Ascii.Lf &
      "destroy ${window_name}.highlight_ss" & Ascii.Lf &
      "}");

   -- Tcl procedure create_handles (7 arguments)
   -- Arguments: widget_name, west x, middle x, east x,
   -- north y, middle y, south y
   --
   -- create 8 resizer handles with unique cursors
   -- bind left mouse to set variables
   Tcl.Ada.Tcl_Eval(Main.Interp,
      "proc create_handles {window_name widget_name xw xm xe yn ym ys} {" & Ascii.Lf &
      "frame ${window_name}.highlight_nw -background black " &
      " -cursor top_left_corner" & Ascii.Lf &
      "place ${window_name}.highlight_nw -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xw -y $yn" & Ascii.Lf &
      "frame ${window_name}.highlight_ne -background black " &
      " -cursor top_right_corner" & Ascii.Lf &
      "place ${window_name}.highlight_ne -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xe -y $yn" & Ascii.Lf &
      "frame ${window_name}.highlight_sw -background black " &
      " -cursor bottom_left_corner" & Ascii.Lf &
      "place ${window_name}.highlight_sw -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xw -y $ys" & Ascii.Lf &
      "frame ${window_name}.highlight_se -background black " &
      " -cursor bottom_right_corner" & Ascii.Lf &
      "place ${window_name}.highlight_se -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xe -y $ys" & Ascii.Lf &
      "frame ${window_name}.highlight_nn -background black " &
      " -cursor top_side" & Ascii.Lf &
      "place ${window_name}.highlight_nn -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xm -y $yn" & Ascii.Lf &
      "frame ${window_name}.highlight_ss -background black " &
      " -cursor bottom_side" & Ascii.Lf &
      "place ${window_name}.highlight_ss -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xm -y $ys" & Ascii.Lf &
      "frame ${window_name}.highlight_ww -background black " &
      " -cursor left_side" & Ascii.Lf &
      "place ${window_name}.highlight_ww -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xw -y $ym" & Ascii.Lf &
      "frame ${window_name}.highlight_ee -background black " &
      " -cursor right_side" & Ascii.Lf &
      "place ${window_name}.highlight_ee -anchor nw " &
      " -width " & Integer'Image(Handle_Size) &
      " -height " & Integer'Image(Handle_Size) &
      " -x $xe -y $ym" & Ascii.Lf &
      "bind ${window_name}.highlight_nw " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_ne " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_sw " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_se " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_nn " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_ss " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_ww " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "bind ${window_name}.highlight_ee " &
      " <Button-1> {set startx %x; set starty %y}" & Ascii.Lf &
      "}");

   -- Tcl procedure move_handles (7 arguments)
   -- Arguments: widget_name, west x, middle x, east x,
   -- north y, middle y, south y
   --
   -- relocates the resizer handles
   Tcl.Ada.Tcl_Eval(Main.Interp,
      "proc move_handles {window_name xw xm xe yn ym ys} {" & Ascii.Lf &
      "place configure ${window_name}.highlight_nw -x $xw -y $yn" & Ascii.Lf &
      "place configure ${window_name}.highlight_ne -x $xe -y $yn" & Ascii.Lf &
      "place configure ${window_name}.highlight_sw -x $xw -y $ys" & Ascii.Lf &
      "place configure ${window_name}.highlight_se -x $xe -y $ys" & Ascii.Lf &
      "place configure ${window_name}.highlight_nn -x $xm -y $yn" & Ascii.Lf &
      "place configure ${window_name}.highlight_ss -x $xm -y $ys" & Ascii.Lf &
      "place configure ${window_name}.highlight_ee -x $xe -y $ym" & Ascii.Lf &
      "place configure ${window_name}.highlight_ww -x $xw -y $ym" & Ascii.Lf &
      "}");

end Gui.Widget;