---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET.ADS 
--  Description : Root of GUI Widget Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
-- Change log:
-- 08/20/98 (mcc) : incorporate workaround for GNAT 3.10p bug 
--                  workaround contributed by Frederic Abiven
-- 02/14/99 Dmitri Anisimkov : Add Set_Property_Entries 
-- and change Set_Properties for ability of creation 
-- inheritable propery editors without designer
---------------------------------------------------------------
-- This starts the Gui-Widget hierarchy.  Most widgets
-- will be a direct descendant of GUI_Widget
--
-- Most other packages will use Widget_Access, which can
-- point to any widget in the hierarchy.  Also the widget
-- list packages are used in Gui-Window to keep track of
-- all the widgets in a window.
--
-- ATTRIBUTES:
-- All widgets must have a location, size and name (see below)
--
-- METHODS:
-- All widgets know how to:
-- 1) Set their values by reading from a file
-- 2) Write themselves to a file
-- 3) Display and undisplay themselves
-- 4) Generate code for themselves
-- 5) Highlight and unhighlight themselves
-- 6) Move themselves (with highlight)
-- 7) Display a properties dialog and 
--    change their values based on the result
--
-- NOTE:  Almost all of the methods in the child packages
-- are implemented by calling the same method in this package
-- to do the common stuff, then doing the stuff unique to 
-- each widget.  See Properties_Dialog_Name below for
-- more info on naming conventions that make this reuse work!
---------------------------------------------------------------
-- 03/11/99 Dmitri Anisimkov : Resizing widget with windows 
--                             functionality.
-- 03/11/99 Dmitri Anisimkov : change properties string format 
--                             in GUI files
-- 03/20/99 Dmitri Anisimkov : Add Relief type for Relief 
--                             properties in different Widgets 
---------------------------------------------------------------
with Ada.Text_Io;
with Tcl;
with Auto_Position;
with Lists_Generic_Key;

package Gui.Widget is

   type Gui_Widget;
   type Widget_Access is access all Gui_Widget'Class;

   type Relative_Resize is (None,Coord,Size);
   type Relief is (Raised, Sunken, Flat, Ridge, Solid, Groove);

   -- Note this is abstract, so GUI_Widget can't be
   -- declared directly, it is only holding
   -- values shared by all its children
   type Gui_Widget is abstract tagged record
      X,Y,Width,Height : Integer;
      relx,rely : Relative_Resize := Relative_Resize'First;
      Name        : String_Pointer := null;
   end record;

   package Relief_Choise is new
       Auto_Position.Enumeration_Entry(Relief);

   -------------------------------------------------
   -- procedure Read_Widget
   --
   -- reads information from a (.GUI) file into 
   -- a previously created GUI_Widget,
   -- assumes keyword already read.
   --
   -- For example, if we have:
   --   LABEL label1 0 5 110 20 "Widget name"
   --
   -- then LABEL was read and a LABEL widget created
   -- (this happens in widget_io), then this gets
   -- called to read the rest and fill in the fields
   -- of the LABEL
   -------------------------------------------------
   procedure Read_Widget(Widget : in out Gui_Widget;
      File        : in Ada.Text_Io.File_Type);

   procedure Get_Widget_Params(Widget : in out Gui_Widget; Params : in String);

   -------------------------------------------------
   -- procedure Write_Widget
   --
   --
   -- Writes information from this Widget to a (.GUI) file 
   -- This includes the entire line including the
   -- type of widget (see Read_Widget above)
   -------------------------------------------------
   procedure Write_Widget(Widget : in Gui_Widget;
      File : in Ada.Text_Io.File_Type);

   -------------------------------------------------
   -- procedure Generate_Widget
   --
   -- Generate Ada/Tcl code for this widget
   -- This does only the part that creates the widget
   -- called from within Gui-Window.Generate_Window
   -- (via Widget_IO.Generate_Widgets)
   -- to create the body of the Generate_Widgets
   -- procedure
   -------------------------------------------------
   procedure Generate_Widget(Widget : in Gui_Widget;
      Window_Name : in String;
      File : in Ada.Text_Io.File_Type;
      Tcl_Only : in Boolean);

   -------------------------------------------------
   -- procedure Generate_Action
   --
   -- Generates code that will cause the Action
   -- to happen when the user interacts with the widget
   -- (e.g., when a button is pushed)
   --
   -- called from Gui-Window.Generate_Window 
   -- (via Widget_IO.Generate_Actions) to
   -- create body of Generate_Widget_Actions
   -- procedure
   --
   -- this is not abstract, but null, so that we can 
   -- call it for every widget, whether or not it has an 
   -- action.
   --
   -- It should be overridden for
   -- each widget containing action(s).
   -------------------------------------------------
   procedure Generate_Action(Widget : in Gui_Widget;
      File : in Ada.Text_Io.File_Type);

   -------------------------------------------------
   -- procedure Generate_Action_Bindings
   --
   -- As above, this is not abstract, but null, and should be 
   -- overridden for each widget containing action(s).
   --
   -- This generates the Tcl code that binds a Tcl_name
   -- to the Ada procedure the user wants done when, 
   -- for example, a button is pushed.
   -------------------------------------------------
   procedure Generate_Action_Binding(Widget : in Gui_Widget;
      File : in Ada.Text_Io.File_Type);

   -------------------------------------------------
   -- procedure Generate_Context_Clauses
   --
   -- As before, not abstract, but null, and should be overridden for
   -- each widget containing action(s).
   --
   -- Since an action is a fully-qualified Ada procedure
   -- this will create with clause(s) for appropriate
   -- packages referenced in the procedure call
   -- (right now only does 1st)
   --
   -- called from Widget_IO.Generate_Actions
   -------------------------------------------------
   procedure Generate_Context_Clause(Widget : in Gui_Widget;
      File : in Ada.Text_Io.File_Type);

   -------------------------------------------------
   -- procedure Display_Widget
   --
   -- display the widget to user's view of window
   -- being edited
   -------------------------------------------------
   procedure Display_Widget(Widget : in Gui_Widget;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   -------------------------------------------------
   -- procedure Undisplay_Widget
   --
   -- remove the widget from user's view of window
   -- being edited
   -------------------------------------------------
   procedure Undisplay_Widget(Widget : in Gui_Widget;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   -------------------------------------------------
   -- procedure Highlight
   --
   -- highlight a selected widget, that is,
   -- display the sizer buttons around it
   -- 
   -- see Subwindow_Actions.Select_widget for a usage
   -------------------------------------------------
   procedure Highlight(Widget : in Gui_Widget;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   -------------------------------------------------
   -- procedure Move_Highlight
   --
   -- if user moves the widget, move the displayed
   -- resizing buttons
   -------------------------------------------------
   procedure Move_Highlight(Widget : in Gui_Widget;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   -------------------------------------------------
   -- procedure Unhighlight
   --
   -- destroy resizing buttons (useful if user
   -- selects a different widget, for example)
   -------------------------------------------------
   procedure Unhighlight(Widget : in Gui_Widget;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   function place_coords(Widget : in Gui_Widget) return string;

   -------------------------------------------------
   -- procedure Set_Properties
   --
   -- run properties dialog with properties from widget
   -- (when user right clicks on widget)
   -- called from Subwindow_Actions.Modify_Widget
   -------------------------------------------------
   procedure Set_Properties(Widget : in out Gui_Widget);
   function Unaccess(Strp : String_Pointer) return String;
   procedure Set_Property_Entries(Widget : in out Gui_Widget;
      Pos : in out Auto_Position.Positioner);

   -------------------------------------------------
   -- procedure Apply_Properties
   --
   -- called when OK pushed in properties dialog
   -------------------------------------------------
   procedure Apply_Properties(Widget : in out Gui_Widget);

   -------------------------------------------------
   -- procedure Check_Properties
   --
   -- looks at entries in dialog, and sees if they are 
   -- ok.  If not, highlights the incorrect entry.
   -------------------------------------------------
   procedure Check_Properties(Widget : in out Gui_Widget;
      Ok : out Boolean);

   -------------------------------------------------
   -- procedure Close_Properties
   --
   -- destroys dialog.  Called after Check_Properties
   -- if user pushes "OK", or directly on "Cancel"
   -------------------------------------------------
   procedure Close_Properties(Widget : in Gui_Widget);

   -------------------------------------------------
   -- function Properties_Dialog_Name
   --
   -- return name of properties dialog
   -- 
   -- This allows us to have common code in all of
   -- the above methods, yet have different dialogs
   -- for each widget.  We use a naming convention 
   -- on the dialogs.  entry1..entry5 are reserved for
   -- Name, X, Y, Width, Height respectively.
   -- entry6... can be used by widgets further down the
   -- food chain.  Since all dialogs use this naming
   -- convention, we get a lot of reuse of checking, etc.
   -- on these properties that all widgets have
   -------------------------------------------------
   function Properties_Dialog_Name(Widget : in Gui_Widget) return String is
      abstract;

   -------------------------------------------------
   -- function Get_Name
   --
   -- simple selector that returns name stored in Widget.Name
   -- GUI_Widget should probably be private, so this
   -- would enable us to find out the name w/o knowing
   -- the representation.
   -------------------------------------------------
   function Get_Name(Widget : in Widget_Access) return String;

   -------------------------------------------------
   -- procedure Set_Location
   --
   -- set x,y,width and height for a widget
   -- would assist in making private (still too many refs
   -- in subwindow_actions to do this)
   -------------------------------------------------
   procedure Set_Location(Widget : in Widget_Access;
      X,Y,Width,Height : in Integer);

   --   package Widget_List_Package is new Lists_Generic(Widget_Access);
   --   package Widget_Key_List_Package is new Widget_List_Package.Key(
   --      GetKey => Get_Name);
   -- GNAT 3.10p doesn't correctly manage nested scope
   -- workaround by Frederic Abiven

   package Wk is new Lists_Generic_Key(Widget_Access,Get_Name);
   package Widget_List_Package renames Wk.List_Package;
   package Widget_Key_List_Package renames Wk.List_Package_Key;

   -- simple renamings to make things a bit shorter
   -- Don't confuse Widget_Pointer (a place in a widget list)
   -- with Widget_Access (a pointer to a widget)
   subtype Widget_Pointer is Widget_List_Package.Position;
   subtype Widget_List is Widget_List_Package.List;


end Gui.Widget;