---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  GUI-WINDOW.ADB 
--  Description : Root of GUI Window Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- Dmitri Anisimkov 3/11/99 Resize widget with windows functionality.
---------------------------------------------------------------
with Gui_Enum;
use type Gui_Enum.Keyword;
with Ada.Text_Io;
with Ada.Integer_Text_Io;
with Menu_Io,Widget_Io,Menu_Generate;
with Gui.Menu;
with File_Helpers;
with Tcl_Helpers;
with Tcl,Tcl.Ada;
with Ada.Unchecked_Deallocation;
use type Gui.Menu.Menu_Pointer;
with Gui.Widget;
use type Gui.Widget.Widget_Pointer;
with Tcl_Utilities;
with Common_Dialogs;
with State;

package body Gui.Window is
   -- addition by mcc on 2/22/99 to allow name to
   -- be displayed below toolbar
   Canvas_Start_Y : constant := 70;
   -- remember height of menu
   Menu_Height : Integer;

   --------------------------------------------------------------
   -- Initialize a GUI_Window
   --------------------------------------------------------------
   procedure Create_Window(Window : in out Gui_Window; Filename : in String;
         Width       : in Integer;
         Height      : in Integer;
         Title       : in String;
         Window_Name : in String) is
   begin
      Window.Filename := new String'(Filename);

      if Window_Name'Length = 0 then
         Window.Window_Name := new String'(".");
      elsif Window_Name(Window_Name'First) = '.' then
         Window.Window_Name := new String'(Window_Name);
      else
         Window.Window_Name := new String'("." & Window_Name);
      end if;

      if Title'Length = 0 then
         Window.Title := null;
      else
         Window.Title := new String'(Tcl_Utilities.Fix_Quotes(Title));
      end if;

      Window.Width := Width;
      Window.Height := Height;
      Gui.Widget.Widget_List_Package.Initialize(Window.Widget_List);
      Gui.Menu.Menu_List_Package.Initialize(Window.Menu);
   end Create_Window;

   --------------------------------------------------------------
   -- get name for display window
   --------------------------------------------------------------
   function Make_Display_Name(Window_Name : in String) return String is
   begin
      if Window_Name = "." then
         return ".display_main";
      elsif Window_Name(Window_Name'First) /= '.' then
         return ".display" & Window_Name;
      else
         return ".display" &
            Window_Name(Window_Name'First+1..Window_Name'Last);
      end if;
   end Make_Display_Name;

   --------------------------------------------------------------
   --------------------------------------------------------------
   function Make_Widget_Area_Name(Window_Name : in String) return String is
   begin
      return Make_Display_Name(Window_Name) & ".rapid_canvas";
   end Make_Widget_Area_Name;

   --------------------------------------------------------------
   --------------------------------------------------------------
   procedure Resize_Window(Interp : in Tcl.Tcl_Interp;
         Window : in out Gui_Window;
         Width  : in Integer;
         Height : in Integer) is
      Display_Name : String := Make_Display_Name(Window.Window_Name.all);
   begin
      Window.Width  := Width;
      Window.Height := Height;
      Tcl.Ada.Tcl_Eval(Interp,
         "place " & Display_Name &
         " -width " & Integer'Image(Window.Width+2) &
         " -height " & Integer'Image(Window.Height+2 + Integer(Menu_Height)));
      Tcl.Ada.Tcl_Eval(Interp,
         "place .resize" &
         " -x " & Integer'Image(Window.Width) &
         " -y " & Integer'Image(Window.Height + Integer(Menu_Height) + 60));
      Tcl.Ada.Tcl_Eval(Interp,"update");
   end Resize_Window;


   --------------------------------------------------------------
   -- reads information from file into GUI_Window
   --------------------------------------------------------------
   procedure Read_Window(Window : in out Gui_Window; Filename : in String) is
      Keyword    : Gui_Enum.Keyword;
      Word       : Word_Type;
      Last       : Natural;
      File       : Ada.Text_Io.File_Type;
      Menu_Count : Integer;
   begin -- Read_Window
      Window.Filename := new String'(Filename);

      Ada.Text_Io.Open(File => File, Name => Window.Filename.all,
         Mode => Ada.Text_Io.In_File);

      Gui_Enum.Io.Get(File,Keyword);
      if Keyword /= Gui_Enum.Window then
         raise Bad_File;
      end if;

      File_Helpers.Get_String(File,Word,Last);
      if Word(1) = '.' then
         Window.Window_Name := new String'(Word(Word'First..Last));
      else
         Window.Window_Name := new String'("." & Word(Word'First..Last));
      end if;

      -- get window title if present
      File_Helpers.Get_String(File,Word,Last);
      if Last >= Word'First then
         Window.Title := new String'(Word(Word'First..Last));
      else
         Window.Title := null;
      end if;

      Ada.Integer_Text_Io.Get(File,Window.Width);
      Ada.Integer_Text_Io.Get(File,Window.Height);

      Gui_Enum.Io.Get(File,Keyword);
      if Keyword = Gui_Enum.Menubar then
         Menu_Count := 1;
         Menu_Io.Read_Menubar(File,Window.Menu,Menu_Count);
         Gui_Enum.Io.Get(File,Keyword);
      else
         Gui.Menu.Menu_List_Package.Initialize(Window.Menu);
      end if;

      Gui.Widget.Widget_List_Package.Initialize(Window.Widget_List);
      if Keyword = Gui_Enum.Widgets then
         Widget_Io.Read_Widgets(File,
            Window.Window_Name.all, Window.Widget_List);
         Gui_Enum.Io.Get(File,Keyword);
      end if;

      -- read keyword in one of previous if statements
      if Keyword /= Gui_Enum.Endof then
         raise Bad_File;
      end if;

      Gui_Enum.Io.Get(File,Keyword);
      if Keyword /= Gui_Enum.Window then
         raise Bad_File;
      end if;

      Ada.Text_Io.Close(File => File);
   exception when Menu_Io.Bad_File =>
         raise Bad_File;
   end Read_Window;

   --------------------------------------------------------------
   -- Writes information to file from GUI_Window
   --------------------------------------------------------------
   procedure Write_Window(Window : in Gui_Window) is
      File : Ada.Text_Io.File_Type;
   begin -- Write_Window
      Ada.Text_Io.Create(File => File,
         Name => Window.Filename.all,
         Mode => Ada.Text_Io.Out_File);

      Gui_Enum.Io.Put(File,Gui_Enum.Window);
      Ada.Text_Io.Put(File," """);
      Ada.Text_Io.Put(File,Window.Window_Name.all);
      Ada.Text_Io.Put(File,"""");

      -- for backward compatibility, we keep title on
      -- same line as name.  This way, Get_String will
      -- return nothing on older .GUI files in the
      -- read.
      if Window.Title /= null then
         Ada.Text_Io.Put(File," """);
         File_Helpers.Put_String(File,Window.Title.all);
         Ada.Text_Io.Put(File,"""");
      end if;

      Ada.Text_Io.New_Line(File);

      Ada.Integer_Text_Io.Put(File,Window.Width);
      Ada.Integer_Text_Io.Put(File,Window.Height);
      Ada.Text_Io.New_Line(File => File, Spacing => 2);

      if not Gui.Menu.Menu_List_Package.Isempty(Window.Menu) then
         Menu_Io.Write_Menubar(File,Window.Menu);
         Ada.Text_Io.New_Line(File => File);
      end if;

      if not Gui.Widget.Widget_List_Package.Isempty(Window.Widget_List) then
         Widget_Io.Write_Widgets(File => File, Widgets => Window.Widget_List);
         Ada.Text_Io.New_Line(File => File);
      end if;

      Gui_Enum.Io.Put(File,Gui_Enum.Endof);
      Ada.Text_Io.Put(File,' ');
      Gui_Enum.Io.Put(File,Gui_Enum.Window);
      Ada.Text_Io.New_Line(File);

      Ada.Text_Io.Close(File => File);
   end Write_Window;

   --------------------------------------------------------------
   -- display the GUI Window
   --------------------------------------------------------------
   procedure Display_Window(Interp : Tcl.Tcl_Interp; Window : in Gui_Window) is

      Display_Name : String := Make_Display_Name(Window.Window_Name.all);
      Widget_Area_Name : String := Make_Widget_Area_Name(
         Window.Window_Name.all);

   begin
      if Window.Title /= null then
         Tcl_Utilities.Change_Label_Text(Interp => Interp,
            Window_Name => ".",
            Name        => "name",
            New_Text    => Tcl_Utilities.Unescape(Window.Title.all));
      else
         Tcl_Utilities.Change_Label_Text(Interp => Interp,
            Window_Name => ".",
            Name        => "name",
            New_Text    => Tcl_Utilities.Unescape(
               Window.Window_Name.all(Window.Window_Name.all'First+1..
                  Window.Window_Name.all'Last)));
         -- omit the "." starting the window name
      end if;
      --      Tcl.Ada.Tcl_Eval(Interp,"toplevel " &
      --         Display_Name);
      Tcl.Ada.Tcl_Eval(Interp,"frame " &
         Display_Name & " -borderwidth 1 -relief raised");
      Tcl.Ada.Tcl_Eval(Interp,"place " &
         Display_Name & " -x 0 -y " & Integer'Image(Canvas_Start_Y));
      --      Tcl.Ada.Tcl_Eval(Interp,"focus " &
      --         Display_Name);

      Tcl.Ada.Tcl_Eval(Interp, "update");

      Tcl.Ada.Tcl_Eval(Interp,
         "canvas " & Widget_Area_Name & " -width " &
         Integer'Image(Window.Width) &
         " -height " &
         Integer'Image(Window.Height));
      --      Tcl.Ada.Tcl_Eval(Interp,
      --         "place " & Display_Name & ".rapid_canvas -x 0 -y 0");

      Tcl.Ada.Tcl_Eval(Interp,
         "bind " & Widget_Area_Name & " <ButtonPress-1> {" & Ascii.Lf &
         "   set startx %x" & Ascii.Lf &
         "   set starty %y" & Ascii.Lf &
         "   select_widget mcc_not_any_widget" & Ascii.Lf &
         "   " & Widget_Area_Name & " create rectangle %x %y %x %y -tag bob}" & Ascii.Lf &
         "bind " & Widget_Area_Name & " <Button1-Motion> {" & Ascii.Lf &
         "   " & Widget_Area_Name & " coords bob $startx $starty %x %y}" & Ascii.Lf &
         "bind " & Widget_Area_Name & " <ButtonRelease-1> {" & Ascii.Lf &
         "   " & Widget_Area_Name & " delete bob" & Ascii.Lf &
         "   create_widget $startx $starty %x %y}");

      --if not Gui.Menu.Menu_List_Package.IsEmpty(Window.Menu) then
      begin
         Menu_Generate.Display_Menu_Code(Interp => Interp,
            Menubar => Window.Menu,
            Window_Name => Display_Name);
      exception
         when others => Common_Dialogs.Ok_Box("Invalid menus",Interp);
      end;
      --end if;

      Tcl.Ada.Tcl_Eval(Interp, "update");

      Tcl.Ada.Tcl_Eval(Interp,
         "pack " & Widget_Area_Name &
         " -side bottom -anchor s -fill both -expand true");

      if not Gui.Widget.Widget_List_Package.Isempty(Window.Widget_List) then
         begin
            Widget_Io.Display_Widgets(Interp => Interp,
               Window_Name => Widget_Area_Name,
               Widgets     => Window.Widget_List);
         exception
            when others =>
               Common_Dialogs.Ok_Box("Invalid widgets",Interp);
         end;
      end if;
      Tcl.Ada.Tcl_Eval(Interp,"update");


      Tcl.Ada.Tcl_Eval(Interp,"set result [winfo height " &
         Menu_Generate.Menubar_Name(Display_Name) & "]");
      Tcl_Utilities.Get_Value(Interp,"result",Menu_Height);

      Tcl.Ada.Tcl_Eval(Interp,
         "place " & Display_Name &
         " -width " & Integer'Image(Window.Width+2) &
         " -height " & Integer'Image(Window.Height+2 + Integer(Menu_Height)));
      Tcl.Ada.Tcl_Eval(Interp,"update");

      Tcl.Ada.Tcl_Eval(Interp,"frame " &
         ".resize -background black -cursor bottom_right_corner" &
         Ascii.Lf &
         "place .resize" &
         " -x " & Integer'Image(Window.Width) &
         " -y " & Integer'Image(Window.Height + Integer(Menu_Height) + 60) &
         " -width 6 -height 6" & Ascii.Lf &
         "bind .resize <Button-1> {set startx %x; set starty %y}" &
         Ascii.Lf &
         "bind .resize <Button1-Motion> " &
         "{resize_window $startx $starty %x %y}");

      declare
         Height_String : String := Integer'Image(Window.Height +
            Integer(Menu_Height) + 75);
      begin
         Tcl.Ada.Tcl_Eval(Interp,
            "wm geometry ." &
            Integer'Image(Window.Width + 10) & "x" &
            Height_String(Height_String'First+1..Height_String'Last)
            );
      end;
      Tcl.Ada.Tcl_Eval(Interp,"update");

      Tcl.Ada.Tcl_Eval(Interp,
         "wm title . ""RAPID - [file tail " &
         " {" & Window.Filename.all & "} ]""");
      --      Tcl.Ada.Tcl_Eval(Interp,
      --         "bind " & Display_Name & " <Configure> " &
      --         "{resize_subwindow_action %W %w %h}"
      --         );
      --      Tcl.Ada.Tcl_Eval(Interp,
      --         "wm title " & Display_Name &
      --         " [file tail " &
      --         " {" & Window.Filename.all & "} ]");
      --      Tcl.Ada.Tcl_Eval(Interp,
      --         "wm protocol " & Display_Name & " WM_DELETE_WINDOW " &
      --         "delete_subwindow_action");
   exception
      when others =>
         Common_Dialogs.Ok_Box("Invalid window",Interp);
   end Display_Window;

   --------------------------------------------------------------
   --------------------------------------------------------------
   procedure Resize_Window(Interp : in Tcl.Tcl_Interp;
         Window : in out Gui_Window;
         Startx : in Integer;
         Starty : in Integer;
         Endx   : in Integer;
         Endy   : in Integer) is
      use Gui.Widget;
      use Widget_List_Package;
      Pos : Position;
      Widget : Gui.Widget.Widget_Access;
   begin
      Resize_Window(Interp => Interp,
         Window => Window,
         Width  => Window.Width  + (Endx-Startx),
         Height => Window.Height + (Endy-Starty));
      if not Isempty(Window.Widget_List) then
         Pos := First(Window.Widget_List);
         loop
            Widget := Retrieve(Window.Widget_List, Pos);
            if Widget.Relx=Coord then
               Widget.X := Widget.X + (Endx-Startx);
            elsif Widget.Relx=Size then
               Widget.Width := Widget.Width + (Endx-Startx);
            end if;
            if Widget.Rely=Coord then
               Widget.Y := Widget.Y + (Endy-Starty);
            elsif Widget.Rely=Size then
               Widget.Height := Widget.Height + (Endy-Starty);
            end if;
            exit when Islast(Window.Widget_List, Pos);
            Goahead(Window.Widget_List,Pos);
         end loop;
         Widget := State.Get_Selection;
         if Widget/=null then
            Unhighlight(
--            Move_Highlight(
               Widget => Widget.all,
               Interp      => Interp,
               Window_Name => Make_Widget_Area_Name(
                  Window.Window_Name.all));
         end if;
      end if;
   end Resize_Window;


   --------------------------------------------------------------
   -- destroy the GUI Window and its resize button
   -- change title of Window
   --------------------------------------------------------------
   procedure Undisplay_Window(Interp : Tcl.Tcl_Interp; Window : in Gui_Window) is
      Display_Name : String := Make_Display_Name(Window.Window_Name.all);
   begin
      Tcl.Ada.Tcl_Eval(Interp,"destroy " &
         Display_Name & Ascii.Lf & "destroy .resize");
      -- change the following if we do MDI
      Tcl.Ada.Tcl_Eval(Interp,
         "wm title . ""RAPID""");
   end Undisplay_Window;

   --------------------------------------------------------------
   -- Generate Ada/Tcl code for this window.
   --------------------------------------------------------------
   procedure Generate_Window(Window : in Gui_Window;
         Tcl_Only : in Boolean) is
      -- Change window name as follows:
      -- 1) change "." to "main"
      -- 2) replace '.' and ' ' with '_'
      -- 3) remove all trailing '_'
      function Convert_Window_Name(Window_Name : in String) return String is
         Result : String := Window_Name;
         Start  : Integer;
      begin
         if Window_Name = "." then
            return "main";
         else
            for I in Result'range loop
               if Result(I) = '.' or else Result(I) = ' ' then
                  Result(I) := '_';
               end if;
            end loop;
         end if;

         Start := Result'First;
         while Result(Start) = '_' loop
            Start := Start + 1;
         end loop;

         return Result(Start..Result'Last);
      end Convert_Window_Name;

      -- Generate spec for the window that looks like:
      -- -- Auto generated...
      -- -- By: Martin C. Carlisle
      -- with Tcl;
      -- package filename is
      --    Interp : Tcl.Tcl_Interp; -- only if Main_Window
      --    Tcl_Error : exception; -- only if Main_Window
      --    Tk_Error  : exception; -- only if Main_Window
      --    procedure Generate_Window(Interp : Tcl.Tcl_Interp);
      -- end filename;
      procedure Generate_Spec(Filename : in String;
            Main_Window : in Boolean) is
         Spec_File : Ada.Text_Io.File_Type;
      begin
         Ada.Text_Io.Create(File => Spec_File, Mode => Ada.Text_Io.Out_File,
            Name => Filename & ".ads");
         Ada.Text_Io.Put_Line(Spec_File,"-- Auto generated by RAPID");
         Ada.Text_Io.Put_Line(Spec_File,"-- By: Martin C. Carlisle");
         Ada.Text_Io.Put_Line(Spec_File,"with Tcl;");
         Ada.Text_Io.Put_Line(Spec_File,
            "package " & Filename & " is");
         if Main_Window then
            Ada.Text_Io.Put_Line(Spec_File,
               "   Interp    : Tcl.Tcl_Interp;");
            Ada.Text_Io.Put_Line(Spec_File,
               "   Tcl_Error : exception;");
            Ada.Text_Io.Put_Line(Spec_File,
               "   Tk_Error  : exception;");
         end if;
         Ada.Text_Io.Put_Line(Spec_File,
            "   procedure Generate_Window(Interp : in Tcl.Tcl_Interp);");
         Ada.Text_Io.Put_Line(Spec_File,
            "end " & Filename & ";");
         Ada.Text_Io.Close(File => Spec_File);
      end Generate_Spec;
      Filename    : String := Convert_Window_Name(Window.Window_Name.all);
      File        : Ada.Text_Io.File_Type;
      Main_Window : Boolean := Window.Window_Name.all = ".";
   begin -- Generate_Window

      if Tcl_Only then
         Ada.Text_Io.Create(File => File, Name => Filename & ".tcl",
            Mode => Ada.Text_Io.Out_File);
      else
         -- Body looks like:
         -- -- Auto generated ...
         -- -- By: Martin C. Carlisle
         -- with Tcl;
         -- with Tcl.Ada;
         -- with Tcl.Tk;
         -- with Interfaces.C;
         -- use type Interfaces.C.Int;
         Generate_Spec(Filename,Main_Window);
         Ada.Text_Io.Create(File => File, Name => Filename & ".adb",
            Mode => Ada.Text_Io.Out_File);
         Ada.Text_Io.Put_Line(File,"-- Auto generated by RAPID");
         Ada.Text_Io.Put_Line(File,"-- By: Martin C. Carlisle");
         Ada.Text_Io.Put_Line(File,"with Tcl;");
         Ada.Text_Io.Put_Line(File,"with Tcl.Ada;");
         Ada.Text_Io.Put_Line(File,"with Tcl.Tk;");
         Ada.Text_Io.Put_Line(File,"with Interfaces.C;");
         Ada.Text_Io.Put_Line(File,"use type Interfaces.C.Int;");
         -- if there is a menu, add:
         -- with Filename_menu;
         if not Gui.Menu.Menu_List_Package.Isempty(Window.Menu) then
            Ada.Text_Io.Put_Line(File,
               "with " & Filename & "_menu;");
         end if;
         -- if there are widgets, add:
         -- with Filename_widgets;
         -- with filename_widget_actions;
         if not Gui.Widget.Widget_List_Package.Isempty(Window.Widget_List) then
            Ada.Text_Io.Put_Line(File,
               "with " & Filename & "_widgets;");
            Ada.Text_Io.Put_Line(File,
               "with " & Filename & "_widget_actions;");
         end if;
         -- add the following:
         -- package body filename is
         --   procedure Generate_Window(Interp : in Tcl.Tcl_Interp) is
         --   begin
         Ada.Text_Io.Put_Line(File,
            "package body " & Filename & " is");
         Ada.Text_Io.Put_Line(File,
            "   procedure Generate_Window(Interp : in Tcl.Tcl_Interp) is");
         Ada.Text_Io.Put_Line(File,
            "   begin");
      end if;

      -- if not main window, needs to be created as a toplevel
      -- and given focus.  Tcl automatically generates main window
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      if Window.Window_Name.all /= "." then
         Tcl_Helpers.Put_Line(File,"toplevel " &
            Window.Window_Name.all,Tcl_Only);
         Tcl_Helpers.Put_Line(File,"focus " &
            Window.Window_Name.all,Tcl_Only);
      end if;
      Tcl_Helpers.Put_Line(File, "update", Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);

      -- Generate menu code into filename_menu if there is a menu
      -- generate call here to Generate_Menu from that package
      if not Gui.Menu.Menu_List_Package.Isempty(Window.Menu) then
         Menu_Generate.Generate_Menu_Code(Menubar => Window.Menu,
            Filename    => Filename & "_menu",
            Window_Name => Window.Window_Name.all,
            Tcl_Only    => Tcl_Only);
         if not Tcl_Only then
            Ada.Text_Io.Put_Line(File,
               "      " & Filename & "_menu.Generate_Menu(Interp);");
         end if;
      end if;

      -- if there are widgets, generate them into filename_widgets
      -- add call to Generate_Widgets from that package in
      -- Generate_window
      if not Gui.Widget.Widget_List_Package.Isempty(Window.Widget_List) then
         Widget_Io.Generate_Widgets(Filename => Filename & "_widgets",
            Window_Name => Window.Window_Name.all,
            Widgets     => Window.Widget_List,
            Tcl_Only    => Tcl_Only);
         if not Tcl_Only then
            Widget_Io.Generate_Actions(Filename => Filename & "_widget_actions",
               Widgets     => Window.Widget_List);
            Ada.Text_Io.Put_Line(File,
               "      " & Filename &
               "_widget_actions.Generate_Actions(Interp);");
            Ada.Text_Io.Put_Line(File,
               "      " & Filename & "_widgets.Generate_Widgets(Interp);");
         end if;
      end if;

      -- set width and height of window
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put_Line(File, "update", Tcl_Only);
      Tcl_Helpers.Put_Line(File,
         Window.Window_Name.all & " configure -width " &
         Integer'Image(Window.Width) &
         " -height " &
         Integer'Image(Window.Height),Tcl_Only);
      Tcl_Helpers.Put_Line(File, "update", Tcl_Only);

      if Window.Title /= null then
         Tcl_Helpers.Put_Line(File, "wm title "&
            Window.Window_Name.all&" """&
            Window.Title.all&'"', Tcl_Only);
      end if;

      Tcl_Helpers.End_Block(File,Tcl_Only);

      -- end Generate_Window procedure
      if not Tcl_Only then
         Ada.Text_Io.Put_Line(File,"   end Generate_Window;");
         Ada.Text_Io.New_Line(File);
      end if;

      -- Main window needs the following to set up the interpreter
      if not Tcl_Only and then Main_Window then
         Ada.Text_Io.Put_Line(File,"begin");
         Ada.Text_Io.Put_Line(File,"   Interp := Tcl.Tcl_Createinterp;");
         Ada.Text_Io.New_Line(File);
         Ada.Text_Io.Put_Line(File,
            "   if Tcl.Tcl_Init (Interp) = Tcl.Tcl_Error then");
         Ada.Text_Io.Put_Line(File,
            "      raise Tcl_Error;");
         Ada.Text_Io.Put_Line(File,
            "   end if;");
         Ada.Text_Io.New_Line(File);
         Ada.Text_Io.Put_Line(File,
            "   if Tcl.Tk.Tk_Init (Interp) = Tcl.Tcl_Error then");
         Ada.Text_Io.Put_Line(File,
            "      raise Tk_Error;");
         Ada.Text_Io.Put_Line(File,
            "   end if;");
      end if;

      if not Tcl_Only then
         Ada.Text_Io.Put_Line(File,
            "end " & Filename & ";");
      end if;

      Ada.Text_Io.Close(File => File);
   end Generate_Window;

   --------------------------------------------------------------
   -- changes the filename for the Gui_Window
   --
   -- 1) Free the filename string
   -- 2) allocate new string
   -- 3) change title of window using filename w/o directory
   --------------------------------------------------------------
   procedure Change_Filename(
         Interp : Tcl.Tcl_Interp;
         Window : in out Gui_Window;
         Filename : in String) is
      procedure Free is new Ada.Unchecked_Deallocation(String,String_Pointer);
   begin
      Free(Window.Filename);
      Window.Filename := new String'(Filename);
      Tcl.Ada.Tcl_Eval(Interp,
         "wm title . ""RAPID - [file tail " &
         " {" & Window.Filename.all & "} ]""");
   end Change_Filename;

end Gui.Window;