---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  MENU_EDIT.ADB
--  Description : Used to visually edit the menus
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Menuedit_Window;
with Tcl.Ada;
with Gui,Gui.Menu,Gui.Window;
use type Gui.String_Pointer;
with Main;
with State;
--with Common_Dialogs; -- for debugging
with Ada.Unchecked_Deallocation;
with Submenu_Dialog,Menuitem_Dialog;
with Tcl_Utilities;
with Menu_Generate;

package body Menu_Edit is

   Indent_Width : constant := 20;
   Item_Height  : constant := 15;
   Canvas_Width : constant := 269;

   Number_Entries : Integer;
   Selected_Item  : Integer;
   Selected_Menu  : Gui.Menu.Menu_Access;
   type Item_Type is (Menu,Item);
   Am_Inserting   : Boolean := False;
   Active_Type    : Item_Type;
   -- we cannot allow two subdialogs to run simultaneously
   Subdialog_Running : Boolean := False;

   type Operation_Type is (Insert,Delete,Edit);

   subtype String_Pointer is Gui.String_Pointer;
   procedure Free is new Ada.Unchecked_Deallocation(String,String_Pointer);
   Current_Window : String_Pointer := null;

   -----------------------------------------------------------------
   -- procedure Find_Selected_Item
   -- 
   -- Look for the appropriate number in the menubar
   -- return if it is found, its position, and perform an operation
   -- (either insert, delete, or edit)
   --
   -- Walk through the list until you find the number 
   -- On finding a submenu, recursively call this procedure
   -- 
   -- If we find the number, delete it, or set the 
   -- package variable Selected_Menu (for edit),
   -- or insert something in the list.  For insert, check if
   -- highlighted is a submenu.  If so, determine where to insert
   -- based on that submenus Insert_Inside setting.
   -----------------------------------------------------------------
   procedure Find_Selected_Item(Seeking : in Integer;
         Menubar   : in out Gui.Menu.Menu_Pointer;
         Position  : out Gui.Menu.Menu_Position;
         Operation : in Operation_Type;
         Found     : out Boolean) is
      Current_Position: Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Current_Menu : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(
            Menubar,Current_Position) loop
         Current_Menu := Gui.Menu.Menu_List_Package.Retrieve(Menubar,
            Current_Position);
         if Current_Menu.Number = Seeking then
            Position := Current_Position;
            Found    := True;
            case Operation is
               when Insert =>
                  declare
                     Before_Menu : Gui.Menu.Menu_Access;
                  begin
                     Before_Menu := Gui.Menu.Menu_List_Package.Retrieve(
                        Menubar,Position);
                     if Before_Menu.all in Gui.Menu.Submenu'Class and then
                           Gui.Menu.Submenu(Before_Menu.all).Insert_Inside then
                        Gui.Menu.Menu_List_Package.AddToFront(
                           Gui.Menu.Submenu(Before_Menu.all).Items,
                           Selected_Menu);
                     else
                        Gui.Menu.Menu_List_Package.Insert(Menubar,
                           Selected_Menu,Position);
                     end if;
                  end;
               when Delete =>
                  Gui.Menu.Menu_List_Package.Delete(Menubar,Position);
               when Edit =>
                  Selected_Menu := Gui.Menu.Menu_List_Package.Retrieve(
                     Menubar,Position);
            end case;
            return;
         elsif Current_Menu.all in GUI.Menu.Submenu'Class then
            Find_Selected_Item(
               Seeking   => Seeking,
               Menubar   => Gui.Menu.Submenu(Current_Menu.all).Items,
               Operation => Operation,
               Position  => Position,
               Found     => Found);
            if Found then
               return;
            end if;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(
            Menubar,Current_Position);
      end loop;
      Found := False;
   end Find_Selected_Item;

   -----------------------------------------------------------------
   -- function Edit_Window_Name
   --
   -- returns name of Edit_Window
   -----------------------------------------------------------------
   function Edit_Window_Name(Window_Name : in String) return String is
   begin
      if Window_Name = "." then
         return ".edit_main_menu";
      else
         return ".edit_" & Window_Name(Window_Name'First+1..Window_Name'Last) &
            "_menu";
      end if;
   end Edit_Window_Name;


   -----------------------------------------------------------------
   -- display submenu (in blue) or menu item (in black)
   -- 
   -- prefix with > for submenu insert inside | for not
   -----------------------------------------------------------------
   procedure Display_Tcl_Item(
         Interp       : in Tcl.Tcl_Interp;
         Menu         : in Gui.Menu.Menu_Access;
         Window_Name  : in String;
         Indent_Level : in Integer;
         Number       : in out Integer;
         Cascade      : in Boolean) is
      Fill_Color : String := "-fill black";
      Prefix     : String := "  ";
      Tag        : String := Integer'Image(Number);
   begin
      if Cascade then
         Fill_Color := "-fill blue ";
         if Gui.Menu.Submenu(Menu.all).Insert_Inside then
            Prefix     := "> ";
         else
            Prefix     := "| ";
         end if;
      end if;

      Menu.Number       := Number;
      Menu.Indent_Level := Indent_Level;

      Tcl.Ada.Tcl_Eval(Interp,
         Window_Name & ".canvas create text " &
         Integer'Image(Indent_Level*Indent_Width) &
         Integer'Image(Menu.Number*Item_Height) &
         " -tag menu" & Tag(Tag'First+1..Tag'Last) &
         " -anchor nw -font Courier -text """ & Prefix &
         Menu.Name.all & """ " & Fill_Color);

      if Menu.Number > Number_Entries then
         Number_Entries := Menu.Number;
      end if;

      Number := Number + 1;
   end Display_Tcl_Item;

   -----------------------------------------------------------------
   -- Display menus in edit window
   -- 
   -- Traverse list calling Display_Tcl_Item
   -- recursively call this adding 1 to indent level
   -- for submenus
   -----------------------------------------------------------------
   procedure Display_Tcl_Menus(
         Interp       : in Tcl.Tcl_Interp;
         Menubar      : in Gui.Menu.Menu_Pointer;
         Indent_Level : in Integer := 1;
         Number       : in out Integer;
         Window_Name  : in String) is

      Position : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Traverse : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Position) loop
         Traverse := Gui.Menu.Menu_List_Package.Retrieve(Menubar,Position);
         if Traverse.all in Gui.Menu.Menu_Item'Class then
            Display_Tcl_Item(
               Interp       => Interp,
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Indent_Level => Indent_Level,
               Number       => Number,
               Cascade      => False);
         elsif Traverse.all in Gui.Menu.Submenu'Class then
            Display_Tcl_Item(
               Interp       => Interp,
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Indent_Level => Indent_Level,
               Number       => Number,
               Cascade      => True);
            Display_Tcl_Menus(
               Interp       => Interp,
               Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
               Indent_Level => Indent_Level + 1,
               Number       => Number,
               Window_Name  => Window_Name);
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Position);
      end loop;
   end Display_Tcl_Menus;

   -----------------------------------------------------------------
   -- pops up a window that Edits the menu
   --
   -- 1) Generate window (this window was autogenerated, but
   --    then edited, so there is no remaining .GUI file).
   --    This could be fixed if canvases were added to RAPID
   -- 2) Change canvas and add scrollbar
   -- 3) Display menus in the canvas
   -- 4) Bind left mouse to highlight (see tcl_bindings.txt)
   -- 5) Bind right mouse to edit (see tcl_bindings.txt)
   -----------------------------------------------------------------
   procedure Edit(Interp : Tcl.Tcl_Interp;
         Menubar : in Gui.Menu.Menu_Pointer;
         Window_Name : in String := ".") is
      Edit_Name : String := Edit_Window_Name(Window_Name);
      Number    : Integer;
   begin -- Edit
      State.Set_Dialog_Running(True);
      Menuedit_Window.Generate_Window(Interp,Edit_Name);
      if Current_Window /= null then
         Free(Current_Window);
      end if;

      Current_Window := new String'(Edit_Name);
      Tcl.Ada.Tcl_Eval(Interp,
         Edit_Name & ".canvas configure -scrollregion {0 0 " &
         Integer'Image(Canvas_Width) & Integer'Image(302) &
         "} -yscrollcommand """ &
         Edit_Name & ".scrollbar set""");
      Tcl.Ada.Tcl_Eval(Interp,
         Edit_Name & ".scrollbar configure -command """ &
         Edit_Name & ".canvas yview""");
      Tcl.Ada.Tcl_Eval(Interp,
         Edit_Name & ".canvas create rectangle 0 0" &
         Integer'Image(Canvas_Width) &
         Integer'Image(Item_Height) &
         " -fill LightGrey -outline LightGrey -tag highlight");

      Number_Entries := 0;
      Selected_Item  := 0;
      Number         := 1;

      Display_Tcl_Menus(
         Interp       => Interp,
         Menubar      => Menubar,
         Number       => Number,
         Window_Name  => Edit_Name);

      Tcl.Ada.Tcl_Eval(Interp,
         "bind " & Edit_Name & ".canvas <Button-1> {highlight_menu %W %x %y}");
      Tcl.Ada.Tcl_Eval(Interp,
         "bind " & Edit_Name & ".canvas <Button-3> " &
         "{highlight_menu %W %x %y; edit_menuitem}");
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol " & Edit_Name &
         " WM_DELETE_WINDOW " &
         "delete_menuwindow_action");
   end Edit;

   -----------------------------------------------------------------
   -- procedure Insert_Menu
   --
   -- run submenu dialog
   -- make transient
   -- give focus to entry 1
   -- set window close to cancel
   -----------------------------------------------------------------
   procedure Insert_Menu is
   begin
      if Subdialog_Running then
         return;
      end if;
      Am_Inserting := True;
      Active_Type  := Menu;
      Subdialog_Running := True;
      Submenu_Dialog.Generate_Window(Main.Interp);
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient .submenu_dialog " &
         Current_Window.all);
      Tcl.Ada.Tcl_Eval(Main.Interp,"focus .submenu_dialog.entry1");
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol .submenu_dialog" &
         " WM_DELETE_WINDOW " &
         "cancel_menuedit_action");
   end Insert_Menu;

   -----------------------------------------------------------------
   -- procedure Insert_Menu
   --
   -- run menuitem dialog
   -- make transient
   -- give focus to entry 1
   -- set window close to cancel
   -----------------------------------------------------------------
   procedure Insert_Choice is
   begin
      if Subdialog_Running then
         return;
      end if;
      Am_Inserting := True;
      Active_Type  := Item;
      Subdialog_Running := True;
      Menuitem_Dialog.Generate_Window(Main.Interp);
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient .menuitem_dialog " &
         Current_Window.all);
      Tcl.Ada.Tcl_Eval(Main.Interp,"focus .menuitem_dialog.entry1");
      Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol .menuitem_dialog" &
         " WM_DELETE_WINDOW " &
         "cancel_menuedit_action");
   end Insert_Choice;


   -----------------------------------------------------------------
   -- procedure Redraw_Display
   -- 
   -- delete everything from canvas and redraw them all
   -- not exactly efficient, but it works
   -----------------------------------------------------------------
   procedure Redraw_Display is
      Edit_Name   : String := Current_Window.all;
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Number      : Integer;
   begin
         Tcl.Ada.Tcl_Eval(Main.Interp,
            Edit_Name & ".canvas delete all");

         Number         := 1;

         Tcl.Ada.Tcl_Eval(Main.Interp,
            Edit_Name & ".canvas create rectangle 0 0" &
            Integer'Image(Canvas_Width) &
            Integer'Image(Item_Height) &
            " -fill LightGrey -outline LightGrey -tag highlight");

         Display_Tcl_Menus(
            Interp       => Main.Interp,
            Menubar      => Window.Menu,
            Number       => Number,
            Window_Name  => Edit_Name);
         if Selected_Item > Number_Entries then
            Selected_Item := Number_Entries;
         end if;

         Tcl.Ada.Tcl_Eval(Main.Interp,
            Edit_Name & ".canvas coords highlight 0 " &
            Integer'Image(Selected_Item*Item_Height) &
            Integer'Image(Canvas_Width) &
            Integer'Image((Selected_Item+1)*Item_Height));
   end Redraw_Display;

   -----------------------------------------------------------------
   -- procedure Delete
   --
   -- call Find_Selected_Item to do work for you
   -- Redraw display if something was found
   -----------------------------------------------------------------
   procedure Delete is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Item        : Gui.Menu.Menu_Position;
      Found       : Boolean;
   begin
      if Subdialog_Running then
         return;
      end if;

      Find_Selected_Item(Seeking => Selected_Item,
         Menubar  => Window.Menu,
         Position => Item,
         Operation=> Delete,
         Found    => Found);

      if Found then
         Redraw_Display;
      end if;

      State.Set_Changed(True);
   end Delete;

   -----------------------------------------------------------------
   -- procedure Edit_Item
   --
   -- run Find_Selected_Item
   -- then performs code similar to insert for appropriate
   -- type of menu (submenu or item), also using Common_Entries
   -- to set name, underline and action text entries.
   -----------------------------------------------------------------
   procedure Edit_Item is
      procedure Common_Entries(Window_Name : in String) is
      begin
         if Selected_Menu.Name /= null then
            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
               Window_Name => Window_Name,
               Name        => "entry1",
               Text        => Tcl_Utilities.Unescape(Selected_Menu.Name.all));
         end if;
         Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry2",
            Text        => Integer'Image(Selected_Menu.Underline_Position));
         if Selected_Menu.Action /= null then
            Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
               Window_Name => Window_Name,
               Name        => "entry3",
               Text        => Tcl_Utilities.Unescape(Selected_Menu.Action.all));
         end if;
      end Common_Entries;

      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      This_Item   : Gui.Menu.Menu_Position;
      Found       : Boolean;
   begin
      if Subdialog_Running then
         return;
      end if;
      Find_Selected_Item(Seeking => Selected_Item,
         Menubar  => Window.Menu,
         Position => This_Item,
         Operation=> Edit,
         Found    => Found);

      Am_Inserting := False;

      if Found then
         if Selected_Menu.all in Gui.Menu.Submenu'Class then
            Active_Type := Menu;
            Subdialog_Running := True;
            Submenu_Dialog.Generate_Window(Main.Interp);
            Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient .submenu_dialog " &
               Current_Window.all);
            Tcl.Ada.Tcl_Eval(Main.Interp,"focus .submenu_dialog.entry1");
            Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol .submenu_dialog" &
               " WM_DELETE_WINDOW " &
               "cancel_menuedit_action");
            Common_Entries(Window_Name => ".submenu_dialog");
         elsif Selected_Menu.all in Gui.Menu.Menu_Item'Class then
            Active_Type := Item;
            Subdialog_Running := True;
            Menuitem_Dialog.Generate_Window(Main.Interp);
            Tcl.Ada.Tcl_Eval(Main.Interp,"wm transient .menuitem_dialog " &
               Current_Window.all);
            Tcl.Ada.Tcl_Eval(Main.Interp,"focus .menuitem_dialog.entry1");
            Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol .menuitem_dialog" &
               " WM_DELETE_WINDOW " &
               "cancel_menuedit_action");
            Common_Entries(Window_Name => ".menuitem_dialog");
            if Gui.Menu.Menu_Item(Selected_Menu.all).Accelerator /= null then
               Tcl_Utilities.Set_Text_Entry(Interp => Main.Interp,
                  Window_Name => ".menuitem_dialog",
                  Name        => "entry4",
                  Text        => Gui.Menu.Menu_Item(
                     Selected_Menu.all).Accelerator.all);
            end if;
         else
            raise Constraint_Error;
         end if;
      end if;


   end Edit_Item;

   -----------------------------------------------------------------
   -- procedure Done_Insert_Dialog
   --    subprocedure Get_Common_Items reads name, underline, action
   --    from dialog
   --
   -- keeps track of whether inserting vs editing (same dialog is used)
   -----------------------------------------------------------------
   procedure Done_Insert_Dialog is
      procedure Get_Common_Items(Window_Name : in String) is
         Word : Gui.Word_Type;
         Last : Integer;
      begin
         Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry1",
            Result      => Word,
            Last        => Last);
         if Last > 0 then
            Free(Selected_Menu.Name);
            Selected_Menu.Name := new String'(
               Tcl_Utilities.Fix_Quotes(Word(1..Last)));
         else
            Free(Selected_Menu.Name);
            Selected_Menu.Name := new String'("Unnamed menu");
         end if;

         begin
         Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry2",
            Result      => Selected_Menu.Underline_Position);
         exception when others => Selected_Menu.Underline_Position := 0;
         end;

         Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
            Window_Name => Window_Name,
            Name        => "entry3",
            Result      => Word,
            Last        => Last);
         if Last > 0 then
            Free(Selected_Menu.Action);
            Selected_Menu.Action := new String'(
               Tcl_Utilities.Fix_Quotes(Word(1..Last)));
         else
            Free(Selected_Menu.Action);
            Selected_Menu.Action := new String'("null");
         end if;
      exception
         when others => null;
      end Get_Common_Items;

      Word    : Gui.Word_Type;
      Last    : Integer;
      Number  : Integer;
      Cascade : Boolean;
      Found   : Boolean;
      Window  : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Item    : Gui.Menu.Menu_Position;
      Menu_Count : Integer;
   begin
      if Am_Inserting then
         if Active_Type = Menu then
            Selected_Menu := new Gui.Menu.Submenu;
         else
            Selected_Menu := new Gui.Menu.Menu_Item;
         end if;

         if Selected_Item > 0 then
            Find_Selected_Item(Seeking => Selected_Item,
               Menubar  => Window.Menu,
               Position => Item,
               Operation=> Insert,
               Found    => Found);
         else
            Found := True;
            Gui.Menu.Menu_List_Package.AddToFront(Window.Menu,
               Selected_Menu);
         end if;

         if not Found then
            return;
         end if;

         Gui.Menu.Renumber_Menus(Window.Menu,Menu_Count);
      end if;

      if Selected_Menu.all in Gui.Menu.Submenu'Class then
         Get_Common_Items(Window_Name => ".submenu_dialog");
         Cascade := True;
         Cancel_Insert_Dialog;
      elsif Selected_Menu.all in Gui.Menu.Menu_Item'Class then
         Get_Common_Items(Window_Name => ".menuitem_dialog");
         Cascade := False;
         Tcl_Utilities.Get_Text_Entry(Interp => Main.Interp,
            Window_Name => ".menuitem_dialog",
            Name        => "entry4",
            Result      => Word,
            Last        => Last);
         if Last > 0 then
            Free(Gui.Menu.Menu_Item(Selected_Menu.all).Accelerator);
            Gui.Menu.Menu_Item(Selected_Menu.all).Accelerator :=
               new String'(Word(1..Last));
         end if;
         Cancel_Insert_Dialog;
      else
         raise Constraint_Error;
      end if;

      if not Am_Inserting then
         declare
            Tag     : String := Integer'Image(Selected_Menu.Number);
         begin
            Tcl.Ada.Tcl_Eval(Main.Interp,
               Current_Window.all & ".canvas delete menu" &
               Tag(Tag'First+1..Tag'Last));
         end;
         -- since Number is in out, use a copy
         Number := Selected_Menu.Number;
         Display_Tcl_Item(
            Interp       => Main.Interp,
            Menu         => Selected_Menu,
            Window_Name  => Current_Window.all,
            Indent_Level => Selected_Menu.Indent_Level,
            Number       => Number,
            Cascade      => Cascade);
      else
         Redraw_Display;
      end if;


      State.Set_Changed(True);
   end Done_Insert_Dialog;

   -----------------------------------------------------------------
   -- procedure Cancel_Insert_Dialog
   --
   -- destroy the dialog
   -----------------------------------------------------------------
   procedure Cancel_Insert_Dialog is
   begin
      if Active_Type = Menu then
         Tcl.Ada.Tcl_Eval(Main.Interp,"destroy .submenu_dialog");
      else
         Tcl.Ada.Tcl_Eval(Main.Interp,"destroy .menuitem_dialog");
      end if;
      Subdialog_Running := False;
   end Cancel_Insert_Dialog;

   -----------------------------------------------------------------
   -- procedure Highlight
   -- 
   -- This would be really easy-- just move the coords of highlight,
   -- but if we are clicking on a submenu in the right horizontal
   -- range, then we need to change Insert_Inside (see declare
   -- block below)
   -----------------------------------------------------------------
   procedure Highlight(Widget : in String; X,Y : in Integer) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Item        : Gui.Menu.Menu_Position;
      Found       : Boolean;
   begin
      Selected_Item := Y/Item_Height;
      if Selected_Item > Number_Entries then
         Selected_Item := Number_Entries;
      end if;

      Tcl.Ada.Tcl_Eval(Main.Interp,
         Widget & " coords highlight 0 " &
         Integer'Image(Selected_Item*Item_Height) &
         Integer'Image(Canvas_Width) &
         Integer'Image((Selected_Item+1)*Item_Height));

      Find_Selected_Item(Seeking => Selected_Item,
         Menubar  => Window.Menu,
         Position => Item,
         Operation=> Edit,
         Found    => Found);

      if Found then
         if x > Selected_Menu.Indent_Level*Indent_Width - Indent_Width/4 and then
            x < Selected_Menu.Indent_Level*Indent_Width + Indent_Width/2 and then
            Selected_Menu.all in Gui.Menu.Submenu'Class then
            declare
               Tag     : String := Integer'Image(Selected_Menu.Number);
               Number  : Integer;
            begin
               Gui.Menu.Submenu(Selected_Menu.all).Insert_Inside :=
                  not Gui.Menu.Submenu(Selected_Menu.all).Insert_Inside;
               Tcl.Ada.Tcl_Eval(Main.Interp,
                  Current_Window.all & ".canvas delete menu" &
                  Tag(Tag'First+1..Tag'Last));

               -- since Number is in out, use a copy
               Number := Selected_Menu.Number;
               Display_Tcl_Item(
                  Interp       => Main.Interp,
                  Menu         => Selected_Menu,
                  Window_Name  => Current_Window.all,
                  Indent_Level => Selected_Menu.Indent_Level,
                  Number       => Number,
                  Cascade      => True);
            end;
         end if;
      end if;
   end Highlight;

   -----------------------------------------------------------------
   -----------------------------------------------------------------
   procedure Done_Edit is
      Window       : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Display_Name : String :=
          Gui.Window.Make_Display_Name(Window.Window_Name.all);
      Width,Height : Integer;
      Was_Changed  : Boolean;
   begin
      if Subdialog_Running then
         return;
      end if;
      Width  := Window.Width;
      Height := Window.Height;
      Tcl.Ada.Tcl_Eval(Main.Interp,
         "destroy " & Current_Window.all);
      State.Set_Dialog_Running(False);
      -- Ugh!  Doing this resizes the window, so 
      -- I remember the window size above, and
      -- reinstate it after redisplaying the menu
      -- even worse, this resize changes the state to true
      Was_Changed := State.Get_Changed;
      Menu_Generate.Display_Menu_Code(Interp => Main.Interp,
         Menubar     => Window.Menu,
         Window_Name => Display_Name,
         Redisplay   => True);
      Tcl.Ada.Tcl_Eval(Main.Interp,"update");

      Tcl.Ada.Tcl_Eval(Main.Interp,
         Display_Name & " configure" &
         " -width " & integer'image(Width) &
         " -height " & integer'image(Height));
      Tcl.Ada.Tcl_Eval(Main.Interp,"update");

      State.Set_Changed(Was_Changed);
   end Done_Edit;
end Menu_Edit;