---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  MENU_GENERATE.ADB
--  Description : Generate Ada code for Menu to file
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
-- Change log:
-- 08/21/98 (mcc) : changed some Puts to Continuation_Puts to
--                  reduce generated line length
---------------------------------------------------------------
with Ada.Text_IO;
with Ada.Characters.Handling;
with Tcl,Tcl.Ada;
with Tcl_Helpers;
with Gui,Gui.Menu;
use type Gui.Menu.Menu_Pointer,Gui.String_Pointer;
with Ada.Strings.Bounded;
with Ada.Unchecked_Deallocation;
with Tcl_Utilities;
--with Common_Dialogs; -- for debugging

package body Menu_Generate is
   type String_Pointer is access all String;

   --Display_Tcl_Version : constant := Tcl.Tcl_Major_Version;
   Display_Tcl_Version : constant := 7;

   ---------------------------------------------------------------
   -- function Menubar_Name
   --
   -- just add .menubar to window name
   ---------------------------------------------------------------
   function Menubar_Name(Window_Name : in String) return String is
   begin
      if Window_Name = "." then
         return ".menubar";
      else
         return Window_Name & ".menubar";
      end if;
   end Menubar_Name;

   ---------------------------------------------------------------
   -- function Command_Prefix
   --
   -- just add menu_command at start of window name
   -- replace '.' with '_' throughout
   ---------------------------------------------------------------
   function Command_Prefix(Window_Name : in String) return String is
      result : String := "menu_command" & Menubar_Name(Window_Name);
   begin
      for i in result'range loop
         if result(i) = '.' then
            result(i) := '_';
         end if;
      end loop;

      return result;
   end Command_Prefix;

   ---------------------------------------------------------------
   -- PROCEDURE Generate_Spec
   --
   -- outputs the menu to an Ada file of the specified name
   -- with .ads added.  Do not call if Tcl_Only is set.
   --
   -- Looks like:
   -- -- Auto generated ...
   -- -- By: Martin C. Carlisle
   -- with Tcl;
   -- package Filename is
   --    procedure Generate_Menu(Interp: Tcl.Tcl_Interp);
   -- end Filename;
   ---------------------------------------------------------------
   procedure Generate_Spec(Filename : in String) is
      Spec_File : Ada.Text_Io.File_Type;
   begin
      Ada.Text_IO.Create(File => Spec_File, Mode => Ada.Text_IO.Out_File,
         Name => Filename & ".ads");
      Ada.Text_IO.Put_Line(Spec_File,"-- Auto generated by RAPID");
      Ada.Text_IO.Put_Line(Spec_File,"-- By: Martin C. Carlisle");
      Ada.Text_IO.Put_Line(Spec_File,"with Tcl;");
      Ada.Text_IO.Put_Line(Spec_File,
         "package " & Filename & " is");
      Ada.Text_IO.Put_Line(Spec_File,
         "   procedure Generate_Menu(Interp : in Tcl.Tcl_Interp);");
      Ada.Text_IO.Put_Line(Spec_File,
         "end " & Filename & ";");
      Ada.Text_IO.Close(File => Spec_File);
   end Generate_Spec;


   ---------------------------------------------------------------
   -- PROCEDURE Generate_Commands
   --
   -- creates Ada commands that will be linked to Tcl commands
   ---------------------------------------------------------------
   procedure Generate_Commands(File : in Ada.Text_IO.File_Type;
         Window_Name : in String;
         Menubar     : in Gui.Menu.Menu_Pointer) is

      procedure Generate_Command(File : in Ada.Text_IO.File_Type;
            Window_Name : in String;
            Menubar     : in Gui.Menu.Menu_Access) is
         Command_Number : constant String := Integer'Image(Menubar.Number);
      begin
         declare
            Command_Name : constant String := Command_Prefix(Window_Name) &
               Command_Number(Command_Number'First+1..Command_Number'Last);
         begin
            Tcl_Helpers.Generate_Command(File,Command_Name,
               Tcl_Utilities.Unescape(Menubar.Action.all));
         end;
      end Generate_Command;
      Position : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Traverse : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Position) loop
         Traverse := Gui.Menu.Menu_List_Package.Retrieve(
            Menubar,Position);
         if Traverse.all in Gui.Menu.Menu_Item'Class then
            Generate_Command(File,Window_Name,Traverse);
         elsif Traverse.all in Gui.Menu.Submenu'Class then
            if Traverse.all.Action /= null then
               Generate_Command(File,Window_Name,Traverse);
            end if;
            Generate_Commands(File,
               Window_Name,Gui.Menu.Submenu(Traverse.all).Items);
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Position);
      end loop;
   end Generate_Commands;

   ---------------------------------------------------------------
   -- PROCEDURE Generate_Command_Bindings
   --
   -- links Ada commands to Tcl commands
   ---------------------------------------------------------------
   procedure Generate_Command_Bindings(File : in Ada.Text_IO.File_Type;
         Window_Name : in String;
         Menubar     : in Gui.Menu.Menu_Pointer) is

      procedure Generate_Command_Binding(File : in Ada.Text_IO.File_Type;
            Window_Name : in String;
            Menubar     : in Gui.Menu.Menu_Access) is
         Command_Number : constant String := Integer'Image(Menubar.Number);
      begin
         declare
            Command_Name : constant String :=
               Command_Prefix(Window_Name) & Command_Number(
               Command_Number'First+1..Command_Number'Last);
         begin
            Tcl_Helpers.Generate_Command_Binding(File => File,
               Command_Name => Command_Name);
         end;
      end Generate_Command_Binding;
      Position : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Traverse : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Position) loop
         Traverse := Gui.Menu.Menu_List_Package.Retrieve(
            Menubar,Position);
         if Traverse.all in Gui.Menu.Menu_Item'Class then
            Generate_Command_Binding(File,Window_Name,Traverse);
         elsif Traverse.all in Gui.Menu.Submenu'Class then
            if Traverse.all.Action /= null then
               Generate_Command_Binding(File,Window_Name,Traverse);
            end if;
            Generate_Command_Bindings(File,
               Window_Name,Gui.Menu.Submenu(Traverse.all).Items);
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Position);
      end loop;
   end Generate_Command_Bindings;

   ---------------------------------------------------------------
   -- PROCEDURE Generate_Context_Clauses
   --
   -- generates with clauses from actions as needed
   ---------------------------------------------------------------
   procedure Generate_Context_Clauses(File : in Ada.Text_IO.File_Type;
         Menubar     : in Gui.Menu.Menu_Pointer) is


      Traverse : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Item     : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Traverse) loop
         Item := Gui.Menu.Menu_List_Package.Retrieve(Menubar,Traverse);
         if Item.all in Gui.Menu.Menu_Item'Class then
            Tcl_Helpers.Generate_With(File,Item.Action.all);
         elsif Item.all in Gui.Menu.Submenu'Class then
            if Item.all.Action /= null then
               Tcl_Helpers.Generate_With(File,Item.Action.all);
            end if;
            Generate_Context_Clauses(File,Gui.Menu.Submenu(Item.all).Items);
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Traverse);
      end loop;
   end Generate_Context_Clauses;

   -- Translate accelerator to Tcl format 
   function Translate_Accelerator(Accelerator : in String) return String is
      package Bounded is new Ada.Strings.Bounded.Generic_Bounded_Length(80);
      use type Bounded.Bounded_String;
      Result : Bounded.Bounded_String := Bounded.Null_Bounded_String;
      Count : Integer := Accelerator'First;
   begin
      while Count <= Accelerator'Last loop
         if Count+3 <= Accelerator'Last and then
               Accelerator(Count..Count+3) = "Ctrl" then
            Result := Result & "Control";
            Count := Count + 4;
         elsif Count+2 <= Accelerator'Last and then
               Accelerator(Count..Count+2) = "Del" then
            Result := Result & "Delete";
            Count := Count + 3;
         elsif Accelerator(Count) = '+' then
            Result := Result & "-";
            Count := Count + 1;
         else
            if Accelerator(Count) /= 'F' or else
                  Count = Accelerator'Last or else
                  not Ada.Characters.Handling.Is_Digit(Accelerator(Count+1)) then
               declare
                  Newresult : String := Accelerator(Count..Accelerator'Last);
               begin
                  for I in Newresult'range loop
                     Newresult(I) := Ada.Characters.Handling.To_Lower(Newresult(I));
                  end loop;
                  Result := Result & ("Key-" & Newresult);
               end;
            else
               Result := Result & ("Key-" & Accelerator(Count..Accelerator'Last));
            end if;
            Count := Accelerator'Last + 1;
         end if;
      end loop;
      return Bounded.To_String(Result);
   end Translate_Accelerator;

   ---------------------------------------------------------------
   -- PROCEDURE Generate_Tcl_Menus
   --
   -- outputs the menus to an Ada file or Tcl file.
   -- Assumes menubar (window_name.menubar) has already been created.
   -- Set Menu_Count to 0 before each call.
   ---------------------------------------------------------------
   procedure Generate_Tcl_Menus(File : in Ada.Text_IO.File_Type;
         Menubar      : in Gui.Menu.Menu_Pointer;
         Window_Name  : in String;
         Menubar_Name : in String;
         Tcl_Only     : in Boolean) is

      procedure Generate_Tcl_Item(File : in Ada.Text_IO.File_Type;
            Menu         : in Gui.Menu.Menu_Access;
            Window_Name  : in String;
            Menubar_Name : in String;
            Tcl_Only     : in Boolean;
            Cascade      : in Boolean) is
      begin
         Tcl_Helpers.Begin_Block(File,Tcl_Only);
         if Cascade then
            if Tcl.TCL_MAJOR_VERSION >= 8 then
               Tcl_Helpers.Put(File,Menubar_Name & " add ",Tcl_Only);
               Tcl_Helpers.Put(File,"cascade ",Tcl_Only);
               Tcl_Helpers.Put(File," -label ",Tcl_Only);
            else
               Tcl_Helpers.Put(File,"menubutton " & Menubar_Name,Tcl_Only);
               declare
                  Number : constant String := Integer'Image(Menu.Number);
               begin
                  Tcl_Helpers.Put(File,".",Tcl_Only);
                  Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last),Tcl_Only);
               end;
               Tcl_Helpers.Put(File," -text ",Tcl_Only);
            end if;
         else
            Tcl_Helpers.Put(File,Menubar_Name & " add ",Tcl_Only);
            Tcl_Helpers.Put(File,"command ",Tcl_Only);
            Tcl_Helpers.Put(File,"-label ",Tcl_Only);
         end if;
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Put(File,Menu.Name.all,Tcl_Only);
         Tcl_Helpers.Put_Quotation(File,Tcl_Only);
         Tcl_Helpers.Continuation_Put(File,
            " -underline " & Integer'Image(Menu.Underline_Position) & " ",
            Tcl_Only);
         if Menu.Action /= null then
            Tcl_Helpers.Put(File,"-command {",Tcl_Only);
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl_Helpers.Put(File,Command_Prefix(Window_Name),Tcl_Only);
               Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last),Tcl_Only);
            end;
            Tcl_Helpers.Put(File,"} ",Tcl_Only);
         end if;

         if Cascade then
            Tcl_Helpers.Continuation_Put(File,"-menu ",Tcl_Only);
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl_Helpers.Put(File,Menubar_Name & ".",Tcl_Only);
               if Tcl.TCL_MAJOR_VERSION < 8 then
                  Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last) & ".", Tcl_Only);
               end if;
               Tcl_Helpers.Put(File,"menu" & Number(Number'First+1..Number'Last),Tcl_Only);
            end;
         elsif Gui.Menu.Menu_Item_Pointer(Menu).Accelerator /= null then
            Tcl_Helpers.Continuation_Put(File,"-accelerator ",Tcl_Only);
            Tcl_Helpers.Put_Quotation(File,Tcl_Only);
            Tcl_Helpers.Put(File,Gui.Menu.Menu_Item_Pointer(Menu).Accelerator.all,Tcl_Only);
            Tcl_Helpers.Put_Quotation(File,Tcl_Only);
            Tcl_Helpers.New_Line(File,Tcl_Only);
            if Window_Name /= "." then
               Tcl_Helpers.Put(File,"bind " & Window_Name & " <",Tcl_Only);
            else
               Tcl_Helpers.Put(File,"bind all <",Tcl_Only);
            end if;
            Tcl_Helpers.Put(File,
               Translate_Accelerator(Gui.Menu.Menu_Item_Pointer(Menu).Accelerator.all),
               Tcl_Only);
            Tcl_Helpers.Put(File,"> {",Tcl_Only);
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl_Helpers.Put(File,Command_Prefix(Window_Name),Tcl_Only);
               Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last),Tcl_Only);
            end;
            Tcl_Helpers.Put(File,"} ",Tcl_Only);
         end if;
         Tcl_Helpers.New_Line(File,Tcl_Only);

         if Tcl.TCL_MAJOR_VERSION < 8 and then Cascade then
            Tcl_Helpers.Put(File, "pack " & Menubar_Name,Tcl_Only);
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl_Helpers.Put(File,".",Tcl_Only);
               Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last),Tcl_Only);
            end;
            Tcl_Helpers.Put(File, " -side top -anchor n -fill x -expand true ",Tcl_Only);
            Tcl_Helpers.New_Line(File,Tcl_Only);
         end if;

         if Cascade then
            Tcl_Helpers.Put(File,"menu ",Tcl_Only);
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl_Helpers.Put(File,Menubar_Name & ".",Tcl_Only);
               if Tcl.TCL_MAJOR_VERSION < 8 then
                  Tcl_Helpers.Put(File,Number(Number'First+1..Number'Last) & ".",Tcl_Only);
               end if;
               Tcl_Helpers.Put(File,"menu" & Number(Number'First+1..Number'Last),Tcl_Only);
            end;
            Tcl_Helpers.New_Line(File,Tcl_Only);
         end if;

         Tcl_Helpers.End_Block(File,Tcl_Only);
      end Generate_Tcl_Item;
      Position : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Traverse : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Position) loop
         Traverse := Gui.Menu.Menu_List_Package.Retrieve(
            Menubar,Position);
         if Traverse.all in Gui.Menu.Menu_Item'Class then
            Generate_Tcl_Item(File => File,
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Menubar_Name => Menubar_Name,
               Tcl_Only     => Tcl_Only,
               Cascade      => False);
         elsif Traverse.all in Gui.Menu.Submenu'Class then
            Generate_Tcl_Item(File => File,
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Menubar_Name => Menubar_Name,
               Tcl_Only     => Tcl_Only,
               Cascade      => True);
            declare
               Number : constant String := Integer'Image(Traverse.Number);
            begin
               if Tcl.TCL_MAJOR_VERSION < 8 then
                  Generate_Tcl_Menus(File => File,
                     Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
                     Window_Name  => Window_Name,
                     Menubar_Name => Menubar_Name &
                     "." &
                     Number(Number'First+1..Number'Last) &
                     ".menu" & Number(Number'First+1..Number'Last),
                     Tcl_Only     => Tcl_Only);
               else
                  Generate_Tcl_Menus(File => File,
                     Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
                     Window_Name  => Window_Name,
                     Menubar_Name => Menubar_Name & ".menu" &
                     Number(Number'First+1..Number'Last),
                     Tcl_Only     => Tcl_Only);
               end if;
            end;
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Position);
      end loop;
   end Generate_Tcl_Menus;

   ---------------------------------------------------------------
   -- PROCEDURE Generate_Body
   --
   -- outputs the menu to an Ada file of the specified name
   -- with .adb added.  If Tcl_Only is set, generates
   -- Tcl/Tk code directly without TASH binding.
   ---------------------------------------------------------------
   procedure Generate_Body(Menubar : in Gui.Menu.Menu_Pointer;
         Filename    : in String;
         Window_Name : in String := ".";
         Tcl_Only    : in Boolean) is

      Body_File : Ada.Text_Io.File_Type;
   begin
      if not Tcl_Only then
         Ada.Text_IO.Create(File => Body_File, Mode => Ada.Text_IO.Out_File,
            Name => Filename & ".adb");
         Ada.Text_IO.Put_Line(Body_File,"-- Auto generated by RAPID");
         Ada.Text_IO.Put_Line(Body_File,"-- By: Martin C. Carlisle");
         Ada.Text_IO.Put_Line(Body_File,"with Tcl;");
         Ada.Text_IO.Put_Line(Body_File,"with Tcl.Ada;");
         Ada.Text_IO.Put_Line(Body_File,"with Interfaces.C;");
         Ada.Text_IO.Put_Line(Body_File,"with Cargv;");
         Generate_Context_Clauses(Body_File,Menubar);
         Ada.Text_IO.New_Line(Body_File);
         Ada.Text_IO.Put_Line(Body_File,
            "package body " & Filename & " is");
         Ada.Text_IO.Put_Line(Body_File,
            "   package CreateCommands is new Tcl.Ada.Generic_Command (Integer);");

         Generate_Commands(Body_File,Window_Name,Menubar);

         Ada.Text_IO.Put_Line(Body_File,
            "   procedure Generate_Menu(Interp : in Tcl.Tcl_Interp) is");
         Ada.Text_IO.Put_Line(Body_File,
            "      Command : Tcl.Tcl_Command;");
         Ada.Text_IO.Put_Line(Body_File,
            "   begin");

         Generate_Command_Bindings(Body_File,Window_Name,Menubar);
      else
         Ada.Text_IO.Create(File => Body_File, Mode => Ada.Text_IO.Out_File,
            Name => Filename & ".tcl");
         Ada.Text_IO.Put_Line(Body_File,"# Auto generated by RAPID");
         Ada.Text_IO.Put_Line(Body_File,"# By: Martin C. Carlisle");
      end if;

      -- now we generate the menus themselves!
      Tcl_Helpers.Begin_Block(Body_File,Tcl_Only);
      if Tcl.TCL_MAJOR_VERSION >= 8 then
         Tcl_Helpers.Put_Line(Body_File, "menu " & Menubar_Name(Window_Name)
            & " -type menubar",Tcl_Only);
         Tcl_Helpers.Put_Line(Body_File,
            Window_Name & " configure -menu " & Menubar_Name(Window_Name),
            Tcl_Only);
      else
         Tcl_Helpers.Put_Line(Body_File, "frame " & Menubar_Name(Window_Name) &
            " -relief raised -borderwidth 1",Tcl_Only);
         Tcl_Helpers.Put_Line(Body_File, "pack " & Menubar_Name(Window_Name) &
            " -side top -fill x -expand true",Tcl_Only);
      end if;
      Tcl_Helpers.End_Block(Body_File,Tcl_Only);

      Generate_Tcl_Menus(File => Body_File,
         Menubar      => Menubar,
         Window_Name  => Window_Name,
         Menubar_Name => Menubar_Name(Window_Name),
         Tcl_Only     => Tcl_Only);

      if not Tcl_Only then
         Ada.Text_IO.Put_Line(Body_File,
            "   end Generate_Menu;");
         Ada.Text_IO.Put_Line(Body_File,
            "end " & Filename & ";");
      end if;
      Ada.Text_IO.Close(Body_File);
   end Generate_Body;
   ---------------------------------------------------------------
   -- PROCEDURE Generate_Menu_Code
   --
   -- outputs the menu to an Ada file(s) of the specified name
   -- with .ads and .adb added.  If Tcl_Only is set, generates
   -- Tcl/Tk code directly without TASH binding.
   ---------------------------------------------------------------

   procedure Generate_Menu_Code(Menubar : in Gui.Menu.Menu_Pointer;
         Filename    : in String;
         Window_Name : in String := ".";
         Tcl_Only    : in Boolean := False) is

   begin -- Generate_Menu_Code
      if not Tcl_Only then
         Generate_Spec(Filename);
      end if;

      Generate_Body(Menubar,Filename,Window_Name,Tcl_Only);
   end Generate_Menu_Code;

   procedure Display_Tcl_Menus(
         Interp       : in Tcl.Tcl_Interp;
         Menubar      : in Gui.Menu.Menu_Pointer;
         Window_Name  : in String;
         Top_Level    : in Boolean := True;
         Menubar_Name : in String) is

      procedure Display_Tcl_Item(
            Menu         : in Gui.Menu.Menu_Access;
            Window_Name  : in String;
            Menubar_Name : in String;
            Top_Level    : in Boolean;
            Cascade      : in Boolean) is
         procedure Delete_String is new
            Ada.Unchecked_Deallocation(String,String_Pointer);
         Command_String     : String_Pointer;
         Action_String      : String_Pointer;
         End_String : String_Pointer;
      begin
         if Cascade then
            if Display_Tcl_Version >= 8 or else not Top_Level then
               Command_String := new String'(Menubar_Name &
                  " add cascade -label ");
            else
               declare
                  Number : constant String := Integer'Image(Menu.Number);
               begin
                  Command_String := new String'("menubutton " &
                     Menubar_Name & "." &
                     Number(Number'First+1..Number'Last) &
                     " -text ");
               end;
            end if;
         elsif Top_Level then
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Command_String := new String'("menubutton " &
                  Menubar_Name & "." &
                  Number(Number'First+1..Number'Last) &
                  " -text ");
            end;
         else
            Command_String := new String'(Menubar_Name &
               " add command -label ");
         end if;

         -- don't display actions (not yet anyway)
         Action_String := new String'(" ");

         if Cascade then
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               if Display_Tcl_Version < 8 then
                  if Top_Level then
                     End_String := new String'("-menu " &
                        Menubar_Name & "." &
                        Number(Number'First+1..Number'Last) & "." &
                        "menu" & Number(Number'First+1..Number'Last));
                  else
                     End_String := new String'("-menu " &
                        Menubar_Name & "." &
                        "menu" & Number(Number'First+1..Number'Last));
                  end if;
               else
                  End_String := new String'("-menu " &
                     Menubar_Name & "." &
                     "menu" & Number(Number'First+1..Number'Last));
               end if;
            end;
         elsif Display_Tcl_Version < 8 and then Top_Level then
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               End_String := new String'("-menu " &
                  Menubar_Name & "." &
                  Number(Number'First+1..Number'Last) & "." &
                  "menu" & Number(Number'First+1..Number'Last));
            end;
         elsif Gui.Menu.Menu_Item_Pointer(Menu).Accelerator /= null then
            End_String := new String'("-accelerator """ &
               Gui.Menu.Menu_Item_Pointer(Menu).Accelerator.all &
               """"
               );
         else
            End_String := new String'("");
         end if;

         Tcl.Ada.Tcl_Eval(Interp,
            Command_String.all & '"' &
            Menu.Name.all & '"' &
            " -underline " & Integer'Image(Menu.Underline_Position)
            & " " &
            Action_String.all &
            End_String.all
            );

         Delete_String(Command_String);
         Delete_String(Action_String);
         Delete_String(End_String);

         if Display_Tcl_Version < 8 and then Top_Level then
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               Tcl.Ada.Tcl_Eval(Interp,"pack " & Menubar_Name &
                  "." & Number(Number'First+1..Number'Last) &
                  " -side left");
               if not Cascade then
                     Tcl.Ada.Tcl_Eval(Interp, "menu " &
                        Menubar_Name & "." &
                        Number(Number'First+1..Number'Last) & "." &
                        "menu" & Number(Number'First+1..Number'Last) &
                        " -tearoff 0");
               end if;
            end;
         end if;

         if Cascade then
            declare
               Number : constant String := Integer'Image(Menu.Number);
            begin
               if Display_Tcl_Version < 8 then
                  if Top_Level then
                     Tcl.Ada.Tcl_Eval(Interp, "menu " &
                        Menubar_Name & "." &
                        Number(Number'First+1..Number'Last) & "." &
                        "menu" & Number(Number'First+1..Number'Last));
                  else
                     Tcl.Ada.Tcl_Eval(Interp, "menu " &
                        Menubar_Name & "." &
                        "menu" & Number(Number'First+1..Number'Last));
                  end if;
               else
                  Tcl.Ada.Tcl_Eval(Interp, "menu " &
                     Menubar_Name & "." &
                     "menu" & Number(Number'First+1..Number'Last));
               end if;
            end;
         end if;

      end Display_Tcl_Item;
      Position : Gui.Menu.Menu_Position :=
         Gui.Menu.Menu_List_Package.First(Menubar);
      Traverse : Gui.Menu.Menu_Access;
   begin
      while not Gui.Menu.Menu_List_Package.IsPastEnd(Menubar,Position) loop
         Traverse := Gui.Menu.Menu_List_Package.Retrieve(Menubar,Position);
         if Traverse.all in Gui.Menu.Menu_Item'Class then
            Display_Tcl_Item(
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Menubar_Name => Menubar_Name,
               Top_Level    => Top_Level,
               Cascade      => False);
         elsif Traverse.all in Gui.Menu.Submenu'Class then
            Display_Tcl_Item(
               Menu         => Traverse,
               Window_Name  => Window_Name,
               Top_Level    => Top_Level,
               Menubar_Name => Menubar_Name,
               Cascade      => True);
            declare
               Number : constant String := Integer'Image(Traverse.Number);
            begin
               if Display_Tcl_Version < 8 then
                  if Top_Level then
                     Display_Tcl_Menus(
                        Interp       => Interp,
                        Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
                        Window_Name  => Window_Name,
                        Top_Level    => False,
                        Menubar_Name => Menubar_Name &
                           "." &
                           Number(Number'First+1..Number'Last) &
                           ".menu" & Number(Number'First+1..Number'Last));
                  else
                     Display_Tcl_Menus(
                        Interp       => Interp,
                        Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
                        Window_Name  => Window_Name,
                        Top_Level    => False,
                        Menubar_Name => Menubar_Name &
                           ".menu" & Number(Number'First+1..Number'Last));
                  end if;
               else
                  Display_Tcl_Menus(
                     Interp       => Interp,
                     Menubar      => Gui.Menu.Submenu(Traverse.all).Items,
                     Window_Name  => Window_Name,
                     Top_Level    => False,
                     Menubar_Name => Menubar_Name & ".menu" &
                     Number(Number'First+1..Number'Last));
               end if;
            end;
         else
            raise Constraint_Error;
         end if;
         Gui.Menu.Menu_List_Package.GoAhead(Menubar,Position);
      end loop;
   end Display_Tcl_Menus;

   ---------------------------------------------------------------
   -- PROCEDURE Display_Menu_Code
   --
   -- outputs the menu to a window
   ---------------------------------------------------------------
   procedure Display_Menu_Code(Interp : Tcl.Tcl_Interp;
         Menubar : in Gui.Menu.Menu_Pointer;
         Window_Name : in String := ".";
         Redisplay   : in Boolean := False) is
   begin
      if Redisplay then
         Tcl.Ada.Tcl_Eval(Interp,"destroy " & Menubar_Name(Window_Name));
      end if;

      if Display_Tcl_Version >= 8 then
         Tcl.Ada.Tcl_Eval(Interp, "menu " & Menubar_Name(Window_Name) &
            " -type menubar" & ASCII.LF &
            Window_Name & " configure -menu " & Menubar_Name(Window_Name));
      else
         Tcl.Ada.Tcl_Eval(Interp, "frame " & Menubar_Name(Window_Name) &
            " -relief raised -borderwidth 1");
         Tcl.Ada.Tcl_Eval(Interp, "pack " & Menubar_Name(Window_Name) &
            " -side top -anchor n -fill x -expand false");
         if Redisplay then
            Tcl.Ada.Tcl_Eval(Interp, "pack " & Menubar_Name(Window_Name) &
               " -before " & Window_Name & ".rapid_canvas");
         end if;
      end if;

      if not Gui.Menu.Menu_List_Package.IsEmpty(Menubar) then
         Display_Tcl_Menus(Interp => Interp,
            Menubar      => Menubar,
            Window_Name  => Window_Name,
            Menubar_Name => Menubar_Name(Window_Name));
      else
         Tcl.Ada.Tcl_Eval(Interp,"menubutton " & Menubar_Name(Window_Name) &
            ".l1 -text ""no menus"" -underline 0 -menu " &
            Menubar_Name(Window_Name) & ".l1.m1");
         Tcl.Ada.Tcl_Eval(Interp,"menu " &
            Menubar_Name(Window_Name) & ".l1.m1");
         Tcl.Ada.Tcl_Eval(Interp,"pack " & Menubar_Name(Window_Name) &
            ".l1");
      end if;

   end Display_Menu_Code;
end Menu_Generate;