---------------------------------------------------------------
--                                                           
--  TCL_HELPERS 
--  Description : Use to auto generate either Tcl or Ada (TASH) code
--                       
--  By: Martin Carlisle
--                                         
-- TCL_HELPERS is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- TCL_HELPERS is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
-- Change log:
-- 02/08/99 (jdb) : added Colors
-- 08/21/98 (mcc) : added Continuation_Put
---------------------------------------------------------------

package body Tcl_Helpers is

   -- begin a connected blob of Tcl code.

   procedure Begin_Block(File : Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean := False) is

   begin -- Begin_Block
      if not Tcl_Only then
         Ada.Text_Io.Put_Line(File,
            "      declare Tcl_Input : constant String := ");
         Ada.Text_Io.Put(File,"         ");
      end if;
   end Begin_Block;

   -- put a line inside a connected blob

   procedure Put(File : Ada.Text_Io.File_Type;
         Item : in String;
         Tcl_Only : in Boolean := False) is

      -- replace each single quote by a double quote
      function Double_Quotes(x : in String) return String is
         y : String(1..x'length*2);
         count : integer := 0;
      begin
         for i in x'range loop
            count := count + 1;
            if x(i) = '"' then
               y(count) := '"';
               count := count + 1;
               y(count) := '"';
            else
               y(count) := x(i);
            end if;
         end loop;
         return y(1..count);
      end Double_Quotes;
   begin -- Put
      if not Tcl_Only then
         Ada.Text_Io.Put(File, """");
         Ada.Text_Io.Put(File, Double_Quotes(Item));
         Ada.Text_Io.Put(File, """ & ");
      else
         Ada.Text_Io.Put(File,Item);
      end if;
   end Put;

   -- put a line inside a connected blob
   -- add a new_line to the Ada code 
   -- use if worried about line length
   procedure Continuation_Put(File : Ada.Text_IO.File_Type;
      Item     : in String;
      Tcl_Only : in Boolean := False) is
   begin
      if not Tcl_Only then
         Ada.Text_Io.New_Line(File);
         Ada.Text_Io.Put(File,"         ");
      end if;
      Put(File,Item,Tcl_Only);
   end Continuation_Put;

   -- put a new line inside a connected blob

   procedure New_Line(File : Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean := False) is

   begin -- New_Line
      if not Tcl_Only then
         Ada.Text_Io.Put(File," ASCII.LF &");
         Ada.Text_Io.New_Line(File);
         Ada.Text_Io.Put(File,"         ");
      else
         Ada.Text_Io.New_Line(File);
      end if;
   end New_Line;

   -- put a line inside a connected blob
   -- equivalent to put followed by new_line

   procedure Put_Line(File : Ada.Text_Io.File_Type;
         Item : in String;
         Tcl_Only : in Boolean := False) is

   begin -- Put_Line
      Put(File,Item,Tcl_Only);
      New_Line(File,Tcl_Only);
   end Put_Line;

   -- writes a quotation mark to the file

   procedure Put_Quotation(File : Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean := False) is

   begin -- Put_Quotation
      if not Tcl_Only then
         Ada.Text_Io.Put(File,"'""'");
         -- prints a quotation mark as a character literal
         Ada.Text_Io.Put(File," & ");
      else
         Ada.Text_Io.Put(File,'"');
      end if;
   end Put_Quotation;

   -- End a connected blob of Tcl/Tk code      

   procedure End_Block(File : Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean := False) is

   begin -- End_Block
      if not Tcl_Only then
         Ada.Text_Io.Put_Line(File,
            "         ASCII.LF;");
         Ada.Text_Io.Put_Line(File,
            "      begin");
         Ada.Text_Io.Put_Line(File,
            "         Tcl.Ada.Tcl_Eval(Interp,Tcl_Input);");
         Ada.Text_Io.Put_Line(File,
            "      end;");
      end if;
   end End_Block;


   procedure Generate_Command(File : Ada.Text_IO.File_Type;
         Command_Name     : in String;
         Action           : in String;
         Number_Arguments : in Natural := 0;
         Client_Data : in String := "") is
      Need_Semicolon : Boolean;
   function ClientData_Str return string is
   begin
      if Client_Data/="" then
        return Client_Data;
      else
         return "Integer";
      end if;
   end;
      ClientData : String := ClientData_Str;
   begin
      Ada.Text_Io.Put_Line(File,"   function " & Command_Name & " (");
      Ada.Text_IO.Put_Line(File,"      ClientData : in "&ClientData&";");
      Ada.Text_IO.Put_Line(File,"      Interp     : in Tcl.Tcl_Interp;");
      Ada.Text_IO.Put_Line(File,"      Argc       : in Interfaces.C.Int;");
      Ada.Text_IO.Put_Line(File,"      Argv       : in CArgv.Chars_Ptr_Ptr");
      Ada.Text_IO.Put_Line(File,"   ) return Interfaces.C.Int;");
      Ada.Text_IO.Put_Line(File,"   pragma Convention (C, " & Command_Name & ");");
      Ada.Text_IO.New_Line(File);
      Ada.Text_IO.Put_Line(File,"   function " & Command_Name & " (");
      Ada.Text_IO.Put_Line(File,"      ClientData : in "&ClientData&";");
      Ada.Text_IO.Put_Line(File,"      Interp     : in Tcl.Tcl_Interp;");
      Ada.Text_IO.Put_Line(File,"      Argc       : in Interfaces.C.Int;");
      Ada.Text_IO.Put_Line(File,"      Argv       : in CArgv.Chars_Ptr_Ptr");
      Ada.Text_IO.Put_Line(File,"   ) return Interfaces.C.Int is");
      Ada.Text_IO.Put_Line(File,"   begin -- " & Command_Name);

      Ada.Text_IO.Put(File,"      " & Action);

      if (Number_Arguments > 0) or (Client_Data/="") then
         Ada.Text_IO.Put_Line(File,"(");
      end if;
      if Client_Data/="" then
         Ada.Text_IO.Put(File,"         ClientData");
         if (Number_Arguments > 0) then
            Ada.Text_IO.Put_Line(File,",");
         else
            Ada.Text_IO.Put(File,")");
         end if;
      end if;
      if Number_Arguments > 0 then
         for i in 1..Number_Arguments-1 loop
            Ada.Text_IO.Put_Line(File,"         Cargv_Helpers.Argument(Argv," &
               Integer'Image(i) & "),");
         end loop;
         Ada.Text_IO.Put(File,"         Cargv_Helpers.Argument(Argv," &
            Integer'Image(Number_Arguments) & ")");
         Ada.Text_IO.Put(File,")");
      end if;

      Need_Semicolon := True;
      for I in reverse Action'range loop
         if Action(I) = ';' then
            Need_Semicolon := False;
            exit;
         elsif Action(I) /= ' ' then
            exit;
         end if;
      end loop;
      if Need_Semicolon then
         Ada.Text_IO.Put_Line(File,";");
      else
         Ada.Text_IO.New_Line(File);
      end if;

      Ada.Text_IO.Put_Line(File,"      return Tcl.TCL_OK;");
      Ada.Text_IO.Put_Line(File,"   end " & Command_Name & ";");
      Ada.Text_IO.New_Line(File);
   end Generate_Command;

   procedure Generate_Command_Binding(File : Ada.Text_IO.File_Type;
         Command_Name : in String;
         Client_Data_Need : boolean := false) is
     function Client_Data return string is
     begin
       if Client_Data_Need then
          return "ClientData";
       else
          return "0";
       end if;
     end;
   begin
      Ada.Text_IO.Put_Line(File,
         "      Command := CreateCommands.Tcl_CreateCommand (");
      if Client_Data_Need then   
         Ada.Text_IO.Put(File,
            "         Interp, Bind_Prefix&""");
      else   
         Ada.Text_IO.Put(File,
            "         Interp, """);
      end if;   
      Ada.Text_IO.Put(File,Command_Name);
      Ada.Text_IO.Put(File,""", ");
      Ada.Text_IO.Put(File,Command_Name);
      Ada.Text_IO.Put(File,"'access, "&Client_Data&", NULL);");
      Ada.Text_IO.New_Line(File);
   end Generate_Command_Binding;

   procedure Generate_With(File : in Ada.Text_IO.File_Type;
         Action     : in String) is
      Start, Last_Period, Ending : Integer;
   begin
      Start := Action'First;
      Last_Period := Start-1;
      Ending := Action'Last;
      while Start <= Ending and then Action(Start) = ' ' loop
         Start := Start + 1;
      end loop;

      Ending := Start;
      while Ending <= Action'Last and then
            Action(Ending) /= ' ' and then
            Action(Ending) /= '(' loop
         if Action(Ending) = '.' then
            Last_Period := Ending;
         end if;
         Ending := Ending + 1;
      end loop;

      if Last_Period > Start then
         Ada.Text_IO.Put_Line(File,
            "with " & Action(Start..Last_Period-1) & ";");
      end if;
   end Generate_With;

   -- return name with "." converted to "_"
   function Undot_Name(Name : in String) return String is
      Result : String := Name;
   begin
      for i in Result'range loop
         if Result(i) = '.' or else Result(i) = ' ' then
            Result(i) := '_';
         end if;
      end loop;

      return Result;
   end Undot_Name;

   -- Returns string containing Write info for colors
   function Colors(FG : in String;
                   BG : in String) return String is
   begin --Colors

      --two colors to return
      if (FG /= "default") AND (BG /= "default") then
         return(" -fg " & FG & " -bg " & BG);

      --return BG
      elsif (FG = "default") AND (BG /= "default") then
         return(" -bg " & BG);

      --return FG
      elsif (FG /= "default") AND (BG = "default") then
         return(" -fg " & FG);

      --return neither
      else
         return("");
      end if;
   end Colors;

end Tcl_Helpers;