---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  WIDGET_IO.ADB
--  Description : Read/Write Widgets to file
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Gui.Widget;
use type Gui.Widget.Widget_Pointer;
with Gui.Widget.Radio_Button;
with Gui_Enum;
use type Gui_Enum.Keyword;
with Lists_Generic, Lists_Generic.Key;
with Limits;

package body Widget_Io is

   ----------------------------------------------------------
   -- procedure Read_Widgets
   --
   -- initialize widget list
   -- read a keyword
   -- create the appropriate widget or stop on End
   -- read widget and add to list
   -- after stop, verify it said "end widgets"
   ----------------------------------------------------------
   procedure Read_Widgets(File : Ada.Text_Io.File_Type;
         Window_Name : in String;
         Widget_List : out Gui.Widget.Widget_List) is

      New_Widget : Gui.Widget.Widget_Access;
      Keyword    : Gui_Enum.Keyword;
      Params : String (1..512);
      Last : Natural;
      Ahead : Character;
      End_Of_Line : Boolean;
   begin -- Read_Widgets
      Gui.Widget.Widget_List_Package.Initialize(Widget_List);

      loop
         Gui_Enum.Io.Get(File,Keyword);

         exit when Keyword = Gui_Enum.Endof;

         New_Widget := Gui_Enum.Allocate_Widget(Keyword);
         loop
            Ada.Text_Io.Look_Ahead(File,Ahead,End_Of_Line);
            exit when End_Of_Line;
            case Ahead is
               when ' ' => Ada.Text_Io.Get(File,Ahead);
               when '-' =>
                  Ada.Text_Io.Get_Line(File,Params,Last);
                  Gui.Widget.Get_Widget_Params(New_Widget.all,
                     Params(1..Last));
                  exit;
               when others =>
                  Gui.Widget.Read_Widget(Widget => New_Widget.all,
                     File => File);
                  exit;
            end case;
         end loop;
         Gui.Widget.Widget_List_Package.Addtorear(Widget_List,
            New_Widget);
      end loop;

      Gui_Enum.Io.Get(File,Keyword);
      if Keyword /= Gui_Enum.Widgets then
         raise Bad_File;
      end if;
   exception
      when others =>
         raise Bad_File;
   end Read_Widgets;


   procedure Write_Widgets(File : Ada.Text_Io.File_Type;
         Widgets : Gui.Widget.Widget_List) is
      Traverse : Gui.Widget.Widget_Pointer :=
         Gui.Widget.Widget_List_Package.First(Widgets);
   begin -- Write_Widgets
      Gui_Enum.Io.Put(File,Gui_Enum.Widgets);
      Ada.Text_Io.New_Line(File);
      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         Gui.Widget.Write_Widget(
            Gui.Widget.Widget_List_Package.Retrieve(Widgets,Traverse).all,
            File);
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;
      Gui_Enum.Io.Put(File,Gui_Enum.Endof);
      Ada.Text_Io.Put(File,' ');
      Gui_Enum.Io.Put(File,Gui_Enum.Widgets);
      Ada.Text_Io.New_Line(File);
   end Write_Widgets;

   procedure Generate_Widgets(Filename : in String;
         Window_Name : in String;
         Widgets     : in Gui.Widget.Widget_List;
         Tcl_Only    : in Boolean) is

      type List_Pointer is access Gui.Widget.Widget_List_Package.List;
      type Double_List_Element is record
         Group : Gui.String_Pointer;
         Head : List_Pointer;
      end record;

      function Get_Group (Double : Double_List_Element) return String is
      begin
         return Double.Group.all;
      end Get_Group;

      package Double_List_Package is new Lists_Generic(Double_List_Element);
      package Double_List_Key_Package is new Double_List_Package.Key(Getkey=>Get_Group);

      The_List : Double_List_Package.List;

      procedure Create_Double_List is

         Traverse : Gui.Widget.Widget_Pointer :=
            Gui.Widget.Widget_List_Package.First(Widgets);
         Double_Position : Double_List_Package.Position := Double_List_Package.First(The_List);
         Double_Element  : Double_List_Element;
         Widget : Gui.Widget.Widget_Access;
         Group : Gui.String_Pointer;

      begin
         Double_List_Package.Initialize(The_List);
         while not Gui.Widget.Widget_List_Package.Ispastend(Widgets,Traverse) loop
            Widget := Gui.Widget.Widget_List_Package.Retrieve(Widgets,Traverse);
            if Widget.all in Gui.Widget.Radio_Button.Radio_Button then
               Group := Gui.Widget.Radio_Button.Radio_Button(Widget.all).Group;
               if Double_List_Key_Package.Ispresent(Group.all,The_List) then
                  Double_Position := Double_List_Key_Package.Find(Group.all,The_List);
                  Double_Element := Double_List_Package.Retrieve(The_List,Double_Position);
                  Gui.Widget.Widget_List_Package.Addtorear
                     (Double_Element.Head.all,Widget);
               else
                  Double_Element.Group := Group;
                  Double_Element.Head := new Gui.Widget.Widget_List_Package.List;
                  Gui.Widget.Widget_List_Package.Initialize(Double_Element.Head.all);
                  Double_List_Package.Addtorear(The_List,Double_Element);
                  Gui.Widget.Widget_List_Package.Addtorear
                     (Double_Element.Head.all,Widget);
               end if;
            end if;
            Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
         end loop;
      end Create_Double_List;

      procedure Generate_Types (File : in out Ada.Text_Io.File_Type) is


         Double_Position : Double_List_Package.Position := Double_List_Package.First(The_List);
         Double_Element  : Double_List_Element;

         Widget_List : List_Pointer;
         Widget : Gui.Widget.Widget_Access;
         Position : Gui.Widget.Widget_List_Package.Position;

      begin
         Ada.Text_Io.New_Line(File);
         while not Double_List_Package.Ispastend(The_List,Double_Position) loop
            Double_Element := Double_List_Package.Retrieve(The_List,Double_Position);
            Ada.Text_Io.New_Line(File);
            Ada.Text_Io.Put_Line(File,"   type " & Double_Element.Group.all & "_Type is (");
            Ada.Text_Io.Put(File,"      ");
            Widget_List := Double_Element.Head;
            Position := Gui.Widget.Widget_List_Package.First(Widget_List.all);
            while not Gui.Widget.Widget_List_Package.Ispastend(Widget_List.all,Position) loop
               Widget := Gui.Widget.Widget_List_Package.Retrieve(Widget_List.all,Position);
               Ada.Text_Io.Put(File,Gui.Widget.Get_Name(Widget) & ", ");
               Gui.Widget.Widget_List_Package.Goahead(Widget_List.all,Position);
            end loop;
            Ada.Text_Io.Put_Line(File,"None_Selected);");
            Double_List_Package.Goahead(The_List,Double_Position);
         end loop;
      end Generate_Types;

      procedure Generate_Headers (File : in out Ada.Text_Io.File_Type) is

         Double_Position : Double_List_Package.Position := Double_List_Package.First(The_List);
         Double_Element  : Double_List_Element;

      begin

         while not Double_List_Package.Ispastend(The_List,Double_Position) loop
            Double_Element := Double_List_Package.Retrieve(The_List,Double_Position);
            Ada.Text_Io.New_Line(File);
            Ada.Text_Io.Put_Line(File,"   function Read_Group (Interp : Tcl.Tcl_Interp) return " &
               Double_Element.Group.all & "_type;");
            Ada.Text_Io.New_Line(File);
            Ada.Text_Io.Put_Line(File,"   procedure Set_Button (Interp : Tcl.Tcl_Interp;");
            Ada.Text_Io.Put_Line(File,"                         Button : in " &
               Double_Element.Group.all & "_Type);");

            Double_List_Package.Goahead(The_List,Double_Position);
         end loop;
      end Generate_Headers;


      procedure Generate_Utilities (File : in out Ada.Text_Io.File_Type) is

         Double_Position : Double_List_Package.Position := Double_List_Package.First(The_List);
         Double_Element  : Double_List_Element;

      begin
         while not Double_List_Package.Ispastend(The_List,Double_Position) loop
            Double_Element := Double_List_Package.Retrieve(The_List,Double_Position);
            Ada.Text_Io.New_Line(File);
            Ada.Text_Io.Put_Line(File,"   function Read_Group (Interp : Tcl.Tcl_Interp) return " &
               Double_Element.Group.all & "_type is");
            Ada.Text_Io.Put_Line(File,"      Result_String : String(1.." &
               Integer'Image(Limits.Max_Name_Length) & ");");
            Ada.Text_Io.Put_Line(File,"      Last          : Natural;");
            Ada.Text_Io.New_Line(File);
            Ada.Text_Io.Put_Line(File,"   begin");
            Ada.Text_Io.Put_Line(File,"      begin");
            Ada.Text_Io.Put_Line(File,"         Tcl_Utilities.Get_Value (Interp, """ &
               Double_Element.Group.all & """, Result_String, Last);");
            Ada.Text_Io.Put_Line(File,"         return " & Double_Element.Group.all &
               "_Type'value(Result_String(1..Last));");
            Ada.Text_Io.Put_Line(File,"      exception");
            Ada.Text_Io.Put_Line(File,"         when others => return None_Selected;");
            Ada.Text_Io.Put_Line(File,"      end;");
            Ada.Text_Io.Put_Line(File,"   end Read_Group;");
            Ada.Text_Io.New_Line(File);

            Ada.Text_Io.Put_Line(File,"   procedure Set_Button (Interp : Tcl.Tcl_Interp;");
            Ada.Text_Io.Put_Line(File,"                         Button : in " &
               Double_Element.Group.all & "_type) is");
            Ada.Text_Io.Put_Line(File,"   begin");
            Ada.Text_Io.Put(File,"      Tcl.Ada.Tcl_Eval(Interp,");
            if Window_Name = "." then
               Ada.Text_Io.Put (File,""".""");
            else
               Ada.Text_Io.Put(File,"""" & Window_Name & ".""");
            end if;
            Ada.Text_Io.Put_Line(File," & Ada.Characters.Handling.To_Lower(" &
               Double_Element.Group.all & "_Type'Image(Button)) & "" select"");");
            Ada.Text_Io.Put_Line(File,"   end Set_Button;");
            Ada.Text_Io.New_Line(File);

            Double_List_Package.Goahead(The_List,Double_Position);
         end loop;
      end Generate_Utilities;




      procedure Generate_Spec(Filename : in String) is
         File : Ada.Text_Io.File_Type;
      begin
         Ada.Text_Io.Create(File => File,
            Name => Filename & ".ads",
            Mode => Ada.Text_Io.Out_File);
         Ada.Text_Io.Put_Line(File,"-- Auto generated by RAPID");
         Ada.Text_Io.Put_Line(File,"-- By: Martin C. Carlisle");
         Ada.Text_Io.Put_Line(File,"with Tcl;");
         Ada.Text_Io.Put_Line(File,"package " & Filename & " is");
         Generate_Types (File);
         Generate_Headers (File);
         Ada.Text_Io.Put_Line(File,
            "   procedure Generate_Widgets(Interp : in Tcl.Tcl_Interp);");
         Ada.Text_Io.Put_Line(File,"end " & Filename & ";");
         Ada.Text_Io.Close(File);
      end Generate_Spec;

      Traverse : Gui.Widget.Widget_Pointer :=
         Gui.Widget.Widget_List_Package.First(Widgets);
      File        : Ada.Text_Io.File_Type;
   begin -- Generate_Widgets
      if Tcl_Only then
         Ada.Text_Io.Create(File => File,
            Name => Filename & ".tcl",
            Mode => Ada.Text_Io.Out_File);
         Ada.Text_Io.Put_Line(File,"# Auto generated by RAPID");
         Ada.Text_Io.Put_Line(File,"# By: Martin C. Carlisle");
      else
         Create_Double_List;
         Generate_Spec(Filename => Filename);
         Ada.Text_Io.Create(File => File,
            Name => Filename & ".adb",
            Mode => Ada.Text_Io.Out_File);
         Ada.Text_Io.Put_Line(File,"-- Auto generated by RAPID");
         Ada.Text_Io.Put_Line(File,"-- By: Martin C. Carlisle");
         Ada.Text_Io.Put_Line(File,"with Tcl;");
         Ada.Text_Io.Put_Line(File,"with Tcl.Ada;");
         Ada.Text_Io.Put_Line(File,"with Tcl_Utilities;");
         Ada.Text_Io.Put_Line(File,"with Interfaces.C;");
         Ada.Text_Io.Put_Line(File,"with Cargv;");
         Ada.Text_Io.Put_Line(File,"with Ada.Characters.Handling;");
         Ada.Text_Io.Put_Line(File,"package body " & Filename & " is");
         Generate_Utilities (File);
         Ada.Text_Io.Put_Line(File,
            "   procedure Generate_Widgets(Interp : in Tcl.Tcl_Interp) is");
         Ada.Text_Io.Put_Line(File,
            "   begin");
      end if;


      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         if Window_Name = "." then
            Gui.Widget.Generate_Widget(
               Widget      => Gui.Widget.Widget_List_Package.Retrieve(
                  Widgets,Traverse).all,
               File        => File,
               Window_Name => Window_Name,
               Tcl_Only    => Tcl_Only);
         else
            Gui.Widget.Generate_Widget(
               Widget      => Gui.Widget.Widget_List_Package.Retrieve(
                  Widgets,Traverse).all,
               File        => File,
               Window_Name => Window_Name & ".",
               Tcl_Only    => Tcl_Only);
         end if;
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;

      if not Tcl_Only then
         Ada.Text_Io.Put_Line(File,
            "   end Generate_Widgets;");
         Ada.Text_Io.Put_Line(File,"end " & Filename & ";");
      end if;
      Ada.Text_Io.Close(File => File);
   end Generate_Widgets;

   procedure Generate_Actions(Filename : in String;
         Widgets  : in Gui.Widget.Widget_List) is
      procedure Generate_Spec(Filename : in String) is
         File : Ada.Text_Io.File_Type;
      begin
         Ada.Text_Io.Create(File => File,
            Name => Filename & ".ads",
            Mode => Ada.Text_Io.Out_File);
         Ada.Text_Io.Put_Line(File,"-- Auto generated by RAPID");
         Ada.Text_Io.Put_Line(File,"-- By: Martin C. Carlisle");
         Ada.Text_Io.Put_Line(File,"with Tcl;");
         Ada.Text_Io.Put_Line(File,"package " & Filename & " is");
         Ada.Text_Io.Put_Line(File,
            "   procedure Generate_Actions(Interp : in Tcl.Tcl_Interp);");
         Ada.Text_Io.Put_Line(File,"end " & Filename & ";");
         Ada.Text_Io.Close(File);
      end Generate_Spec;
      File : Ada.Text_Io.File_Type;
      Traverse : Gui.Widget.Widget_Pointer :=
         Gui.Widget.Widget_List_Package.First(Widgets);
   begin -- Generate_Actions
      Generate_Spec(Filename => Filename );
      Ada.Text_Io.Create(File => File,
         Name => Filename & ".adb",
         Mode => Ada.Text_Io.Out_File);
      Ada.Text_Io.Put_Line(File,"-- Auto generated by RAPID");
      Ada.Text_Io.Put_Line(File,"-- By: Martin C. Carlisle");
      Ada.Text_Io.Put_Line(File,"with Tcl;");
      Ada.Text_Io.Put_Line(File,"with Tcl.Ada;");
      Ada.Text_Io.Put_Line(File,"with Interfaces.C;");
      Ada.Text_Io.Put_Line(File,"with Cargv;");
      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         Gui.Widget.Generate_Context_Clause(
            Widget      => Gui.Widget.Widget_List_Package.Retrieve(
               Widgets,Traverse).all,
            File        => File);
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;
      Ada.Text_Io.Put_Line(File,"package body " & Filename & " is");
      Ada.Text_Io.Put_Line(File,
         "   package CreateCommands is new Tcl.Ada.Generic_Command (Integer);");

      Traverse := Gui.Widget.Widget_List_Package.First(Widgets);
      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         Gui.Widget.Generate_Action(
            Widget      => Gui.Widget.Widget_List_Package.Retrieve(
               Widgets,Traverse).all,
            File        => File);
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;

      Ada.Text_Io.Put_Line(File,
         "   procedure Generate_Actions(Interp : in Tcl.Tcl_Interp) is");
      Ada.Text_Io.Put_Line(File,
         "      Command : Tcl.Tcl_Command;");
      Ada.Text_Io.Put_Line(File,
         "   begin");
      -- Added to avoid empty executable statements section:
      -- this would happen if Window contained only radio buttons
      -- fix submitted by Maj David Nash, US Military Academy (West Point)
      Ada.Text_Io.Put_Line(File,
         "      null;");
      Traverse := Gui.Widget.Widget_List_Package.First(Widgets);
      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         Gui.Widget.Generate_Action_Binding(
            Widget      => Gui.Widget.Widget_List_Package.Retrieve(
               Widgets,Traverse).all,
            File        => File);
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;

      Ada.Text_Io.Put_Line(File,
         "   end Generate_Actions;");
      Ada.Text_Io.Put_Line(File,"end " & Filename & ";");
      Ada.Text_Io.Close(File);
   end Generate_Actions;

   procedure Display_Widgets(Interp : in Tcl.Tcl_Interp;
         Window_Name : in String;
         Widgets  : in Gui.Widget.Widget_List) is
      Traverse : Gui.Widget.Widget_Pointer :=
         Gui.Widget.Widget_List_Package.First(Widgets);
   begin
      while not Gui.Widget.Widget_List_Package.Ispastend(
            Widgets,Traverse) loop
         if Window_Name = "." then
            Gui.Widget.Display_Widget(
               Widget      => Gui.Widget.Widget_List_Package.Retrieve(
                  Widgets,Traverse).all,
               Interp        => Interp,
               Window_Name => Window_Name);
         else
            Gui.Widget.Display_Widget(
               Widget      => Gui.Widget.Widget_List_Package.Retrieve(
                  Widgets,Traverse).all,
               Interp        => Interp,
               Window_Name => Window_Name);
         end if;
         Gui.Widget.Widget_List_Package.Goahead(Widgets,Traverse);
      end loop;
   end Display_Widgets;

end Widget_Io;